package de.unihannover.orderingExperimentAnalysis;

import java.io.FileWriter;
import java.io.IOException;
import java.util.*;
import java.util.Map.Entry;
import java.util.function.Function;

import com.opencsv.CSVWriter;

import de.unihannover.orderingExperimentAnalysis.SessionData.Defect;

/*
Analyses the logs from the experiment and save data into a csv file
 */

public class AnalyzeExperimentLogs {
	
	//Change with the name of the csv to create
    private final static String csvFile = "AllDevelopers.csv";

    public static void main(String[] args) throws Exception {
        try {
            analyzeLogs(getFiles());
        } catch (final Exception e) {
            System.out.println();
            e.printStackTrace(System.out);
        }
    }

    public static String[] getFiles() {
        return new String[] {
                "./first10Developers.log",
                "./first30Developers.log",
                "./second30Developers.log"
        };
    }

    private static void analyzeLogs(String... filenames) throws IOException {
        final LogRecords log = LogRecords.load(filenames);
        final Collection<SessionData> sessions = log.getSplitByParticipant();

        System.out.println("Number of sessions: " + sessions.size());

        final Map<String, Integer> countsPerState = getCountsPerLastState(sessions);
        for (final Entry<String, Integer> e : countsPerState.entrySet()) {
            System.out.println("  last state " + e.getKey() + ": " + e.getValue());
        }

        for (final SessionData s : sessions) {
            final Collection<String> unclassified = s.getUnclassifiedRemarks();
            if (!unclassified.isEmpty()) {
                System.out.println("== unclassified remarks " + s.getId());
                for (final String r : unclassified) {
                    System.out.println(r.replace("\"", "\\\"").replace(": #", ": putDef(ret, \"#") + "\", );");
                }
            }
        }

        sessions.removeIf((SessionData s) -> !(s.isRealEffort(0) || s.isRealEffort(1) || s.isRealEffort(2)));
        System.out.println("Number of sessions with at least one review completed: " + sessions.size());
        System.out.println("Number of sessions with all needed data completed: " + sessions.stream().filter(AnalyzeExperimentLogs::neededDataComplete).count());

        System.out.println();

        final Map<String, Function<SessionData, ? extends Object>> columns = new LinkedHashMap<>();
        columns.put("pId", (SessionData s) -> "p" + s.getId());
        columns.put("shortId", SessionData::getShortId);
        columns.put("totalDuration", SessionData::getTotalDuration);
        columns.put("excludeAsDuplicate", SessionData::isDuplicateParticipation);

        columns.put("treatment", SessionData::getGuidanceApproach);

        columns.put("jedit", (SessionData s) -> s.getVariable("jedit", ""));


        /*
        Intro questions on experiment instructions
         */

        columns.put("intro1", (SessionData s) -> s.getVariable("intro1"));
        columns.put("intro2", (SessionData s) -> s.getVariable("intro2"));
        columns.put("viewWidth", SessionData::getViewWidth);
        columns.put("viewHeight", SessionData::getViewHeight);
        columns.put("screenWidth", SessionData::getScreenWidth);
        columns.put("screenHeight", SessionData::getScreenHeight);
        columns.put("browser", SessionData::getBrowser);
        columns.put("defectsFoundTutorial", (SessionData s) -> s.isRealEffort(-1) ? s.getDefectsFoundForReview(-1) : "NA");
        columns.put("netTimeForReviewWithoutIdlingR0", (SessionData s) -> s.isRealEffort(0) ? s.getNetTimeForReview(0, false) : "NA");
        columns.put("netTimeForReviewWithIdlingR0", (SessionData s) -> s.isRealEffort(0) ? s.getNetTimeForReview(0, true) : "NA");
        columns.put("grossTimeForReviewR0", (SessionData s) -> s.getGrossTimeForReview(0));
        columns.put("defectsFoundR0", (SessionData s) -> s.isRealEffort(0) ? s.getDefectsFoundForReview(0) : "NA");
        columns.put("defectsFoundAfterTenMinR0", (SessionData s) -> s.isRealEffort(0) ? s.getDefectsFoundAfter(0, 10) : "NA");
        columns.put("defectsFoundAfterTwentyMinR0", (SessionData s) -> s.isRealEffort(0) ? s.getDefectsFoundAfter(0, 20) : "NA");
        columns.put("defectsFoundAfterThirtyMinR0", (SessionData s) -> s.isRealEffort(0) ? s.getDefectsFoundAfter(0, 30) : "NA");
        columns.put("defectsTotalR0", (SessionData s) -> s.getDefectsTotalForReview(0));
        columns.put("timeForUnderstandingR0", (SessionData s) -> s.getGrossTimeForUnderstanding(0));
        columns.put("understandingCorrectR0", (SessionData s) -> s.getUnderstandingCorrect(0));
        columns.put("understandingTotalR0", (SessionData s) -> s.getUnderstandingTotal(0));
        columns.put("scrollAmountR0", (SessionData s) -> s.getScrollAmount(0));
        columns.put("pauseCountR0", (SessionData s) -> s.getPauseCount(0));
        columns.put("descriptionCountR0", (SessionData s) -> s.getDescriptionCount(0));
        columns.put("treatmentR1", (SessionData s) -> s.getTreatment(1));
        columns.put("changeR1", (SessionData s) -> s.getChange(1));
        columns.put("netTimeForReviewWithoutIdlingR1", (SessionData s) -> s.isRealEffort(1) ? s.getNetTimeForReview(1, false) : "NA");
        columns.put("netTimeForReviewWithIdlingR1", (SessionData s) -> s.isRealEffort(1) ? s.getNetTimeForReview(1, true) : "NA");
        columns.put("grossTimeForReviewR1", (SessionData s) -> s.getGrossTimeForReview(1));
        columns.put("defectsFoundR1", (SessionData s) -> s.isRealEffort(1) ? s.getDefectsFoundForReview(1) : "NA");
        columns.put("defectsFoundAfterTenMinR1", (SessionData s) -> s.isRealEffort(1) ? s.getDefectsFoundAfter(1, 10) : "NA");
        columns.put("defectsFoundAfterTwentyMinR1", (SessionData s) -> s.isRealEffort(1) ? s.getDefectsFoundAfter(1, 20) : "NA");
        columns.put("defectsFoundAfterThirtyMinR1", (SessionData s) -> s.isRealEffort(1) ? s.getDefectsFoundAfter(1, 30) : "NA");
        columns.put("defectsTotalR1", (SessionData s) -> s.isRealEffort(1) ? s.getDefectsTotalForReview(1) : "NA");
        columns.put("timeForUnderstandingR1", (SessionData s) -> s.getGrossTimeForUnderstanding(1));
        columns.put("understandingCorrectR1", (SessionData s) -> s.getUnderstandingCorrect(1));
        columns.put("understandingTotalR1", (SessionData s) -> s.getUnderstandingTotal(1));
        columns.put("scrollAmountR1", (SessionData s) -> s.getScrollAmount(1));
        columns.put("pauseCountR1", (SessionData s) -> s.getPauseCount(1));
        columns.put("descriptionCountR1", (SessionData s) -> s.getDescriptionCount(1));
        columns.put("treatmentR2", (SessionData s) -> s.getTreatment(2));
        columns.put("changeR2", (SessionData s) -> s.getChange(2));
        columns.put("netTimeForReviewWithoutIdlingR2", (SessionData s) -> s.isRealEffort(2) ? s.getNetTimeForReview(2, false) : "NA");
        columns.put("netTimeForReviewWithIdlingR2", (SessionData s) -> s.isRealEffort(2) ? s.getNetTimeForReview(2, true) : "NA");
        columns.put("grossTimeForReviewR2", (SessionData s) -> s.getGrossTimeForReview(2));
        columns.put("defectsFoundR2", (SessionData s) -> s.isRealEffort(2) ? s.getDefectsFoundForReview(2) : "NA");
        columns.put("defectsFoundAfterTenMinR2", (SessionData s) -> s.isRealEffort(2) ? s.getDefectsFoundAfter(2, 10) : "NA");
        columns.put("defectsFoundAfterTwentyMinR2", (SessionData s) -> s.isRealEffort(2) ? s.getDefectsFoundAfter(2, 20) : "NA");
        columns.put("defectsFoundAfterThirtyMinR2", (SessionData s) -> s.isRealEffort(2) ? s.getDefectsFoundAfter(2, 30) : "NA");
        columns.put("defectsTotalR2", (SessionData s) -> s.isRealEffort(2) ? s.getDefectsTotalForReview(2) : "NA");
        columns.put("timeForUnderstandingR2", (SessionData s) -> s.getGrossTimeForUnderstanding(2));
        columns.put("understandingCorrectR2", (SessionData s) -> s.getUnderstandingCorrect(2));
        columns.put("understandingTotalR2", (SessionData s) -> s.getUnderstandingTotal(2));
        columns.put("scrollAmountR2", (SessionData s) -> s.getScrollAmount(2));
        columns.put("pauseCountR2", (SessionData s) -> s.getPauseCount(2));
        columns.put("descriptionCountR2", (SessionData s) -> s.getDescriptionCount(2));
        columns.put("comparisonUnderstanding", SessionData::getComparisonUnderstanding);
        columns.put("comparisonComplicated", SessionData::getComparisonComplicated);
        columns.put("pageReloads", SessionData::determinePageReloads);
        columns.put("q_W_A", SessionData::questionCorrectWA);
        columns.put("q_W_B", SessionData::questionCorrectWB);
        columns.put("q_C1_A", SessionData::questionCorrectC1A);
        columns.put("q_C1_B", SessionData::questionCorrectC1B);
        columns.put("q_C1_C", SessionData::questionCorrectC1C);
        columns.put("q_C1_D", SessionData::questionCorrectC1D);
        columns.put("q_C2_A", SessionData::questionCorrectC2A);
        columns.put("q_C2_B", SessionData::questionCorrectC2B);
        columns.put("q_C2_C", SessionData::questionCorrectC2C);
        for (final Defect d : WU.values()) {
            columns.put("d_WU_" + d, (SessionData s) -> s.foundDefect(0, d));
        }
        for (final Defect d : P1.values()) {
            columns.put("d_D1_" + d, (SessionData s) -> s.foundDefect(s.getReviewNumberWith(Patch.P1), d));
        }
        for (final Defect d : P2.values()) {
            columns.put("d_D2_" + d, (SessionData s) -> s.foundDefect(s.getReviewNumberWith(Patch.P2), d));
        }
        for (final Defect d : WU.values()) {
            columns.put("timeUntil_WU_" + d, (SessionData s) -> s.getNetTimeUntilFound(0, d));
        }
        for (final Defect d : P1.values()) {
            columns.put("timeUntil_D1_" + d, (SessionData s) -> s.getNetTimeUntilFound(s.getReviewNumberWith(Patch.P1), d));
        }
        for (final Defect d : P2.values()) {
            columns.put("timeUntil_D2_" + d, (SessionData s) -> s.getNetTimeUntilFound(s.getReviewNumberWith(Patch.P2), d));
        }
        


        /*
         * Mental load questions (small task)
         */

        columns.put("MLDifficult_0", (SessionData s) -> s.getVariable("MLDifficult_0"));
        columns.put("MLComplicated_0", (SessionData s) -> s.getVariable("MLComplicated_0"));
        columns.put("MLChallenge_0", (SessionData s) -> s.getVariable("MLChallenge_0"));
        columns.put("MLEasy_0", (SessionData s) -> s.getVariable("MLEasy_0"));
        columns.put("MLUn_0", (SessionData s) -> s.getVariable("MLUn_0"));
        columns.put("MLSol_0", (SessionData s) -> s.getVariable("MLSol_0"));

        /*
        Mental effort questions (small task)
         */

        columns.put("MELittle_0", (SessionData s) -> s.getVariable("MELittle_0"));
        columns.put("MENotHard_0", (SessionData s) -> s.getVariable("MENotHard_0"));
        columns.put("MEHard_0", (SessionData s) -> s.getVariable("MEHard_0"));
        columns.put("MEEff_0", (SessionData s) -> s.getVariable("MEEff_0"));
        columns.put("MEFoc_0", (SessionData s) -> s.getVariable("MEFoc_0"));
        columns.put("MEBes_0", (SessionData s) -> s.getVariable("MEBes_0"));

        /*
        Flow questions (small task)
         */

        columns.put("Flow0_0", (SessionData s) -> s.getVariable("Flow0_0"));
        columns.put("Flow1_0", (SessionData s) -> s.getVariable("Flow1_0"));
        columns.put("Flow2_0", (SessionData s) -> s.getVariable("Flow2_0"));
        columns.put("Flow3_0", (SessionData s) -> s.getVariable("Flow3_0"));
        columns.put("Flow4_0", (SessionData s) -> s.getVariable("Flow4_0"));
        columns.put("Flow5_0", (SessionData s) -> s.getVariable("Flow5_0"));
        columns.put("Flow6_0", (SessionData s) -> s.getVariable("Flow6_0"));
        columns.put("Flow7_0", (SessionData s) -> s.getVariable("Flow7_0"));
        columns.put("Flow8_0", (SessionData s) -> s.getVariable("Flow8_0"));


        /*
         * Mental load questions (first large task)
         */

        columns.put("MLDifficult_1", (SessionData s) -> s.getVariable("MLDifficult_1"));
        columns.put("MLComplicated_1", (SessionData s) -> s.getVariable("MLComplicated_1"));
        columns.put("MLChallenge_1", (SessionData s) -> s.getVariable("MLChallenge_1"));
        columns.put("MLEasy_1", (SessionData s) -> s.getVariable("MLEasy_1"));
        columns.put("MLUn_1", (SessionData s) -> s.getVariable("MLUn_1"));
        columns.put("MLSol_1", (SessionData s) -> s.getVariable("MLSol_1"));

        /*
        Mental effort questions (first large task)
         */

        columns.put("MELittle_1", (SessionData s) -> s.getVariable("MELittle_1"));
        columns.put("MENotHard_1", (SessionData s) -> s.getVariable("MENotHard_1"));
        columns.put("MEHard_1", (SessionData s) -> s.getVariable("MEHard_1"));
        columns.put("MEEff_1", (SessionData s) -> s.getVariable("MEEff_1"));
        columns.put("MEFoc_1", (SessionData s) -> s.getVariable("MEFoc_1"));
        columns.put("MEBes_1", (SessionData s) -> s.getVariable("MEBes_1"));

        /*
        Flow questions (first large task)
         */

        columns.put("Flow0_1", (SessionData s) -> s.getVariable("Flow0_1"));
        columns.put("Flow1_1", (SessionData s) -> s.getVariable("Flow1_1"));
        columns.put("Flow2_1", (SessionData s) -> s.getVariable("Flow2_1"));
        columns.put("Flow3_1", (SessionData s) -> s.getVariable("Flow3_1"));
        columns.put("Flow4_1", (SessionData s) -> s.getVariable("Flow4_1"));
        columns.put("Flow5_1", (SessionData s) -> s.getVariable("Flow5_1"));
        columns.put("Flow6_1", (SessionData s) -> s.getVariable("Flow6_1"));
        columns.put("Flow7_1", (SessionData s) -> s.getVariable("Flow7_1"));
        columns.put("Flow8_1", (SessionData s) -> s.getVariable("Flow8_1"));


        /*
         * Mental load questions (second large task)
         */

        columns.put("MLDifficult_2", (SessionData s) -> s.getVariable("MLDifficult_2"));
        columns.put("MLComplicated_2", (SessionData s) -> s.getVariable("MLComplicated_2"));
        columns.put("MLChallenge_2", (SessionData s) -> s.getVariable("MLChallenge_2"));
        columns.put("MLEasy_2", (SessionData s) -> s.getVariable("MLEasy_2"));
        columns.put("MLUn_2", (SessionData s) -> s.getVariable("MLUn_2"));
        columns.put("MLSol_2", (SessionData s) -> s.getVariable("MLSol_2"));

        /*
        Mental effort questions (second large task)
         */

        columns.put("MELittle_2", (SessionData s) -> s.getVariable("MELittle_2"));
        columns.put("MENotHard_2", (SessionData s) -> s.getVariable("MENotHard_2"));
        columns.put("MEHard_2", (SessionData s) -> s.getVariable("MEHard_2"));
        columns.put("MEEff_2", (SessionData s) -> s.getVariable("MEEff_2"));
        columns.put("MEFoc_2", (SessionData s) -> s.getVariable("MEFoc_2"));
        columns.put("MEBes_2", (SessionData s) -> s.getVariable("MEBes_2"));

        /*
        Flow questions (second large task)
         */

        columns.put("Flow0_2", (SessionData s) -> s.getVariable("Flow0_2"));
        columns.put("Flow1_2", (SessionData s) -> s.getVariable("Flow1_2"));
        columns.put("Flow2_2", (SessionData s) -> s.getVariable("Flow2_2"));
        columns.put("Flow3_2", (SessionData s) -> s.getVariable("Flow3_2"));
        columns.put("Flow4_2", (SessionData s) -> s.getVariable("Flow4_2"));
        columns.put("Flow5_2", (SessionData s) -> s.getVariable("Flow5_2"));
        columns.put("Flow6_2", (SessionData s) -> s.getVariable("Flow6_2"));
        columns.put("Flow7_2", (SessionData s) -> s.getVariable("Flow7_2"));
        columns.put("Flow8_2", (SessionData s) -> s.getVariable("Flow8_2"));


        /*
        Checklist variables (review 0)
         */

        columns.put("CheckGen1_R0", (SessionData s) -> s.findChecklistElement("Gen1", 0));
        columns.put("CheckGen2_R0", (SessionData s) -> s.findChecklistElement("Gen2", 0));
        columns.put("CheckGen3_R0", (SessionData s) -> s.findChecklistElement("Gen3", 0));
        columns.put("CheckCl1_R0", (SessionData s) -> s.findChecklistElement("Cl1", 0));
        columns.put("CheckCl2_R0", (SessionData s) -> s.findChecklistElement("Cl2", 0));
        columns.put("CheckMet1_R0", (SessionData s) -> s.findChecklistElement("Met1", 0));
        columns.put("CheckMet2_R0", (SessionData s) -> s.findChecklistElement("Met2", 0));
        columns.put("CheckMet3_R0", (SessionData s) -> s.findChecklistElement("Met3", 0));
        columns.put("CheckAr1_R0", (SessionData s) -> s.findChecklistElement("Ar1", 0));
        columns.put("CheckVar1_R0", (SessionData s) -> s.findChecklistElement("Var1", 0));
        columns.put("CheckVar2_R0", (SessionData s) -> s.findChecklistElement("Var2", 0));
        columns.put("CheckIf1_R0", (SessionData s) -> s.findChecklistElement("If1", 0));
        columns.put("CheckIf2_R0", (SessionData s) -> s.findChecklistElement("If2", 0));
        columns.put("CheckLo1_R0", (SessionData s) -> s.findChecklistElement("Lo1", 0));
        columns.put("CheckLo2_R0", (SessionData s) -> s.findChecklistElement("Lo2", 0));
        columns.put("CheckRe1_R0", (SessionData s) -> s.findChecklistElement("Re1", 0));
        columns.put("CheckEx1_R0", (SessionData s) -> s.findChecklistElement("Ex1", 0));
        columns.put("CheckFC1_R0", (SessionData s) -> s.findChecklistElement("FC1", 0));

        /*
        Checklist variables (review 1)
         */

        columns.put("CheckGen1_R1", (SessionData s) -> s.findChecklistElement("Gen1", 1));
        columns.put("CheckGen2_R1", (SessionData s) -> s.findChecklistElement("Gen2", 1));
        columns.put("CheckGen3_R1", (SessionData s) -> s.findChecklistElement("Gen3", 1));
        columns.put("CheckCl1_R1", (SessionData s) -> s.findChecklistElement("Cl1", 1));
        columns.put("CheckCl2_R1", (SessionData s) -> s.findChecklistElement("Cl2", 1));
        columns.put("CheckMet1_R1", (SessionData s) -> s.findChecklistElement("Met1", 1));
        columns.put("CheckMet2_R1", (SessionData s) -> s.findChecklistElement("Met2", 1));
        columns.put("CheckMet3_R1", (SessionData s) -> s.findChecklistElement("Met3", 1));
        columns.put("CheckAr1_R1", (SessionData s) -> s.findChecklistElement("Ar1", 1));
        columns.put("CheckVar1_R1", (SessionData s) -> s.findChecklistElement("Var1", 1));
        columns.put("CheckVar2_R1", (SessionData s) -> s.findChecklistElement("Var2", 1));
        columns.put("CheckIf1_R1", (SessionData s) -> s.findChecklistElement("If1", 1));
        columns.put("CheckIf2_R1", (SessionData s) -> s.findChecklistElement("If2", 1));
        columns.put("CheckLo1_R1", (SessionData s) -> s.findChecklistElement("Lo1", 1));
        columns.put("CheckLo2_R1", (SessionData s) -> s.findChecklistElement("Lo2", 1));
        columns.put("CheckRe1_R1", (SessionData s) -> s.findChecklistElement("Re1", 1));
        columns.put("CheckEx1_R1", (SessionData s) -> s.findChecklistElement("Ex1", 1));
        columns.put("CheckFC1_R1", (SessionData s) -> s.findChecklistElement("FC1", 1));

         /*
        Checklist variables (review 2)
         */

        columns.put("CheckGen1_R2", (SessionData s) -> s.findChecklistElement("Gen1", 2));
        columns.put("CheckGen2_R2", (SessionData s) -> s.findChecklistElement("Gen2", 2));
        columns.put("CheckGen3_R2", (SessionData s) -> s.findChecklistElement("Gen3", 2));
        columns.put("CheckCl1_R2", (SessionData s) -> s.findChecklistElement("Cl1", 2));
        columns.put("CheckCl2_R2", (SessionData s) -> s.findChecklistElement("Cl2", 2));
        columns.put("CheckMet1_R2", (SessionData s) -> s.findChecklistElement("Met1", 2));
        columns.put("CheckMet2_R2", (SessionData s) -> s.findChecklistElement("Met2", 2));
        columns.put("CheckMet3_R2", (SessionData s) -> s.findChecklistElement("Met3", 2));
        columns.put("CheckAr1_R2", (SessionData s) -> s.findChecklistElement("Ar1", 2));
        columns.put("CheckVar1_R2", (SessionData s) -> s.findChecklistElement("Var1", 2));
        columns.put("CheckVar2_R2", (SessionData s) -> s.findChecklistElement("Var2", 2));
        columns.put("CheckIf1_R2", (SessionData s) -> s.findChecklistElement("If1", 2));
        columns.put("CheckIf2_R2", (SessionData s) -> s.findChecklistElement("If2", 2));
        columns.put("CheckLo1_R2", (SessionData s) -> s.findChecklistElement("Lo1", 2));
        columns.put("CheckLo2_R2", (SessionData s) -> s.findChecklistElement("Lo2", 2));
        columns.put("CheckRe1_R2", (SessionData s) -> s.findChecklistElement("Re1", 2));
        columns.put("CheckEx1_R2", (SessionData s) -> s.findChecklistElement("Ex1", 2));
        columns.put("CheckFC1_R2", (SessionData s) -> s.findChecklistElement("FC1", 2));



        columns.put("R0_Strategy_General", (SessionData s) -> s.findStrategyElement(1, 0, 0, 0));
        columns.put("R0_Strategy_Class_0_Item_1", (SessionData s) -> s.findStrategyElement(2, 0, 0, 0));
        columns.put("R0_Strategy_Class_0_Item_4", (SessionData s) -> s.findStrategyElement(5, 0, 0, 0));
        columns.put("R0_Strategy_Class_1_Item_1", (SessionData s) -> s.findStrategyElement(2, 1, 0, 0));

        columns.put("R0_Strategy_Chunk_0_Item_8", (SessionData s) -> s.findStrategyElement(9, 0, 0, 0));
        columns.put("R0_Strategy_Chunk_1_Item_6", (SessionData s) -> s.findStrategyElement(7, 0, 1, 0));
        columns.put("R0_Strategy_Chunk_1_Item_7", (SessionData s) -> s.findStrategyElement(8, 0, 1, 0));
        columns.put("R0_Strategy_Chunk_1_Item_8", (SessionData s) -> s.findStrategyElement(9, 0, 1, 0));
        columns.put("R0_Strategy_Chunk_2_Item_6", (SessionData s) -> s.findStrategyElement(7, 1, 0, 0));
        columns.put("R0_Strategy_Chunk_2_Item_7", (SessionData s) -> s.findStrategyElement(8, 1, 0, 0));
        columns.put("R0_Strategy_Chunk_2_Item_8", (SessionData s) -> s.findStrategyElement(9, 1, 0, 0));
        columns.put("R0_Strategy_Chunk_2_Item_10", (SessionData s) -> s.findStrategyElement(11, 1, 0, 0));
        columns.put("R0_Strategy_Chunk_2_Item_11", (SessionData s) -> s.findStrategyElement(12, 1, 0, 0));
        columns.put("R0_Strategy_Chunk_2_Item_12", (SessionData s) -> s.findStrategyElement(13, 1, 0, 0));

        columns.put("R0_Strategy_Final", (SessionData s) -> s.findStrategyElement(17, 2, 0, 0));

        for (StrategyElement el : strategyItemsP1) {
            columns.put("Strategy_P1".concat("_").concat(Integer.toString(el.classIndex)).concat("_").concat(Integer.toString(el.chunkIndex)).concat("_").concat(Integer.toString(el.strategyStep - 1)),
                    AnalyzeExperimentLogs::getStrategyItemP1);
        }
        countP1 = 0;

        for (StrategyElement el : strategyItemsP2) {
            columns.put("Strategy_P2".concat("_").concat(Integer.toString(el.classIndex)).concat("_").concat(Integer.toString(el.chunkIndex)).concat("_").concat(Integer.toString(el.strategyStep - 1)),
                    AnalyzeExperimentLogs::getStrategyItemP2);
        }
        countP2 = 0;


        /*
        Reviews context questions
         */
        columns.put("smallContext", SessionData::getContextAnswerSmallReview);
        columns.put("largeAContext", SessionData::getContextAnswerLargeAReview);
        columns.put("largeBContext", SessionData::getContextAnswerLargeBReview);


        /*
        SUS questions of the checklist/strategy
         */

        columns.put("checkFreq", (SessionData s) -> s.getVariable("checkFreq"));
        columns.put("checkCom", (SessionData s) -> s.getVariable("checkCom"));
        columns.put("checkEasy", (SessionData s) -> s.getVariable("checkEasy"));
        columns.put("checkSup", (SessionData s) -> s.getVariable("checkSup"));
        columns.put("checkInt", (SessionData s) -> s.getVariable("checkInt"));
        columns.put("checkInc", (SessionData s) -> s.getVariable("checkInc"));
        columns.put("checkLea", (SessionData s) -> s.getVariable("checkLea"));
        columns.put("checkCumb", (SessionData s) -> s.getVariable("checkCumb"));
        columns.put("checkCon", (SessionData s) -> s.getVariable("checkCon"));
        columns.put("checkMor", (SessionData s) -> s.getVariable("checkMor"));

        /*
        Demographics questions
         */
        columns.put("gender", (SessionData s) -> s.getVariable("gender"));
        columns.put("age", (SessionData s) -> s.getVariable("age"));
        columns.put("english", (SessionData s) -> s.getVariable("english"));
        columns.put("education", (SessionData s) -> s.getVariable("education"));
        columns.put("role", (SessionData s) -> s.getVariable("role"));
        columns.put("profDevExp", (SessionData s) -> s.getVariable("profDevExp"));
        columns.put("javaExp", (SessionData s) -> s.getVariable("javaExp"));
        columns.put("programPract", (SessionData s) -> s.getVariable("programPract"));
        columns.put("codeReviewExp", (SessionData s) -> s.getVariable("CrExp"));
        columns.put("reviewPract", (SessionData s) -> s.getVariable("reviewPract"));
        columns.put("reviewExpOften", (SessionData s) -> s.getVariable("reviewExp"));
        //columns.put("workStartHours", (SessionData s) -> toNumber(s.getVariable("workStartHours")));
        columns.put("hours_worked", (SessionData s) -> s.getVariable("hours_worked"));
        columns.put("stress", (SessionData s) -> s.getVariable("stress"));



        columns.put("prize", SessionData::isPrize);
        columns.put("controlledSetting", SessionData::isControlledSetting);
        columns.put("treatmentCombination", SessionData::getTreatmentCombination);

        columns.put("final remarks", (SessionData s) -> s.getVariable("furtherRemarks"));





        //print header
        List<String> keys = new ArrayList<String>();
        for (final String s : columns.keySet()) {
        	keys.add(s);
            System.out.print(s);
            System.out.print(";");
        }
        System.out.println();
        
        CSVWriter csvWrite = new CSVWriter(new FileWriter(csvFile));
        String[] header = new String[keys.size()];
        
        for(int i=0; i<keys.size(); i++) {
        	header[i]=keys.get(i);
        }
        
        csvWrite.writeNext(header);
        

        //print rows
        for (final SessionData s : sessions) {
        	int i = 0;
        	String[] csvRow = new String[columns.values().size()];
            for (final Function<SessionData, ? extends Object> f : columns.values()) {
            	System.out.println(keys.get(i)+ ": ");
                final Object o = f.apply(s);
                System.out.print(o == null ? "NA" : o.toString());
                csvRow[i] = o == null ? "NA" : o.toString();
                System.out.print(";\n");
                i++;
            }
            csvWrite.writeNext(csvRow);
            System.out.println();
            countP1 = 0;
            countP2 = 0;
        }
        csvWrite.close();
    }

    private static Map<String, Integer> getCountsPerLastState(Collection<SessionData> sessions) {
        final Map<String, Integer> ret = new LinkedHashMap<>();
        for (final SessionData s : sessions) {
            final String lastState = s.getLastState();
            final Integer oldCount = ret.get(lastState);
            ret.put(lastState, oldCount == null ? 1 : oldCount + 1);
        }
        return ret;
    }

    static double toNumber(String variable) {
        return Double.parseDouble(variable.replace(',', '.'));
    }

    private static boolean neededDataComplete(SessionData s) {
        return !s.isJeditIncomplete() && s.isRealEffort(0) && s.isRealEffort(1) && s.isRealEffort(2);
    }


    private static String getChangeInReviewOne(SessionData s) {
        return s.getChange(1).toString();
    }

    private static int countP1 = 0;
    //Keep in mind that the strategy step reported here is the strategy step in the experimentUI experiment.js + 1
    private static List<StrategyElement> strategyItemsP1 = new ArrayList<>(Arrays.asList(
            new StrategyElement(1, 0, 0),

            new StrategyElement(2, 0, 0),
            new StrategyElement(4, 0, 0),
            new StrategyElement(5, 0, 0),
            new StrategyElement(2, 2, 0),
            new StrategyElement(3, 2, 0),
            new StrategyElement(5, 2, 0),
            new StrategyElement(2, 3, 0),
            new StrategyElement(3, 3, 0),
            new StrategyElement(5, 3, 0),
            new StrategyElement(2, 4, 0),
            new StrategyElement(3, 4, 0),
            new StrategyElement(5, 4, 0),
            new StrategyElement(2, 5, 0),
            new StrategyElement(3, 5, 0),
            new StrategyElement(4, 5, 0),
            new StrategyElement(5, 5, 0),
            new StrategyElement(2, 6, 0),

            new StrategyElement(7, 0, 1),
            new StrategyElement(8, 0, 1),
            new StrategyElement(6, 2, 0),
            new StrategyElement(7, 2, 0),
            new StrategyElement(8, 2, 0),
            new StrategyElement(9, 2, 0),
            new StrategyElement(10, 2, 0),
            new StrategyElement(11, 2, 0),
            new StrategyElement(12, 2, 0),
            new StrategyElement(13, 2, 0),
            new StrategyElement(17, 2, 0),
            new StrategyElement(7, 3, 0),
            new StrategyElement(8, 3, 0),
            new StrategyElement(9, 3, 0),
            new StrategyElement(9, 3, 1),
            new StrategyElement(10, 3, 1),
            new StrategyElement(11, 3, 1),
            new StrategyElement(17, 3, 1),
            new StrategyElement(6, 3, 2),
            new StrategyElement(6, 4, 0),
            new StrategyElement(7, 4, 0),
            new StrategyElement(8, 4, 0),
            new StrategyElement(9, 4, 0),
            new StrategyElement(10, 4, 0),
            new StrategyElement(11, 4, 0),
            new StrategyElement(12, 4, 0),
            new StrategyElement(13, 4, 0),
            new StrategyElement(17, 4, 0),
            new StrategyElement(6, 5, 0),
            new StrategyElement(7, 5, 0),
            new StrategyElement(8, 5, 0),
            new StrategyElement(9, 5, 0),
            new StrategyElement(10, 5, 0),
            new StrategyElement(11, 5, 0),
            new StrategyElement(12, 5, 0),
            new StrategyElement(13, 5, 0),
            new StrategyElement(17, 5, 0),
            new StrategyElement(7, 6, 0),
            new StrategyElement(8, 6, 0),
            new StrategyElement(9, 6, 0),
            new StrategyElement(10, 6, 0),
            new StrategyElement(11, 6, 0),
            new StrategyElement(12, 6, 0),
            new StrategyElement(13, 6, 0),
            new StrategyElement(14, 6, 0),
            new StrategyElement(17, 6, 0),
            new StrategyElement(8, 6, 1),
            new StrategyElement(9, 6, 1),
            new StrategyElement(10, 6, 1),
            new StrategyElement(11, 6, 1),
            new StrategyElement(8, 6, 2),
            new StrategyElement(9, 6, 2),
            new StrategyElement(10, 6, 2),
            new StrategyElement(11, 6, 2),
            new StrategyElement(9, 6, 3),
            new StrategyElement(10, 6, 3),
            new StrategyElement(11, 6, 3),

            new StrategyElement(17, 6, 0)
        )
    );

    private static int countP2 = 0;
    //Keep in mind that the strategy step reported here is the strategy step in the experimentUI experiment.js + 1
    private static List<StrategyElement> strategyItemsP2 = new ArrayList<>(Arrays.asList(
            new StrategyElement(1, 0, 0),

            new StrategyElement(2, 0, 0),
            new StrategyElement(4, 0, 0),
            new StrategyElement(5, 0, 0),
            new StrategyElement(2, 1, 0),
            new StrategyElement(4, 1, 0),
            new StrategyElement(5, 1, 0),
            new StrategyElement(2, 2, 0),
            new StrategyElement(2, 3, 0),
            new StrategyElement(4, 3, 0),
            new StrategyElement(5, 3, 0),
            new StrategyElement(2, 4, 0),
            new StrategyElement(2, 5, 0),
            new StrategyElement(2, 6, 0),
            new StrategyElement(3, 6, 0),
            new StrategyElement(4, 6, 0),
            new StrategyElement(5, 6, 0),

            new StrategyElement(7, 0, 0),
            new StrategyElement(8, 0, 0),
            new StrategyElement(9, 0, 0),
            new StrategyElement(7, 0, 1),
            new StrategyElement(8, 0, 1),
            new StrategyElement(10, 0, 1),
            new StrategyElement(6, 0, 2),
            new StrategyElement(7, 0, 2),
            new StrategyElement(8, 0, 2),
            new StrategyElement(9, 0, 2),
            new StrategyElement(10, 0, 2),
            new StrategyElement(11, 0, 2),
            new StrategyElement(12, 0, 2),
            new StrategyElement(13, 0, 2),
            new StrategyElement(16, 0, 2),
            new StrategyElement(9, 1, 1),
            new StrategyElement(7, 2, 0),
            new StrategyElement(9, 2, 0),
            new StrategyElement(10, 2, 0),
            new StrategyElement(11, 2, 0),
            new StrategyElement(7, 3, 0),
            new StrategyElement(8, 3, 0),
            new StrategyElement(9, 3, 0),
            new StrategyElement(7, 3, 1),
            new StrategyElement(8, 3, 1),
            new StrategyElement(9, 3, 1),
            new StrategyElement(7, 3, 2),
            new StrategyElement(9, 3, 2),
            new StrategyElement(12, 3, 2),
            new StrategyElement(13, 3, 2),
            new StrategyElement(6, 4, 0),
            new StrategyElement(9, 4, 0),
            new StrategyElement(10, 4, 0),
            new StrategyElement(12, 4, 0),
            new StrategyElement(13, 4, 0),
            new StrategyElement(7, 4, 1),
            new StrategyElement(9, 4, 1),
            new StrategyElement(10, 4, 1),
            new StrategyElement(11, 4, 1),
            new StrategyElement(7, 4, 2),
            new StrategyElement(9, 4, 2),
            new StrategyElement(10, 4, 2),
            new StrategyElement(11, 4, 2),
            new StrategyElement(7, 6, 0),
            new StrategyElement(8, 6, 0),
            new StrategyElement(12, 6, 0),
            new StrategyElement(13, 6, 0),
            new StrategyElement(6, 6, 1),
            new StrategyElement(6, 6, 2),
            new StrategyElement(7, 6, 2),
            new StrategyElement(8, 6, 2),
            new StrategyElement(9, 6, 2),
            new StrategyElement(10, 6, 2),
            new StrategyElement(11, 6, 2),
            new StrategyElement(12, 6, 2),
            new StrategyElement(13, 6, 2),
            new StrategyElement(6, 6, 3),
            new StrategyElement(7, 6, 3),
            new StrategyElement(8, 6, 3),
            new StrategyElement(9, 6, 3),
            new StrategyElement(10, 6, 3),
            new StrategyElement(11, 6, 3),
            new StrategyElement(12, 6, 3),
            new StrategyElement(13, 6, 3),
            new StrategyElement(6, 6, 4)
        )
    );

    private static String getStrategyItemP1(SessionData s) {
        if(s.getChange(1).equals(Patch.P1)) {
            StrategyElement element = strategyItemsP1.get(countP1);
            countP1++;
            return s.findStrategyElement(element.strategyStep, element.classIndex, element.chunkIndex, 1);
        }
        else {
            if(s.getChange(2).equals(Patch.P1)) {
                StrategyElement element = strategyItemsP1.get(countP1);
                countP1++;
                return s.findStrategyElement(element.strategyStep, element.classIndex, element.chunkIndex, 2);
            }
        }
        return "Invalid";
    }

    private static String getStrategyItemP2(SessionData s) {
        if(s.getChange(1).equals(Patch.P2)) {
            StrategyElement element = strategyItemsP2.get(countP2);
            countP2++;
            return s.findStrategyElement(element.strategyStep, element.classIndex, element.chunkIndex, 1);
        }
        else {
            if(s.getChange(2).equals(Patch.P2)) {
                StrategyElement element = strategyItemsP2.get(countP2);
                countP2++;
                return s.findStrategyElement(element.strategyStep, element.classIndex, element.chunkIndex, 2);
            }
        }
        return "Invalid";
    }

    public static class StrategyElement{
        public int strategyStep;
        public int classIndex;
        public int chunkIndex;

        public StrategyElement(int strategyStep, int classIndex, int chunkIndex){
            this.strategyStep = strategyStep;
            this.classIndex = classIndex;
            this.chunkIndex = chunkIndex;
        }
    }

}
