package de.unihannover.orderingExperimentAnalysis;

import static de.unihannover.orderingExperimentAnalysis.SessionData.Defect.putDef;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import de.unihannover.orderingExperimentAnalysis.SessionData.Defect;
import de.unihannover.orderingExperimentAnalysis.SessionData.FP;

public enum P1 implements Defect {

    /**
     * Initialize path2 wrongly in RenameBrowserTask constructor (hunk j).
     */
    WRONG_PATH_ASSIGNMENT,
    /**
     * Swap from and to in call RenameBrowserTask constructor in VFSBrowser.rename(from) (hunk b).
     */
    FROM_TO_SWITCHED_RENAME1,
    /**
     * Swap from and to in call RenameBrowserTask constructor in VFSBrowser.rename(from,to) (hunk c).
     */
    FROM_TO_SWITCHED_RENAME2,
    /**
     * Missing latch.countDown when session == null in VFSBrowser.delete (hunk a).
     */
    MISSING_LATCH_DECREMENT,
    /**
     * Typo in Message (pat instead of path) (hunk n).
     */
    TYPO_PATH,
    /**
     * Wrong type in toString of one of the tasks (hunk i).
     */
    WRONG_TOSTRING_MKDIR,
    /**
     * InterruptedException is logged as Error in VFSBrowser.delete (hunk a).
     */
    INTERRUPTED_EXCEPTION,
    /**
     * Provide wrong boolean value to setCancellable in one of the tasks (hunk h).
     */
    CANCELLABLE_WRONG,
    /**
     * Missing initialization of this.loadInfo in ListDirectoryBrowserTask constructor (hunk k).
     */
    LOADINFO_NOT_SET;

    @Override
    public String getShortString() {
        return this.getClass().getSimpleName() + "_" + this.ordinal();
    }

    @Override
    public boolean isDelocalized() {
        return this == FROM_TO_SWITCHED_RENAME1 || this == FROM_TO_SWITCHED_RENAME2;
    }

    static Map<String, List<Defect>> getTrueDefects() {
        final Map<String, List<Defect>> ret = new LinkedHashMap<>();

        // First group of developers - pilot study
        
        putDef(ret, "#iR;13;we should need to keep all object at class level and used them with 'this' keyword", FP.FP); 
        putDef(ret, "#iR;30;camel case should be follow", FP.FP); 
        putDef(ret, "#hR;40;catch block should have more generic code,need to create one generic method along with required parameter and it should have genric retirn type", FP.FP); 
        putDef(ret, "#hR;46;try catch should not be in finally block", FP.FP); 
        putDef(ret, "#dR;839;we should need to use single  if close along with OR operator for same return type and return statement should be meaning full instead of blank or null", FP.FP); 
        putDef(ret, "#dR;847;need to create new class which is impleneted by Runnable interface,that can be use multiple time from another class", FP.FP);
        putDef(ret, "#iR;1;Provided class implementation", FP.FP);
        putDef(ret, "#kR;49;path spelling changed from path1 to path", FP.FP); 
        putDef(ret, "#nR;106;path spelling changed from path1 to path", FP.FP);
        putDef(ret, "#lR;42;Added variable", FP.FP);
        putDef(ret, "#mR;61;path spelling changed from path1 to path", FP.FP);
        putDef(ret, "#mR;64;path spelling changed from path1 to path", FP.FP);
        putDef(ret, "#mR;70;path spelling changed from path1 to path", FP.FP);
        putDef(ret, "#mR;77;path spelling changed from path1 to path", FP.FP);
        putDef(ret, "#mR;87;path spelling changed from path1 to path", FP.FP);
        putDef(ret, "#jR;1;Provided implemetation of RenameBrowserTask class", FP.FP);
        putDef(ret, "#bR;774;only 6 Parameters are passed insted 7", FP.FP);
        putDef(ret, "#dR;847;Used Runnable interface", FP.FP);
        putDef(ret, "#cR;801;Used Runnable interface", FP.FP);
        putDef(ret, "#mR;70;path spelling", FP.FP);
        putDef(ret, "#mR;77;path spelling", FP.FP);
        putDef(ret, "#mR;87;path spelling", FP.FP);
        putDef(ret, "#hR;1;Better compressing the code by removing all blank lines from the class file.", FP.FP);
        putDef(ret, "#hR;29;Change try{} finally combination to try-with-resource released version is java 7.", FP.FP);
        putDef(ret, "#aR;712;In one line", FP.FP);
        putDef(ret, "#fR;57;Should remove the blank space.", FP.FP);
        putDef(ret, "#dR;847;Rename a file * @param from the full path name of the file to be renamed * @param newname the new name (only filename, not full path)", FP.FP);
        putDef(ret, "#lR;42;private final Object[] loadInfo;", FP.FP);
        putDef(ret, "#nR;106;path=\" + path + ']';", P1.TYPO_PATH );
        putDef(ret, "#hR;30;setcancellable is true", P1.CANCELLABLE_WRONG);
        putDef(ret, "#fR;46;Changed Path1 to path", FP.FP);
        putDef(ret, "#iR;62;type=MKDIR", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#nR;106;it should be path", P1.TYPO_PATH);
        putDef(ret, "#cR;808;parameter sequence is wrong. it should be from and then to", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#lR;42;it should final", FP.FP);
        putDef(ret, "#aR;708;latch.countDown();", P1.MISSING_LATCH_DECREMENT);
        putDef(ret, "#lR;42;final", FP.FP);
        putDef(ret, "#nR;106;path", P1.TYPO_PATH);
        putDef(ret, "#iR;63;type=MKDIR", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;61;Okay", FP.FP);
        putDef(ret, "#hR;36;okay", FP.FP);
        putDef(ret, "#kR;56;where  you have set input param lodinfo", P1.LOADINFO_NOT_SET);
        putDef(ret, "#nR;106;okay", FP.FP);
        putDef(ret, "#mR;61;okay", FP.FP);
        putDef(ret, "#mR;64;okay", FP.FP);
        putDef(ret, "#jR;77;okay", FP.FP);
        putDef(ret, "#bR;773;we directly pass the Runnable interface here, no need to create delayed AWT instance", FP.FP);
        putDef(ret, "#dR;871;we directly pass the Runnable interface here, no need to create runnable  instance", FP.FP);
        putDef(ret, "#aR;732;okay", FP.FP);
        putDef(ret, "#cR;808;we directly pass the Runnable interface here, no need to create delayed AWT instance", FP.FP);
        putDef(ret, "#iR;49;try-catch block should not be in finally block", FP.FP);
        putDef(ret, "#mR;79;try-catch block should not be in finally block", FP.FP);
        putDef(ret, "#jR;71;try-catch block should not be in finally block", FP.FP);
        putDef(ret, "#dR;847;we need to write a separate class which implements runnable interface and write a logic over there.", FP.FP);
        putDef(ret, "#dR;847;Need to create new class which is implemented by runnable interface that we can access multiple times.", FP.FP);
        putDef(ret, "#jR;37;Camelcase should be follow.", FP.FP);
        putDef(ret, "#jR;56;Return statement should be meaning full instead of blank or null", FP.FP);
        putDef(ret, "#jR;71;Try catch should not be in finally block , Catch block should be more generic & code need to be create one generic method along with required generic parameter.", FP.FP);
        putDef(ret, "#iR;13;We should need to keep all object at class level & used them with this keyword.", FP.FP);
        putDef(ret, "#hR;19;no issue , Okay", FP.FP);
        putDef(ret, "#aR;703;no issue , Okay", FP.FP);
        putDef(ret, "#aR;720;no issue , Okay", FP.FP);
        putDef(ret, "#bR;771;we can pass directly Runnable interface as input to RenameBrowserTask, no need to create runnable instance", FP.FP);
        putDef(ret, "#cR;802;we can pass directly Runnable interface as input to RenameBrowserTask, no need to create runnable instance", FP.FP);
        putDef(ret, "#dR;848;Okay", FP.FP);
        putDef(ret, "#dR;870;Okay", FP.FP);
        putDef(ret, "#fR;54;no issue , Okay", FP.FP);
        putDef(ret, "#mR;61;no issue , Okay", FP.FP);
        putDef(ret, "#mR;64;no issue , Okay", FP.FP);
        putDef(ret, "#mR;70;no issue , Okay", FP.FP);
        putDef(ret, "#mR;87;no issue , Okay", FP.FP);
        putDef(ret, "#jR;41;no issue , Okay", FP.FP);
        putDef(ret, "#iR;35;no issue , Okay", FP.FP);
        putDef(ret, "#iR;13;we need to keep all the objects at class level and use them with this keyword", FP.FP);
        putDef(ret, "#iR;30;camel case should be followed", FP.FP);
        putDef(ret, "#hR;31;initialization for the array of string is missing", FP.FP);
        putDef(ret, "#hR;46;try catch block should not be present in finally block", FP.FP);
        putDef(ret, "#dR;847;Need to create class which would implement runnable class , and runnable should be created seperatly", FP.FP);
        putDef(ret, "#jR;47;we could use single if for conditional checking by using OR(||) oprrator", FP.FP);
        putDef(ret, "#jR;56;retrun type statement should be meaning full insted of blank or null", FP.FP);
        putDef(ret, "#jR;56;retrun type statement should be meaning full", FP.FP);
        
        
        // Second group of developers - pilot study
        
        
        putDef(ret, "#iR;31;path should be defined as Instance variable in AbstractBrowserTask", FP.FP);
        putDef(ret, "#iR;43;Line no 43 to 46 are repeating with line no 53 to 56 , we can make one method and avoid duplication of code", FP.FP);
        putDef(ret, "#mR;73;Line no 74 to 77 are repeating with line no 84 to 87, we can make one method and avoid duplication of code", FP.FP);
        putDef(ret, "#nR;106;We can not use pat as coding standards, we should use meaningful name", P1.TYPO_PATH);
        putDef(ret, "#hR;40;Line no 41 to 44 are repeating with line no 51 to 54, we can make one method and avoid duplication of code", FP.FP);
        putDef(ret, "#kR;56;What is use of \"Object[] loadInfo\"?", P1.LOADINFO_NOT_SET);
        putDef(ret, "#jR;33;It should be \"this.path2 = path2;\"", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;47;We can use ternary operator to reduce the code lines", FP.FP);
        putDef(ret, "#jR;65;Line no 66 to 69 are repeating with line no 76 to 79, we can make one method and avoid duplication of code", FP.FP);
        putDef(ret, "#lR;42;As \"Object[] loadInfo\" is not used anywhere in \"ListDirectoryBrowserTask.java\", no need to define it.", FP.FP);
        putDef(ret, "#iR;42;code dublication  here we have written same code two times so that we can reduce the lengh of code by removing catch block beacouse we can write try with finally or catch", FP.FP);
        putDef(ret, "#iR;43;repeated code", FP.FP);
        putDef(ret, "#iR;61;overrided string mehod to get content of  the object", FP.FP);
        putDef(ret, "#kR;57;here we are passing object to the super class parameterize constructor", FP.FP);
        putDef(ret, "#nR;106;\",pat=\"  make it as path", P1.TYPO_PATH);
        putDef(ret, "#mR;61; asuming path is instance variable created inside super class of ListDirectoryBrouserTask  thats why we access", FP.FP);
        putDef(ret, "#jR;33;Replacing path1 by path2", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;47;nested if can be use", FP.FP);
        putDef(ret, "#bR;768;runnable interface run() method overrided here using annonymus inner class", FP.FP);
        putDef(ret, "#hR;32;assuming getProperty(String str,object[] args) method of Editor class", FP.FP);
        putDef(ret, "#hR;40;duplication of code", FP.FP);
        putDef(ret, "#mR;73;code duplication we can reduce number of lines of code by removing duplicate code", FP.FP);
        putDef(ret, "#kR;58;initializing loadInfo with this.loadInfo = loadInfo;", P1.LOADINFO_NOT_SET);
        putDef(ret, "#lR;42;it should be final", FP.FP);
        putDef(ret, "#jR;33;it should be this.path2=path2;", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#iR;42;code duplication we can reduce number of lines of code by removing duplicate code", FP.FP);
        putDef(ret, "#iR;32;In try block you should write only risky code.remaining code should write out side try block", FP.FP);
        putDef(ret, "#jR;47;Here if check the file is null the we will get exception.", FP.FP);
        putDef(ret, "#aR;704;code can be optimised with the help of foreach loop insted of for loop", FP.FP);
        putDef(ret, "#jR;47;file resource should be closed at the end of if block", FP.FP);
        putDef(ret, "#dR;848;to run any thread we need to call start method to invoke in run method of runnable.", FP.FP);
        putDef(ret, "#jR;30;unnecessary use of path2 variable.", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#hR;28;naming conventions is not corrent for method.", FP.FP);
        putDef(ret, "#aR;665;need to compile and run code to check exact use of fucntions and weather is work without exceptions.", FP.FP);
        putDef(ret, "#dR;850;If request is start at first thread then it should also end at same thread", FP.FP);
        putDef(ret, "#dR;857;Please check before casting is instanceOf", FP.FP);
        putDef(ret, "#jR;30;Path 2 is not been used as a parameter ?", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;48;Better to negate this condition so that line of code can be removed", FP.FP);
        putDef(ret, "#aR;665;Add check for files array is not empty", FP.FP);
        putDef(ret, "#aR;680;Use foreach loop", FP.FP);
        putDef(ret, "#jR;47;file resource should be closed properly", FP.FP);
        putDef(ret, "#aR;704;could make use of for each loop for better performance", FP.FP);
        putDef(ret, "#iR;13;class shoud be start with public", FP.FP);
        putDef(ret, "#iR;2;Remove unwanted space", FP.FP);
        putDef(ret, "#fR;46;instead of path , use \" virtual file path\", naming is required.", FP.FP);
        putDef(ret, "#fR;54;instead of path , use \" virtual file path\", naming is required.", FP.FP);
        putDef(ret, "#oR;1;if  \"BrowserIORequest.java\" file is blank & not in used delete it", FP.FP);
        putDef(ret, "#kR;49;instead of path , use \" virtual file path\", naming is required.", FP.FP);
        putDef(ret, "#kR;55;instead of path , use \" virtual file path\", naming is required.", FP.FP);
        putDef(ret, "#kR;57;instead of path , use \" virtual file path\", naming is required.", FP.FP);
        putDef(ret, "#kR;57;instead of path , use \" virtual file path\", naming is required.", FP.FP);
        putDef(ret, "#nR;106;instead of path , use \" virtual file path\", naming is required.", FP.FP);
        putDef(ret, "#mR;61;instead of path , use \" virtual file path\", naming is required.", FP.FP);
        putDef(ret, "#mR;64;instead of path , use \" virtual file path\", naming is required.", FP.FP);
        putDef(ret, "#mR;70;instead of path , use \" virtual file path\", naming is required.", FP.FP);
        putDef(ret, "#mR;77;instead of path , use \" virtual file path\", naming is required.", FP.FP);
        putDef(ret, "#mR;87;instead of path , use \" virtual file path\", naming is required.", FP.FP);
        putDef(ret, "#jR;15;class shoud be start with public", FP.FP);
        putDef(ret, "#bR;768;no need to create delayedAWT Runnable object we can directly pass the Runnable interface with anonymous function to RenameBrowserTask method", FP.FP);
        putDef(ret, "#bR;774;no need to create delayedAWT Runnable object we can directly pass the Runnable interface with anonymous function to RenameBrowserTask method", FP.FP);
        putDef(ret, "#dR;847;no need to create delayedAWT Runnable object we can directly pass the Runnable interface with anonymous function to RenameBrowserTask method", FP.FP);
        putDef(ret, "#dR;871;no need to create [Task mkdirTask = new MkDirBrowserTask] new object, you can directly pass MkDirBrowserTask constructor to ThreadUtilities.runInBackground()", FP.FP);
        putDef(ret, "#aR;703;CountDownlatch object not necssary to create as final", FP.FP);
        putDef(ret, "#eR;40;if \"AbstractBrowserTask.java\" file is blank & not in used delete it", FP.FP);
        putDef(ret, "#hR;30;no issue found", FP.FP);
        putDef(ret, "#aR;718;	no need to create [Task task = new DeleteBrowserTask] new object, 	you can directly pass DeleteBrowserTask constructor to ThreadUtilities.runInBackground(task)", FP.FP);
        putDef(ret, "#bR;769;no need to create delayedAWT Runnable object we can directly pass the Runnable interface with anonymous function to RenameBrowserTask method", FP.FP);
        putDef(ret, "#bR;775;no need to create renameTask Runnable object we can directly pass the Runnable interface with anonymous function to RenameBrowserTask method", FP.FP);
        putDef(ret, "#cR;804;no need to create delayedAWT Runnable object we can directly pass the Runnable interface with anonymous function to RenameBrowserTask method", FP.FP);
        putDef(ret, "#dR;855;no need to create [Task mkdirTask = new MkDirBrowserTask] new object, 	you can directly pass MkDirBrowserTask constructor to ThreadUtilities.runInBackground()", FP.FP);
        putDef(ret, "#gR;63;instead of path , use \" virtual file path\", naming is required.", FP.FP);
        putDef(ret, "#lR;42;it must be initialized", FP.FP);
        putDef(ret, "#jR;15;class should be start with public", FP.FP);
        putDef(ret, "#iR;52;not getting clear idea", FP.FP);
        putDef(ret, "#aR;713;Should be added inside try-catch block", FP.FP);
        putDef(ret, "#nR;106;'path' spelling is misspelled as 'pat'", P1.TYPO_PATH);
        putDef(ret, "#jR;86;Reflection could be used for toString method", FP.FP); 
        putDef(ret, "#cR;804;Not sure but start and endRequest should be done on the same thread", FP.FP); 
        putDef(ret, "#fR;57;The taskManager seems to be a singleton file , but the way its acquiring the instance is not right", FP.FP); 
        
        
        //Third group of developers - pilot study
        
        
        putDef(ret, "#aR;703;we can directly pass the Runnable interface with anonymous function to RenameBrowserTask method CountDownlatch object not necssary to create as final", FP.FP); 
        putDef(ret, "#bR;768;no need to create delayedAWT Runnable object", FP.FP); 
        putDef(ret, "#dR;848;we can directly pass the Runnable interface with anonymous function to RenameBrowserTask method CountDownlatch object not necssary to create as final", FP.FP); 
        putDef(ret, "#dR;870;no need to create [Task mkdirTask = new MkDirBrowserTask] new object, you can directly pass MkDirBrowserTask constructor to ThreadUtilities.runInBackground() if \"BrowserIORequest.java\" file is blank & not in used delete it", FP.FP); 
        putDef(ret, "#fR;47;instead of path , use \" virtual file path\", naming is required.", FP.FP); 
        putDef(ret, "#fR;55;instead of path , use \" virtual file path\", naming is required.", FP.FP); 
        putDef(ret, "#iR;14;class should be start with public", FP.FP); 
        putDef(ret, "#jR;48;Again to path verified here, chaces for \"NullPointerException\"", FP.FP); 
        putDef(ret, "#hR;47;please write finally block after catch block to close rosources.", FP.FP); 
        putDef(ret, "#dR;847;Write a seperate class which implement Runnable intreface and write login in that for modularity.", FP.FP); 
        putDef(ret, "#jR;47;The file resources should be closed.", FP.FP); 
        putDef(ret, "#aR;704;Code can be optimised using foreach loop instead of for loop.", FP.FP); 
        putDef(ret, "#cR;808;this keyword is not required.", FP.FP);
        putDef(ret, "#mR;75;catch and finally's catch block code can be written in a private method of the class and can be called in this block", FP.FP);
        putDef(ret, "#kR;56;no need to use  loadInfo as parameter in constructor as it is not used", P1.LOADINFO_NOT_SET);
        putDef(ret, "#dR;811;invocation of this method is not found", FP.FP);
        putDef(ret, "#dR;823;declaration of path is missing.", FP.FP);
        putDef(ret, "#aR;704;code can be optimised here using foreach loop", FP.FP);
        putDef(ret, "#lR;42;this can be declared as final and intialize in constructor.", FP.FP);
        putDef(ret, "#bR;771;thread should not start in different thread and end in other thread in this kind of application..", FP.FP);
        putDef(ret, "#dR;858;Before casting we should use instanceof checker.", FP.FP);
        putDef(ret, "#eR;40;if you are not using AbstractBrowserTask.java then remove it", FP.FP);
        putDef(ret, "#hR;25;no error found", FP.FP);
        putDef(ret, "#aR;722;no need to create a Task", FP.FP);
        putDef(ret, "#bR;769;no need to create delayedAWT Runnable object", FP.FP);
        putDef(ret, "#cR;802;no need to create delayedAWT Runnable object", FP.FP);
        putDef(ret, "#dR;870;no need to create [Task mkdirTask = new MkDirBrowserTask] new object, 	"
        		+ "you can directly pass MkDirBrowserTask constructor to ThreadUtilities.runInBackground()  	if  \"BrowserIORequest.java\" file is blank & not in used delete it", FP.FP);
        putDef(ret, "#jR;28;no error found", FP.FP);
        putDef(ret, "#iR;12;no error found", FP.FP);
        putDef(ret, "#oR;1;remove if you are not using this class", FP.FP);


        /**
         * Pilot - internal
         */

        putDef(ret, "#jR;33;to set the path, shouldn't this be \"this.path2=path2\"?", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;43;should be path1", FP.FP);
        putDef(ret, "#jR;53;again, browsertask gets \"String path1\"...", FP.FP);
        putDef(ret, "#jR;60;again \"path1\"", FP.FP);
        putDef(ret, "#bR;775;to & from switched... RenameBrowserTask(this, session, vfs, from, to, delayedAWT)", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#cR;808;to and from switched...", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#jR;25;do you still need  path1 and path2? path2 willbe == to path1 anyway..", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;87;path or path1? should it just be path instead of path  and path2?", FP.FP);
        putDef(ret, "#fR;40;splitting the  code chunk in twomadethe  review harder", FP.FP);
        putDef(ret, "#gR;63;what about the runnable?", FP.FP);
        putDef(ret, "#hR;24;you're passing null, which will make the if pass  and not create the listener", FP.FP);
        putDef(ret, "#kR;56;load info is not used after the changes in the super constructor", P1.LOADINFO_NOT_SET);
        putDef(ret, "#nR;106;typo? pat->path", P1.TYPO_PATH);
        putDef(ret, "#hR;24;runnable not initialized", FP.FP);
        putDef(ret, "#hR;37;is this an io error?", FP.FP);
        putDef(ret, "#kR;56;load info unused", P1.LOADINFO_NOT_SET);
        putDef(ret, "#nR;106;typo path", P1.TYPO_PATH);

        /**
         * first 10 developers
         */

        putDef(ret, "#fR;57;TaskManager.getInstance().addTaskListener(listener);", FP.FP);
        putDef(ret, "#hR;30;browser.setCancellable(false);", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;32;browser.setStatus(Editor.getProperty(\"vfs.status.deleting\", args));", FP.FP);
        putDef(ret, "#hR;41;browser.setCancellable(false);", FP.FP);
        putDef(ret, "#hR;51;browser.setCancellable(false);", FP.FP);
        putDef(ret, "#mR;62;browser.setStatus(Editor.getProperty(\"vfs.status.listing-directory\", args));", FP.FP);
        putDef(ret, "#mR;68;browser.setCancellable(true);", FP.FP);
        putDef(ret, "#mR;74;browser.setCancellable(false);", FP.FP);
        putDef(ret, "#mR;84;browser.setCancellable(false);", FP.FP);
        putDef(ret, "#mR;91;browser.setCancellable(false);", FP.FP);
        putDef(ret, "#nR;106;+ \",vfs=\" + vfs + \",path=\" + path + \"]\";", P1.TYPO_PATH);
        putDef(ret, "#iR;33;browser.setCancellable(true);", FP.FP);
        putDef(ret, "#iR;34;browser.setStatus(Editor.getProperty(\"vfs.status.mkdir\", args));", FP.FP);
        putDef(ret, "#iR;43;browser.setCancellable(false);", FP.FP);
        putDef(ret, "#iR;53;browser.setCancellable(false);", FP.FP);
        putDef(ret, "#jR;39;browser.setCancellable(false);", FP.FP);
        putDef(ret, "#jR;41;browser.setStatus(Editor.getProperty(\"vfs.status.renaming\", args));", FP.FP);
        putDef(ret, "#jR;66;browser.setCancellable(false);", FP.FP);
        putDef(ret, "#jR;76;browser.setCancellable(false);", FP.FP);
        putDef(ret, "#aR;713;TaskManager.getInstance().addTaskListener(new TaskAdapter()", FP.FP);
        putDef(ret, "#aR;718;TaskManager.getInstance().addTaskListener(new TaskAdapter()", FP.FP);
        putDef(ret, "#hR;34;its not recommended to start method name with underscore.underscore is optional", FP.FP);
        putDef(ret, "#hR;40;as part of catch block user understandable exception is not thrown to end user", FP.FP);
        putDef(ret, "#hR;50;as part of catch block user understandable exception is not thrown to end user", FP.FP);
        putDef(ret, "#mR;62;browser.setStatus(Editor.getProperty(\"vfs.status.deleting\", args));", FP.FP);
        putDef(ret, "#mR;68;browser.setStatus(Editor.getProperty(\"vfs.status.deleting\", args));", FP.FP);
        putDef(ret, "#jR;39;browser.setCancellable(true);", FP.FP);
        putDef(ret, "#jR;66;browser.setStatus(Editor.getProperty(\"vfs.status.renaming\", args));", FP.FP);
        putDef(ret, "#aR;718;TaskManager.getInstance().removeTaskListener(this);", FP.FP);
        putDef(ret, "#eR;40;It is not implementing any interface", FP.FP);
        putDef(ret, "#dR;811;not any variable declared", FP.FP);
        putDef(ret, "#fR;51;browser not used", FP.FP);
        putDef(ret, "#fR;52;session not used", FP.FP);
        putDef(ret, "#fR;53;vfs not used", FP.FP);
        putDef(ret, "#fR;54;path not used", FP.FP);
        putDef(ret, "#hR;22;constructor never return any value", FP.FP);
        putDef(ret, "#hR;23;session, vsf, path, browser in constructor not used", FP.FP);
        putDef(ret, "#hR;28;_run() method not returning any value, its return type is void", FP.FP);
        putDef(ret, "#hR;36;not eadge", FP.FP);
        putDef(ret, "#hR;52;Should be give understandable message", FP.FP);
        putDef(ret, "#kR;42;Its not implementing or extmding anything", P1.LOADINFO_NOT_SET);
        putDef(ret, "#kR;55;not checking any parameters validity", FP.FP);
        putDef(ret, "#iR;30;_run() method not returning any value", FP.FP);
        putDef(ret, "#iR;31;not checking any validity", FP.FP);
        putDef(ret, "#iR;38;not eadge cases", FP.FP);
        putDef(ret, "#jR;37;not returning any value", FP.FP);
        putDef(ret, "#jR;60;not eadge", FP.FP);

        /**
         * E - first 30 developers (second inter-rater batch)

        putDef(ret, "#hR;28;method name should start with small case.its not recommended to start with underscore", FP.FP);
        putDef(ret, "#hR;34;method name should start with small case.its not recommended to start with underscore", FP.FP);
        putDef(ret, "#hR;40;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#hR;50;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#mR;73;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#mR;83;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#iR;30;method name should start with small case.its not recommended to start with underscore", FP.FP);
        putDef(ret, "#iR;42;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#iR;50;method name should start with small case.its not recommended to start with underscore", FP.FP);
        putDef(ret, "#jR;37;method name should start with small case.its not recommended to start with underscore", FP.FP);
        putDef(ret, "#jR;65;exception is catched but not thrown user understandable exception to end user", FP.FP);

        putDef(ret, "#nR;106;Spelling mistake in path variable name", P1.TYPO_PATH);
        putDef(ret, "#mR;60;method name should start with small case not with underscore.", FP.FP);
        putDef(ret, "#mR;70;method name should statrt with small case .its not recommended to start with underscore", FP.FP);
        putDef(ret, "#mR;81;method name should statrt with small case .its not recommended to start with underscore", FP.FP);
        putDef(ret, "#iR;50;method name should start with small case .not recommended to start with underscore", FP.FP);
        putDef(ret, "#iR;52;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#aR;728;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#cR;801;we can use single abstract method instead ofcreating new", FP.FP);
        putDef(ret, "#mR;60;method name should be start with small case", FP.FP);
        putDef(ret, "#mR;83;exception is catched but not thrown user understandable exception to end userType a message", FP.FP);
        putDef(ret, "#iR;52;exception is catched but not thrown user understandable exception to end userType a message", FP.FP);
        putDef(ret, "#aR;728;we can throw and show user friendly message to end user", P1.INTERRUPTED_EXCEPTION);
        putDef(ret, "#dR;850;Runnable interface of run method", FP.FP);
        putDef(ret, "#dR;872;Run the runnable in the threadpool", FP.FP);
        putDef(ret, "#fR;55;Null is getting from DeleteBrowserTask", FP.FP);
        putDef(ret, "#fR;57;in case of DeleteBrowserTask no addTaskListener is added", FP.FP);
        putDef(ret, "#hR;24;Sending null to Runnable task", FP.FP);
        putDef(ret, "#hR;37;Showwing error in case of sesson or path is not sfound", FP.FP);
        putDef(ret, "#hR;48;Always end the session", FP.FP);
        putDef(ret, "#jR;17;path 2 is declare but not initialized", FP.FP);

        putDef(ret, "#mR;60;method name should start with small case.its not recommended to start with underscore", FP.FP);
        putDef(ret, "#jR;75;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#kR;57;Object[] loadInfo is not used in constructor", P1.LOADINFO_NOT_SET);
        putDef(ret, "#mR;68;browser.setCancellable(false);", FP.FP);
        putDef(ret, "#iR;33;browser.setCancellable(false);", FP.FP);
        putDef(ret, "#bR;775;this, session, vfs, from,to, delayedAWT);", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#cR;808;this, session, vfs, from,to, delayedAWT);", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#nR;106;[type=LIST_DIRECTORY\" + \",vfs=\" + vfs + \",path=\" + path + \"]\";", P1.TYPO_PATH);
        putDef(ret, "#jR;30;Object session, VFS vfs, String path, String path2,", FP.FP);
        putDef(ret, "#jR;32;super(browser, session, vfs, path, awtRunnable);", FP.FP);
        putDef(ret, "#jR;33;this.path = path;", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#aR;713;TaskManager.getInstance().addTaskListener(new TaskAdapter() {", FP.FP);
        putDef(ret, "#jR;33;this.path2 = path;", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;40;String[] args = {path1, path2};", FP.FP);
        putDef(ret, "#fR;50;we can't use Object directly..", FP.FP);
        putDef(ret, "#hR;51;we need to test BufferWriter is not null before closing it.", FP.FP);

         third batch of remarks for inter-rater agreement

        putDef(ret, "#mR;61;run methods couple of parameters are not used", FP.FP);
        putDef(ret, "#mR;81;It will create confusion if any error occurs.", FP.FP);
        putDef(ret, "#jR;30;we can't use Object as an arguments in method call.", FP.FP);
        putDef(ret, "#bR;771;this method is not defined in the class", FP.FP);
        putDef(ret, "#hR;28;method name should start with small case letter but not with underscore", FP.FP);
        putDef(ret, "#hR;40;the exception is handled but userUnderstandable exception is not thrown", FP.FP);
        putDef(ret, "#hR;50;the exception is handled but userUnderstandable exception is not thrown", FP.FP);
        putDef(ret, "#mR;60;method name should not start with underscore .its recommended to start with small case letter", FP.FP);
        putDef(ret, "#mR;73;exception is handled but user understandable exception is not thrown", FP.FP);
        putDef(ret, "#mR;83;exception is handled but user understandable exception is not thrown", FP.FP);
        putDef(ret, "#nR;106;tostring method should be properly override", FP.FP);
        putDef(ret, "#jR;37;method name should start with small case letter not with underscore", FP.FP);

        putDef(ret, "#hR;28;naming convention", FP.FP);
        putDef(ret, "#hR;30;method not found", FP.FP);
        putDef(ret, "#hR;32;method not found", FP.FP);
        putDef(ret, "#hR;51;method not found", FP.FP);
        putDef(ret, "#nR;106;spelling", P1.TYPO_PATH);
        putDef(ret, "#aR;729;log.error should be used", P1.INTERRUPTED_EXCEPTION);
        putDef(ret, "#nR;106;There is a typo in variable name for path", P1.TYPO_PATH);
        putDef(ret, "#fR;54;avoid ambitiyy", FP.FP);
        putDef(ret, "#fR;64;Used public", FP.FP);
        putDef(ret, "#mR;60;Method name should not start with underscore and method name starts with small case is recommonded", FP.FP);
        putDef(ret, "#iR;50;Method name should not start with underscore, starting with small case is recommended", FP.FP);
        putDef(ret, "#iR;52;Exception has been handled, but user understadable exception didn't thrown", FP.FP);
        putDef(ret, "#aR;728;Exception has been handled but user understable exception not thrown", P1.INTERRUPTED_EXCEPTION);
        putDef(ret, "#cR;806;shoud use comments for that user understandable", FP.FP);
        putDef(ret, "#hR;41;should also use logger there", FP.FP);
        putDef(ret, "#hR;42;should use approperite message to understands", FP.FP);
        putDef(ret, "#hR;52;user proper error message or throw exception", FP.FP);
        putDef(ret, "#mR;74;should use proper exception msg here", FP.FP);
        putDef(ret, "#nR;106;should use \"path\" as proper text for toString", P1.TYPO_PATH);
        putDef(ret, "#aR;721;should use javadoc and proper commnets", FP.FP);
        putDef(ret, "#aR;729;should use proper exception message and print logger information", P1.INTERRUPTED_EXCEPTION);
        putDef(ret, "#eR;40;removed abstractBrowserTask parameterized constructor", FP.FP);
        putDef(ret, "#bR;769;updated rename code1", FP.FP);
        putDef(ret, "#bR;776;updated rename code2", FP.FP);
        putDef(ret, "#cR;801;updated rename code3", FP.FP);
        putDef(ret, "#cR;808;updated rename code 4", FP.FP);
        putDef(ret, "#dR;847;formed make dir code to new style", FP.FP);
        putDef(ret, "#dR;871;finished make dir code according to above", FP.FP);
        putDef(ret, "#fR;46;updated paths param info to consolidated path", FP.FP);
        putDef(ret, "#fR;54;changed path param as per new path style", FP.FP);
        putDef(ret, "#fR;64;updated paths declaration as per new style", FP.FP);
        putDef(ret, "#oR;1;removed BrowserIORequest stuff", FP.FP);
        putDef(ret, "#hR;21;added DeleteBrowserTask Stuffs", FP.FP);
        putDef(ret, "#hR;38;added DeleteBrowserTask stuffs2", FP.FP);
        putDef(ret, "#kR;42;new declared loadInfo array as private class variable", FP.FP);
        putDef(ret, "#kR;50;renamed path var as per new style", FP.FP);
        putDef(ret, "#kR;56;updated path param name", FP.FP);
        putDef(ret, "#kR;58;updated path param name", FP.FP);
        putDef(ret, "#mR;61;renamed path var according to new declr style", FP.FP);
        putDef(ret, "#mR;64;renamed path var according to new declr style", FP.FP);
        putDef(ret, "#mR;70;renamed path var according to new declr style", FP.FP);
        putDef(ret, "#mR;77;renamed path var according to new declr style", FP.FP);
        putDef(ret, "#mR;87;renamed path var according to new declr style", FP.FP);
        putDef(ret, "#nR;106;renamed path var according to new declr style", FP.FP);
        putDef(ret, "#iR;1;moved MkDirBrowserTask ro package browser", FP.FP);
        putDef(ret, "#iR;3;added MkDirBrowserTask code", FP.FP);
        putDef(ret, "#jR;1;moved RenameBrowserTask to pkg browser", FP.FP);
        putDef(ret, "#jR;4;added RenameBrowserTask code", FP.FP);
        putDef(ret, "#aR;703;Optimized the loop", FP.FP);
        putDef(ret, "#aR;711;Updated subtask delete code as per New Task based guideline", FP.FP);
        putDef(ret, "#aR;732;updated endRequest call even according to new approach", FP.FP);

        putDef(ret, "#hR;28;error logs and logs are not proper", FP.FP);
        putDef(ret, "#hR;34;check exception", FP.FP);
        putDef(ret, "#jR;37;add loggers", FP.FP);
        putDef(ret, "#eR;40;remove code", FP.FP);
        putDef(ret, "#bR;768;change code", FP.FP);
        putDef(ret, "#bR;774;change code", FP.FP);
        putDef(ret, "#cR;802;change code", FP.FP);
        putDef(ret, "#cR;808;change code", FP.FP);
        putDef(ret, "#dR;848;change code", FP.FP);
        putDef(ret, "#dR;870;change code", FP.FP);
        putDef(ret, "#fR;46;change code", FP.FP);
        putDef(ret, "#fR;50;change code", FP.FP);
        putDef(ret, "#fR;54;change code", FP.FP);
        putDef(ret, "#fR;64;change code", FP.FP);
        putDef(ret, "#oR;1;remove code", FP.FP);
        putDef(ret, "#hR;1;add code", FP.FP);
        putDef(ret, "#kR;42;add code", FP.FP);
        putDef(ret, "#kR;50;change code", FP.FP);
        putDef(ret, "#kR;56;change code", FP.FP);
        putDef(ret, "#kR;58;change code", FP.FP);
        putDef(ret, "#mR;61;change code", FP.FP);
        putDef(ret, "#mR;64;change code", FP.FP);
        putDef(ret, "#mR;70;change code", FP.FP);
        putDef(ret, "#mR;77;change code", FP.FP);
        putDef(ret, "#mR;87;change code", FP.FP);
        putDef(ret, "#nR;106;change code", FP.FP);
        putDef(ret, "#iR;1;add code", FP.FP);
        putDef(ret, "#iR;20;add code", FP.FP);
        putDef(ret, "#jR;1;add code", FP.FP);
        putDef(ret, "#jR;5;add code", FP.FP);
        putDef(ret, "#aR;703;add code", FP.FP);
        putDef(ret, "#aR;712;change code", FP.FP);
        putDef(ret, "#aR;733;change code", FP.FP);

        putDef(ret, "#bR;768;use single abstract method style", FP.FP);

        */

        /**
         * Final version - first 30 developers
         */

        putDef(ret, "#hR;28;method name should start with small case.its not recommended to start with underscore", FP.FP);
        putDef(ret, "#hR;34;method name should start with small case.its not recommended to start with underscore", FP.FP);
        putDef(ret, "#hR;40;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#hR;50;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#mR;73;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#mR;83;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#iR;30;method name should start with small case.its not recommended to start with underscore", FP.FP);
        putDef(ret, "#iR;42;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#iR;50;method name should start with small case.its not recommended to start with underscore", FP.FP);
        putDef(ret, "#jR;37;method name should start with small case.its not recommended to start with underscore", FP.FP);
        putDef(ret, "#jR;65;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#nR;106;Spelling mistake in path variable name", P1.TYPO_PATH);
        putDef(ret, "#mR;60;method name should start with small case not with underscore.", FP.FP);
        putDef(ret, "#mR;70;method name should statrt with small case .its not recommended to start with underscore", FP.FP);
        putDef(ret, "#mR;81;method name should statrt with small case .its not recommended to start with underscore", FP.FP);
        putDef(ret, "#iR;50;method name should start with small case .not recommended to start with underscore", FP.FP);
        putDef(ret, "#iR;52;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#aR;728;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#cR;801;we can use single abstract method instead ofcreating new", FP.FP);
        putDef(ret, "#mR;60;method name should be start with small case", FP.FP);
        putDef(ret, "#mR;83;exception is catched but not thrown user understandable exception to end userType a message", FP.FP);
        putDef(ret, "#iR;52;exception is catched but not thrown user understandable exception to end userType a message", FP.FP);
        putDef(ret, "#aR;728;we can throw and show user friendly message to end user", P1.INTERRUPTED_EXCEPTION);
        putDef(ret, "#dR;850;Runnable interface of run method", FP.FP);
        putDef(ret, "#dR;872;Run the runnable in the threadpool", FP.FP);
        putDef(ret, "#fR;55;Null is getting from DeleteBrowserTask", FP.FP);
        putDef(ret, "#fR;57;in case of DeleteBrowserTask no addTaskListener is added", FP.FP);
        putDef(ret, "#hR;24;Sending null to Runnable task", FP.FP);
        putDef(ret, "#hR;37;Showwing error in case of sesson or path is not sfound", FP.FP);
        putDef(ret, "#hR;48;Always end the session", FP.FP);
        putDef(ret, "#jR;17;path 2 is declare but not initialized", FP.FP);
        putDef(ret, "#mR;60;method name should start with small case.its not recommended to start with underscore", FP.FP);
        putDef(ret, "#jR;75;exception is catched but not thrown user understandable exception to end user", FP.FP);
        putDef(ret, "#kR;57;Object[] loadInfo is not used in constructor", P1.LOADINFO_NOT_SET);
        putDef(ret, "#mR;68;browser.setCancellable(false);", FP.FP);
        putDef(ret, "#iR;33;browser.setCancellable(false);", FP.FP);
        putDef(ret, "#bR;775;this, session, vfs, from,to, delayedAWT);", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#cR;808;this, session, vfs, from,to, delayedAWT);", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#nR;106;[type=LIST_DIRECTORY\" + \",vfs=\" + vfs + \",path=\" + path + \"]\";", P1.TYPO_PATH);
        putDef(ret, "#jR;30;Object session, VFS vfs, String path, String path2,", FP.FP);
        putDef(ret, "#jR;32;super(browser, session, vfs, path, awtRunnable);", FP.FP);
        putDef(ret, "#jR;33;this.path = path;", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#aR;713;TaskManager.getInstance().addTaskListener(new TaskAdapter() {", FP.FP);
        putDef(ret, "#jR;33;this.path2 = path;", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;40;String[] args = {path1, path2};", FP.FP);
        putDef(ret, "#fR;50;we can't use Object directly..", FP.FP);
        putDef(ret, "#hR;51;we need to test BufferWriter is not null before closing it.", FP.FP);

        /**
        2nd batch
         */

        putDef(ret, "#mR;61;run methods couple of parameters are not used", FP.FP);
        putDef(ret, "#mR;81;It will create confusion if any error occurs.", FP.FP);
        putDef(ret, "#jR;30;we can't use Object as an arguments in method call.", FP.FP);
        putDef(ret, "#bR;771;this method is not defined in the class", FP.FP);
        putDef(ret, "#hR;28;method name should start with small case letter but not with underscore", FP.FP);
        putDef(ret, "#hR;40;the exception is handled but userUnderstandable exception is not thrown", FP.FP);
        putDef(ret, "#hR;50;the exception is handled but userUnderstandable exception is not thrown", FP.FP);
        putDef(ret, "#mR;60;method name should not start with underscore .its recommended to start with small case letter", FP.FP);
        putDef(ret, "#mR;73;exception is handled but user understandable exception is not thrown", FP.FP);
        putDef(ret, "#mR;83;exception is handled but user understandable exception is not thrown", FP.FP);
        putDef(ret, "#nR;106;tostring method should be properly override", FP.FP);
        putDef(ret, "#jR;37;method name should start with small case letter not with underscore", FP.FP);
        putDef(ret, "#hR;28;naming convention", FP.FP);
        putDef(ret, "#hR;30;method not found", FP.FP);
        putDef(ret, "#hR;32;method not found", FP.FP);
        putDef(ret, "#hR;51;method not found", FP.FP);
        putDef(ret, "#nR;106;spelling", P1.TYPO_PATH);
        putDef(ret, "#aR;729;log.error should be used", P1.INTERRUPTED_EXCEPTION);
        putDef(ret, "#nR;106;There is a typo in variable name for path", P1.TYPO_PATH);
        putDef(ret, "#fR;54;avoid ambitiyy", FP.FP);
        putDef(ret, "#fR;64;Used public", FP.FP);
        putDef(ret, "#mR;60;Method name should not start with underscore and method name starts with small case is recommonded", FP.FP);
        putDef(ret, "#iR;50;Method name should not start with underscore, starting with small case is recommended", FP.FP);
        putDef(ret, "#iR;52;Exception has been handled, but user understadable exception didn't thrown", FP.FP);
        putDef(ret, "#aR;728;Exception has been handled but user understable exception not thrown", FP.FP);
        putDef(ret, "#cR;806;shoud use comments for that user understandable", FP.FP);
        putDef(ret, "#hR;41;should also use logger there", FP.FP);
        putDef(ret, "#hR;42;should use approperite message to understands", FP.FP);
        putDef(ret, "#hR;52;user proper error message or throw exception", FP.FP);
        putDef(ret, "#mR;74;should use proper exception msg here", FP.FP);
        putDef(ret, "#nR;106;should use \"path\" as proper text for toString", P1.TYPO_PATH);
        putDef(ret, "#aR;721;should use javadoc and proper commnets", FP.FP);
        putDef(ret, "#aR;729;should use proper exception message and print logger information", P1.INTERRUPTED_EXCEPTION);
        putDef(ret, "#eR;40;removed abstractBrowserTask parameterized constructor", FP.FP);
        putDef(ret, "#bR;769;updated rename code1", FP.FP);
        putDef(ret, "#bR;776;updated rename code2", FP.FP);
        putDef(ret, "#cR;801;updated rename code3", FP.FP);
        putDef(ret, "#cR;808;updated rename code 4", FP.FP);
        putDef(ret, "#dR;847;formed make dir code to new style", FP.FP);
        putDef(ret, "#dR;871;finished make dir code according to above", FP.FP);
        putDef(ret, "#fR;46;updated paths param info to consolidated path", FP.FP);
        putDef(ret, "#fR;54;changed path param as per new path style", FP.FP);
        putDef(ret, "#fR;64;updated paths declaration as per new style", FP.FP);
        putDef(ret, "#oR;1;removed BrowserIORequest stuff", FP.FP);
        putDef(ret, "#hR;21;added DeleteBrowserTask Stuffs", FP.FP);
        putDef(ret, "#hR;38;added DeleteBrowserTask stuffs2", FP.FP);
        putDef(ret, "#kR;42;new declared loadInfo array as private class variable", FP.FP);


        /**
         * 3rd batch
         */

        putDef(ret, "#kR;50;renamed path var as per new style", FP.FP);
        putDef(ret, "#kR;56;updated path param name", FP.FP);
        putDef(ret, "#kR;58;updated path param name", FP.FP);
        putDef(ret, "#mR;61;renamed path var according to new declr style", FP.FP);
        putDef(ret, "#mR;64;renamed path var according to new declr style", FP.FP);
        putDef(ret, "#mR;70;renamed path var according to new declr style", FP.FP);
        putDef(ret, "#mR;77;renamed path var according to new declr style", FP.FP);
        putDef(ret, "#mR;87;renamed path var according to new declr style", FP.FP);
        putDef(ret, "#nR;106;renamed path var according to new declr style", FP.FP);
        putDef(ret, "#iR;1;moved MkDirBrowserTask ro package browser", FP.FP);
        putDef(ret, "#iR;3;added MkDirBrowserTask code", FP.FP);
        putDef(ret, "#jR;1;moved RenameBrowserTask to pkg browser", FP.FP);
        putDef(ret, "#jR;4;added RenameBrowserTask code", FP.FP);
        putDef(ret, "#aR;703;Optimized the loop", FP.FP);
        putDef(ret, "#aR;711;Updated subtask delete code as per New Task based guideline", FP.FP);
        putDef(ret, "#aR;732;updated endRequest call even according to new approach", FP.FP);
        putDef(ret, "#hR;28;error logs and logs are not proper", FP.FP);
        putDef(ret, "#hR;34;check exception", FP.FP);
        putDef(ret, "#jR;37;add loggers", FP.FP);
        putDef(ret, "#eR;40;remove code", FP.FP);
        putDef(ret, "#bR;768;change code", FP.FP);
        putDef(ret, "#bR;774;change code", FP.FP);
        putDef(ret, "#cR;802;change code", FP.FP);
        putDef(ret, "#cR;808;change code", FP.FP);
        putDef(ret, "#dR;848;change code", FP.FP);
        putDef(ret, "#dR;870;change code", FP.FP);
        putDef(ret, "#fR;46;change code", FP.FP);
        putDef(ret, "#fR;50;change code", FP.FP);
        putDef(ret, "#fR;54;change code", FP.FP);
        putDef(ret, "#fR;64;change code", FP.FP);
        putDef(ret, "#oR;1;remove code", FP.FP);
        putDef(ret, "#hR;1;add code", FP.FP);
        putDef(ret, "#kR;42;add code", FP.FP);
        putDef(ret, "#kR;50;change code", FP.FP);
        putDef(ret, "#kR;56;change code", FP.FP);
        putDef(ret, "#kR;58;change code", FP.FP);
        putDef(ret, "#mR;61;change code", FP.FP);
        putDef(ret, "#mR;64;change code", FP.FP);
        putDef(ret, "#mR;70;change code", FP.FP);
        putDef(ret, "#mR;77;change code", FP.FP);
        putDef(ret, "#mR;87;change code", FP.FP);
        putDef(ret, "#nR;106;change code", FP.FP);
        putDef(ret, "#iR;1;add code", FP.FP);
        putDef(ret, "#iR;20;add code", FP.FP);
        putDef(ret, "#jR;1;add code", FP.FP);
        putDef(ret, "#jR;5;add code", FP.FP);
        putDef(ret, "#aR;703;add code", FP.FP);

        //end of 3rd batch

        putDef(ret, "#aR;712;change code", FP.FP);
        putDef(ret, "#aR;733;change code", FP.FP);
        putDef(ret, "#bR;768;use single abstract method style", FP.FP);

        /*
        E - second 30 developers
         */

        putDef(ret, "#bR;768;instead of anonymous inner class we can take lambda expression to reduce line of code", FP.FP);
        putDef(ret, "#cR;801;instead of anonymous inner class we can take lambda expression to reduce line of code", FP.FP);
        putDef(ret, "#dR;847;instead of anonymous inner class we can take lambda expression to reduce line of code", FP.FP);
        putDef(ret, "#jR;37;its recommended to start method name with small case .not recommended to start with underscore", FP.FP);
        putDef(ret, "#jR;60;its recommended to start method name with small case .not recommended to start with underscore", FP.FP);
        putDef(ret, "#jR;75;exception is handled but not thrown user understandable exception", FP.FP);
        putDef(ret, "#aR;713;instead of anonymous inner class we can take lambda expression to reduce line of code", FP.FP);
        putDef(ret, "#aR;732;instead of anonymous inner class we can take lambda expression to reduce line of code", FP.FP);
        putDef(ret, "#bR;768;Instead of anonymous inner class we can take lambda expression to reduce line of code", FP.FP);
        putDef(ret, "#cR;801;Instead of anonymous inner class we can take lambda expression to reduce line of code", FP.FP);
        putDef(ret, "#dR;847;Instead of anonymous inner class we can take lambda expression to reduce line of code", FP.FP);
        putDef(ret, "#hR;34;its recommended to start variable name with small case .not recommended to start with underscore", FP.FP);
        putDef(ret, "#hR;50;exception is handled but not thrown user understandable exception", FP.FP);
        putDef(ret, "#iR;36;its recommended to start variable name with small case .not recommended to start with underscore", FP.FP);
        putDef(ret, "#iR;44;exception is handled but not thrown user understandable exception", FP.FP);
        putDef(ret, "#jR;46;its recommended to start variable name with small case .not recommended to start with underscore", FP.FP);
        putDef(ret, "#jR;60;its recommended to start variable name with small case .not recommended to start with underscore", FP.FP);
        putDef(ret, "#jR;73;its recommended to start variable name with small case .not recommended to start with underscore", FP.FP);
        putDef(ret, "#aR;728;exception is handled but not thrown user understandable exception", INTERRUPTED_EXCEPTION);
        putDef(ret, "#hR;16;Variables are not protected", FP.FP);
        putDef(ret, "#hR;40;Only IOExceptons are handled, Exceptin clause and null check shuld be there", FP.FP);
        putDef(ret, "#iR;24;paramater check is not happening in constructor", FP.FP);
        putDef(ret, "#iR;30;Its a void method , it doesn't return any value. constructor will not return value", FP.FP);
        putDef(ret, "#jR;17;no use of getter and setter methods in class", FP.FP);
        putDef(ret, "#jR;37;void doesn't return any value", FP.FP);
        putDef(ret, "#jR;65;Include exception catch block", FP.FP);
        putDef(ret, "#cR;786;use always equals method to compare two strings", FP.FP);
        putDef(ret, "#fR;61;please use this variable declarations at the start of method", FP.FP);
        putDef(ret, "#fR;62;please use this variable declarations at the start of method", FP.FP);
        putDef(ret, "#fR;63;please use this variable declarations at the start of method", FP.FP);
        putDef(ret, "#fR;64;please use this variable declarations at the start of method", FP.FP);
        putDef(ret, "#hR;28;dont use special caracter for the method name.", FP.FP);
        putDef(ret, "#hR;36;dont use special caracter for the method name, Dont us underscore here.", FP.FP);
        putDef(ret, "#hR;41;use log.error here in catch block", FP.FP);
        putDef(ret, "#nR;106;use proper naming path here.", TYPO_PATH);
        putDef(ret, "#iR;30;Dont use underScore in method name", FP.FP);
        putDef(ret, "#iR;38;Dont use underScore in method name", FP.FP);
        putDef(ret, "#iR;44;use log.error in catch nlock", FP.FP);
        putDef(ret, "#jR;37;Dont use underScore in method name", FP.FP);
        putDef(ret, "#jR;43;Dont use underScore in method name", FP.FP);
        putDef(ret, "#jR;44;Dont use underScore in method name", FP.FP);
        putDef(ret, "#jR;46;Dont use underScore in method name", FP.FP);
        putDef(ret, "#jR;55;define string array properly", FP.FP);
        putDef(ret, "#jR;60;Dont use underScore in method name", FP.FP);
        putDef(ret, "#jR;67;use log.error in catch block", FP.FP);
        putDef(ret, "#jR;77;use log.error in catch block", FP.FP);
        putDef(ret, "#aR;729;use log.error in catch block", INTERRUPTED_EXCEPTION);
        putDef(ret, "#iR;30;underscore not use", FP.FP);
        putDef(ret, "#hR;59;override is not required", FP.FP);
        putDef(ret, "#nR;106;pat replaced by path", TYPO_PATH);
        putDef(ret, "#fR;61;All these fields should be before constructor", FP.FP);
        putDef(ret, "#hR;30;don't call setCancellable(false) thrice call it before try block", FP.FP);
        putDef(ret, "#hR;31;change name to paths instead of args", FP.FP);
        putDef(ret, "#kR;57;remove unnecessary loadInfo field from constructor as it in not used in constructor", LOADINFO_NOT_SET);
        putDef(ret, "#mR;61;change args to paths", FP.FP);
        putDef(ret, "#mR;64;what is the use of giving value of path to canonPath", FP.FP);
        putDef(ret, "#nR;106;change pat toh path", TYPO_PATH);
        putDef(ret, "#iR;63;change DELETE to MKDIR", WRONG_TOSTRING_MKDIR);
        putDef(ret, "#jR;30;remove uneccessary path2 var from constructor param", FP.FP);
        putDef(ret, "#hR;30;don't call setCancellable(false) twice", CANCELLABLE_WRONG);
        putDef(ret, "#jR;48;condition is wrong it should be !path.equalsIgnoreCase(path2)", FP.FP);
        putDef(ret, "#bR;768;we can use lamada intead of annonymus inner class", FP.FP);
        putDef(ret, "#cR;801;we can use lamada instead of inner class to reduce line of code", FP.FP);
        putDef(ret, "#dR;847;we can take lamda expression istead of annonymous inner class to reduce lines of code", FP.FP);
        putDef(ret, "#bR;768;we can use lamada intead of annonymus inner class", FP.FP);
        putDef(ret, "#hR;28;method name can be start with small case instead of _", FP.FP);
        putDef(ret, "#hR;34;method name can be start with small case instead of _", FP.FP);
        putDef(ret, "#hR;40;user friendly exception is not thrown", FP.FP);
        putDef(ret, "#hR;50;user friendly exception is not thrown", FP.FP);
        putDef(ret, "#mR;70;mehod name can be start with small case instead of _", FP.FP);
        putDef(ret, "#mR;73;user friendly exception is not thrown", FP.FP);
        putDef(ret, "#mR;81;mehod name can be start with small case instead of _", FP.FP);
        putDef(ret, "#nR;106;to string method is not properly overriden", FP.FP);
        putDef(ret, "#iR;30;mehod name can be start with small case but not with _", FP.FP);
        putDef(ret, "#iR;42;user friendly messege is not thrown", FP.FP);
        putDef(ret, "#iR;52;user understanble exception is not thrown", FP.FP);
        putDef(ret, "#jR;37;it is recomnded to start method name with  small case but not with _", FP.FP);
        putDef(ret, "#jR;65;user understanble exception is not thown", FP.FP);
        putDef(ret, "#jR;75;user understandable exception is not thrown", FP.FP);
        putDef(ret, "#aR;713;we can use lamada expression here instead of using annoymous claass", FP.FP);
        putDef(ret, "#aR;728;exception is catched but  not thorwn   user understandable exception", INTERRUPTED_EXCEPTION);
        putDef(ret, "#aR;732;we can use lamada insted of using annonymous class", FP.FP);
        putDef(ret, "#fR;64;For setting the path variable added.", FP.FP);
        putDef(ret, "#hR;13;Added some method in this class", FP.FP);
        putDef(ret, "#hR;29;run() method is added", FP.FP);
        putDef(ret, "#hR;24;here null passed", FP.FP);
        putDef(ret, "#hR;36;Not need to check", FP.FP);
        putDef(ret, "#mR;66;loadInfo should be pass", FP.FP);
        putDef(ret, "#nR;106;Path=\" + path+  \" like", FP.FP);
        putDef(ret, "#jR;87;Path1", FP.FP);
        putDef(ret, "#aR;699;not needed", FP.FP);
        putDef(ret, "#aR;703;this varible not use", FP.FP);

    /*
        P - second 30 developers
         */

        putDef(ret, "#nR;106;,pat= should be ,path=", P1.TYPO_PATH);
        putDef(ret, "#fR;56;Class missing", FP.FP);
        putDef(ret, "#hR;34;path Null check missing", FP.FP);
        putDef(ret, "#hR;60;browser,session,loadinfo missing", FP.FP);
        putDef(ret, "#mR;93;null check and size check missing", FP.FP);
        putDef(ret, "#nR;104;No all args are not used in this", FP.FP);
        putDef(ret, "#nR;105;No it doesn't", FP.FP);
        putDef(ret, "#nR;106;pat spelled incorrectly \"path=\" ...", P1.TYPO_PATH);
        putDef(ret, "#iR;62;Not in this", FP.FP);
        putDef(ret, "#iR;63;type should not be Delete for MKDir", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#jR;29;path1 and path2 are assigned same values", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;50;code missing", FP.FP);
        putDef(ret, "#aR;712;Not check for null directly accessed", FP.FP);
        putDef(ret, "#jR;33;this.path2 = path2;", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;43;path1 = vfs._canonPath(session, path1, browser);", FP.FP);
        putDef(ret, "#jR;53;VFSManager.error(browser, path1,", FP.FP);
        putDef(ret, "#jR;60;if (!vfs._rename(session, path1, path2, browser)) {", FP.FP);
        putDef(ret, "#jR;87;+ \",vfs=\" + vfs + \",path1=\" + path1", FP.FP);
        putDef(ret, "#nR;106;path spelling was not correct", P1.TYPO_PATH);
        putDef(ret, "#eR;40;removed old AbstractBrowserTask class n constructor", FP.FP);
        putDef(ret, "#bR;768;implemented Runnable in delayedAWT", FP.FP);
        putDef(ret, "#cR;801;implemented Runnable in delayedAWT", FP.FP);
        putDef(ret, "#dR;847;implemented Runnable in delayedAWT", FP.FP);
        putDef(ret, "#dR;869;added MkDirBrowserTask", FP.FP);
        putDef(ret, "#fR;46;updated javadoc for updated path", FP.FP);
        putDef(ret, "#fR;50;updated path param merged", FP.FP);
        putDef(ret, "#fR;54;returned updated path", FP.FP);
        putDef(ret, "#fR;64;updated path related fields", FP.FP);
        putDef(ret, "#oR;1;remove old BrowserIORequest", FP.FP);
        putDef(ret, "#hR;1;created new DeleteBrowserTask", FP.FP);
        putDef(ret, "#kR;42;updated ListDirectoryBrowserTask  for new path param style", FP.FP);
        putDef(ret, "#kR;50;updated javadoc for upd path'", FP.FP);
        putDef(ret, "#kR;56;updated path name in args var", FP.FP);
        putDef(ret, "#kR;58;updated super params", FP.FP);
        putDef(ret, "#mR;61;updated path name in _run method", FP.FP);
        putDef(ret, "#mR;64;updated path name", FP.FP);
        putDef(ret, "#mR;70;updated path name", FP.FP);
        putDef(ret, "#mR;77;updated path name", FP.FP);
        putDef(ret, "#mR;87;updated path name", FP.FP);
        putDef(ret, "#nR;106;updated path name", FP.FP);
        putDef(ret, "#iR;1;created new MkDirBrowserTask", FP.FP);
        putDef(ret, "#jR;1;created new RenameBrowserTask", FP.FP);
        putDef(ret, "#aR;703;created a CountDownLatch", FP.FP);
        putDef(ret, "#aR;711;updated DeleteBrowserTask code", FP.FP);
        putDef(ret, "#aR;732;updated to ue.invok", FP.FP);
        putDef(ret, "#kR;58;additional two parameters required", FP.FP);
        putDef(ret, "#nR;106;pat is not defined gets compile error", TYPO_PATH);
        putDef(ret, "#aR;715;code does not fullfill requirement", FP.FP);
        putDef(ret, "#aR;729;incorrect syntax", INTERRUPTED_EXCEPTION);
        putDef(ret, "#fR;41;must implements runnable", FP.FP);
        putDef(ret, "#hR;14;Not full fill operation", FP.FP);
        putDef(ret, "#hR;28;need to properly  sync", FP.FP);
        putDef(ret, "#jR;46;check IOexception", FP.FP);
        putDef(ret, "#aR;713;task must be optimized", FP.FP);
        putDef(ret, "#hR;22;path name must needed", FP.FP);
        putDef(ret, "#hR;3;must import AbstractBrowserTask", FP.FP);
        putDef(ret, "#jR;46;check file availble or not", FP.FP);
        putDef(ret, "#hR;23;there is no null check of strings", FP.FP);
        putDef(ret, "#kR;45;message does not explain what class is doing", FP.FP);
        putDef(ret, "#kR;58;there is no null check andother constraints", FP.FP);
        putDef(ret, "#iR;26;constructor doesnt check for the parameter validity", FP.FP);
        putDef(ret, "#fR;55;we did not given any condition here", FP.FP);


        return ret;
    }
}