package de.unihannover.orderingExperimentAnalysis;

import static de.unihannover.orderingExperimentAnalysis.SessionData.Defect.putDef;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import de.unihannover.orderingExperimentAnalysis.SessionData.Defect;
import de.unihannover.orderingExperimentAnalysis.SessionData.FP;

public enum P2 implements Defect {
    /**
     * Missing check for end < text.length in isWholeWord (hunk q).
     */
    MISSING_CHECK_END,
    /**
     * Missing check for start != 0 in isWholeWord (hunk q).
     */
    MISSING_CHECK_START,
    /**
     * Use this.noWordSep == null instead of noWordSep == null in SearchMatcher.setNoWordSep (hunk o).
     */
    SET_NO_WORD_SEP_WRONG,
    /**
     * Wrongly uses == instead of != in isEndWord (hunk r).
     */
    IS_END_WORD_WRONG_RESULT,
    /**
     * Use of wrong property name "wordBreakChars" in SearchAndReplace._replace (hunk g).
     */
    WRONG_PROPERTY,
    /**
     * Forget setting match.end in BoyerMooreSearchMatcher.nextMatch (hunk m).
     */
    MATCH_END_NOT_SET,
    /**
     * Missing continue in PatternSearchMatcher.nextMatch, leading to wrong checks (hunk j).
     */
    MISSING_CONTINUE,
    /**
     * Typo in Message "Mathc" (hunk t).
     */
    TYPO_MATCH,
    /**
     * Call to wrong constructor of PatternSearchMatcher in SearchAndReplace.getSearchMatcher() (hunk e)
     */
    WHOLEWORDMISSING,
    /**
     * Recursion in BoyerMooreSearchMatcher.nextMatch, can lead to stackoverflow (hunk m).
     */
    DANGEROUS_RECURSION;

    @Override
    public String getShortString() {
        return this.getClass().getSimpleName() + "_" + this.ordinal();
    }

    @Override
    public boolean isDelocalized() {
        return this == WRONG_PROPERTY || this == MATCH_END_NOT_SET || this == WHOLEWORDMISSING;
    }

    static Map<String, List<Defect>> getTrueDefects() {
        final Map<String, List<Defect>> ret = new LinkedHashMap<>();

        //First group of developers - pilot study
        
        
        putDef(ret, "#dR;227;Provided implemetation of getSearchMatcher()", FP.FP);
        putDef(ret, "#jR;175;need to provide continue; after line 175", P2.MISSING_CONTINUE);
        putDef(ret, "#iR;73;Creates a new regular expression already compiled", FP.FP);
        putDef(ret, "#fR;654;String noWordSep = buffer.getStringProperty(\"noWordSep\"); matcher.setNoWordSep(noWordSep);", FP.FP);
        putDef(ret, "#gR;1194; 		String noWordSep = buffer.getStringProperty(\"noWordSep\"); matcher.setNoWordSep(noWordSep);", P2.WRONG_PROPERTY);
        putDef(ret, "#nR;242;String noWordSep = buffer.getStringProperty(\"noWordSep\"); matcher.setNoWordSep(noWordSep);", FP.FP);
        putDef(ret, "#bR;50;Added String variable", FP.FP);
        putDef(ret, "#cR;53;noWordSep parameter is not passed in costructor", FP.FP); 
        putDef(ret, "#cR;57;noWordSep = searchMatcher.getNoWordSep();", FP.FP); 
        putDef(ret, "#pR;75;Added getNoWordSep()", FP.FP);
        putDef(ret, "#pR;75;Added getNoWordSep()", FP.FP);
        putDef(ret, "#rR;109;Added isEndWord() and provided its implementation", FP.FP);
        putDef(ret, "#tR;135;Added toString()", FP.FP); 
        putDef(ret, "#gR;1195;better using toStrng() method from Object class to convert to String type and each method calling like getMode(), getProperty() may be null, hence need to handle that null value before and call those methods.", FP.FP); 
        putDef(ret, "#gR;1196;may get NullPointerException", FP.FP);
        putDef(ret, "#gR;1196;may get NullPointerException", FP.FP);
        putDef(ret, "#cR;57;Need to verify method getNoWordSep()", FP.FP);
        putDef(ret, "#iR;77;May this makes an infinite loop", FP.FP);
        putDef(ret, "#nR;242;Need to verify teh called methods", P2.WRONG_PROPERTY);
        putDef(ret, "#kR;41;same may occur infinite loop, bcz if we create an object for the BoyermooreSearchMatcher class by passing these 3 parameters this constructor will get called inside their is this(pattern,ignoreCase,false) so it calls again the same constructor due the reference stored inside the \"this\" keyword stored is BMSM.java class Object.", FP.FP);
        putDef(ret, "#lR;58;IN one line", FP.FP);
        putDef(ret, "#fR;654;Same here may get NullPointerException, hence handle those method calls.", FP.FP);
        putDef(ret, "#tR;134;Match spelling standard", P2.TYPO_MATCH);
        putDef(ret, "#rR;109;The method is boolean but  nothing is return here like true or false", P2.TYPO_MATCH);
        putDef(ret, "#cR;57;third parameter is not passed", FP.FP);
        putDef(ret, "#iR;77;with the help of this keywork we can called to the another constructor but in that case re is variable , equalignore is method and false is boolean value thats way nothing is called", FP.FP);
        putDef(ret, "#oR;62;It will occur error", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#qR;89;In this line required End-1 instead of start-1", FP.FP);
        putDef(ret, "#aR;709;Implementing method of set enable", FP.FP); 
        putDef(ret, "#dR;227;Implenting this method to search similar word to searh", FP.FP);
        putDef(ret, "#gR;1195;Inthis get property of wordBeakChars", P2.WRONG_PROPERTY); 
        putDef(ret, "#pR;75;with  the help of this method get the string the without seprate string", FP.FP);
        putDef(ret, "#bR;50;No need to create nowordsep instance", FP.FP); 
        putDef(ret, "#mR;91;match.end = match.start + pattern.length;", P2.MATCH_END_NOT_SET); 
        putDef(ret, "#jR;176;continue;", P2.MISSING_CONTINUE); 
        putDef(ret, "#tR;137;this will work-> return \"Match[\" + start + ',' + end + ']'", P2.TYPO_MATCH); 
        putDef(ret, "#pR;76;return noWordSep;", FP.FP); 
        putDef(ret, "#gR;1195; String wordBreakChars variable added to get propery and mode", P2.WRONG_PROPERTY); 
        putDef(ret, "#dR;231;searchMatcher.setNoWordSep(noWordSep);", FP.FP);
        putDef(ret, "#bR;50;String noWordSep variable added", FP.FP); //not significant
        putDef(ret, "#jR;174;after tthat need to write   continue;", P2.MISSING_CONTINUE);
        putDef(ret, "#mR;77;int pos = match( if (pos == -1)", FP.FP);
        putDef(ret, "#eR;330;wholeWord is not passed inside constructor   new PatternSearchMatcher(re, ignoreCase);", P2.WHOLEWORDMISSING);
        putDef(ret, "#fR;655;noWordSep variable is created", FP.FP);
        putDef(ret, "#rR;115;returntype == TextUtilities.WORD_CHAR; is wrong here  we need here !=", FP.FP);
        putDef(ret, "#tR;134;overrided toString() to get content of the object", FP.FP);
        putDef(ret, "#dR;227;muliple comments should be written in  /* */ blog", FP.FP);
        putDef(ret, "#dR;231;we can return directly like, return this.searchMatcher.setNoWordSep(this.noWordSep);", FP.FP); 
        putDef(ret, "#hR;62;Okay", FP.FP);
        putDef(ret, "#eR;334;Okay", FP.FP);
        putDef(ret, "#fR;656;Casting is not required", FP.FP);
        putDef(ret, "#gR;1196;casting is not required", FP.FP);
        putDef(ret, "#nR;243;casting is not required", FP.FP);
        putDef(ret, "#cR;57;no need use this.noWordSep= searchMatch.getNoWordSep(); use only noWordSep=searchMatcher.getNoWordSep(); because noWodSep is not a input parameter of constructor", FP.FP);
        putDef(ret, "#pR;76;we ca return only noWordSep no need to use this.noWordSep;", FP.FP);
        putDef(ret, "#rR;116;we can return directly : - return TextUtilities.WORD_CHAR != TextUtilities.getCharType(next, noWordSep);", FP.FP); 
        putDef(ret, "#mR;83;variable name should not be start from underscore", FP.FP);
        putDef(ret, "#fR;656;avoid typecasting, toString() or any other method should be preferred", FP.FP); 
        putDef(ret, "#pR;75;if we have to access that method outside the package then we should decleare method with access modifier.", FP.FP); 
        putDef(ret, "#nR;243;Avoid Typecasting , we can also use toString() method.", FP.FP);
        putDef(ret, "#fR;658;Make searchMatcher interrupttible, let SearchMatcher.nextMatch() throw InterruptedException", FP.FP); 
        putDef(ret, "#lR;65;prefer not yo use _ in variable declaration", FP.FP);
        putDef(ret, "#gR;1196;Casting is not required", FP.FP);
        putDef(ret, "#aR;710;Okay", FP.FP);
        putDef(ret, "#dR;228;multi line comments should be /* */ block", FP.FP);
        putDef(ret, "#bR;50;Okay", FP.FP);
        putDef(ret, "#cR;57;Okay", FP.FP);
        putDef(ret, "#hR;61;Okay", FP.FP);
        putDef(ret, "#jR;174;Okay", FP.FP);
        putDef(ret, "#iR;77;Okay", FP.FP);
        putDef(ret, "#nR;243;Casting is not required", FP.FP);
        putDef(ret, "#kR;40;Okay", FP.FP);
        putDef(ret, "#mR;85;Okay", FP.FP);
        putDef(ret, "#eR;333;Okay", FP.FP);
        putDef(ret, "#lR;66;Okay", FP.FP);
        putDef(ret, "#fR;655;Casting is not required", FP.FP);
        putDef(ret, "#rR;110;Okay", FP.FP);
        putDef(ret, "#oR;63;Okay", FP.FP);
        putDef(ret, "#qR;89;Okay", FP.FP);
        putDef(ret, "#uR;127;Okay", FP.FP);
        putDef(ret, "#pR;72;Okay", FP.FP);
        putDef(ret, "#sR;122;Okay", FP.FP);
        putDef(ret, "#tR;136;Okay", FP.FP);
        putDef(ret, "#nR;243;Type casting could have been avoided here", FP.FP);
        
        
        
        // Second group of developers - pilot study
        
        
        putDef(ret, "#qR;87;isWholeWord added", FP.FP);
        putDef(ret, "#kR;35;BoyerMooreSearchMatcher() Added", FP.FP);
        putDef(ret, "#tR;134;toString() added", FP.FP);
        putDef(ret, "#dR;227;documention", FP.FP);
        putDef(ret, "#pR;75;getNoWordSep() added", FP.FP);
        putDef(ret, "#oR;60;setNoWordSep() added", FP.FP);
        putDef(ret, "#uR;129;String noWordSep Added", FP.FP);
        putDef(ret, "#mR;84;Recognizing Extra word characters in \"Whole word\" option in searches", FP.FP);
        putDef(ret, "#fR;656;Please avoid typecasting use toString method or any other mehod preffered.", FP.FP);
        putDef(ret, "#pR;75;Access modifier should be public.when we generate getter,setter method.", FP.FP);
        putDef(ret, "#tR;137;In toString class name spale is incorrect \"Mathc\".It Should \"Match\" in toString().", P2.TYPO_MATCH);
        putDef(ret, "#mR;81;returnValue is not declared and initialized.", FP.FP);
        putDef(ret, "#dR;232;this keyword is not  required here.", FP.FP);
        putDef(ret, "#rR;116;boolean type should return some value from true or false.", FP.FP);
        putDef(ret, "#sR;123;incorrect syntax", FP.FP);
        putDef(ret, "#tR;137;typo mistake", P2.TYPO_MATCH);
        putDef(ret, "#oR;65;not required .. only if condition is enough.", FP.FP);
        putDef(ret, "#qR;90;can write in while loop with the same condition.", FP.FP);
        putDef(ret, "#dR;231;not required to write this keyword.", FP.FP);
        putDef(ret, "#nR;243;Please check for null and then do casting using instaanceOf", FP.FP);
        putDef(ret, "#oR;65;Add condition noWordsep string not be null", FP.FP);
        putDef(ret, "#qR;90;need  some more information about checking words", P2.MISSING_CHECK_START);
        putDef(ret, "#gR;1196;Use toString() method instead", FP.FP);
        putDef(ret, "#jR;171;need some more comments", FP.FP); 
        putDef(ret, "#nR;243;Typecasting without the use of instanceOf should be prohibited.", FP.FP); 
        putDef(ret, "#oR;60;The noWordSep should also be checked.", P2.SET_NO_WORD_SEP_WRONG); 
        putDef(ret, "#dR;232;Use of this keyword is not required here", FP.FP); 
        putDef(ret, "#dR;228;for group comment, must use /* */ block", FP.FP);
        putDef(ret, "#kR;42;not getting clear idea", FP.FP);
        putDef(ret, "#lR;54;no issue found", FP.FP);
        putDef(ret, "#mR;86;no issue found", FP.FP);
        putDef(ret, "#jR;174;no issue found", FP.FP);
        putDef(ret, "#iR;77;no issue found", FP.FP);
        putDef(ret, "#hR;64;no issue found", FP.FP);
        putDef(ret, "#eR;333;no issue found", FP.FP);
        putDef(ret, "#nR;244;Casting is not required", FP.FP);
        putDef(ret, "#bR;50;it is okay, no issue found", FP.FP);
        putDef(ret, "#aR;710;need to verify condition , before setting boolean value", FP.FP);
        putDef(ret, "#cR;57;instead of this.noWordSep = searchMatcher.getNoWordSep(); you can use noWordSep = searchMatcher.getNoWordSep();", FP.FP);
        putDef(ret, "#pR;75;For standard practice, method \"String getNoWordSep()\" should be created with public keyword", FP.FP);
        putDef(ret, "#oR;61;using ternary operator  , you can reduce the code", FP.FP);
        putDef(ret, "#uR;128;no issue found", FP.FP);
        putDef(ret, "#rR;111;within TextUtilities you can write another static , where you can check /compare if (currentCharType != TextUtilities.WORD_CHAR)  and based on it can return the value", FP.FP);
        putDef(ret, "#qR;91;no issue found", FP.FP);
        putDef(ret, "#qR;98;no issue found", FP.FP);
        putDef(ret, "#tR;137;no issue found", FP.FP);
        putDef(ret, "#sR;122;no issue found", FP.FP);
        putDef(ret, "#kR;42;no issue found", FP.FP);
        putDef(ret, "#gR;1195;Casting is not required", FP.FP);
        putDef(ret, "#aR;711;need validate first", FP.FP);
        putDef(ret, "#dR;229;for group comment, must use /* */ block", FP.FP);
        putDef(ret, "#bR;50;okay", FP.FP); 
        putDef(ret, "#hR;64;Okay", FP.FP); 
        putDef(ret, "#mR;87;Okay", FP.FP); 
        putDef(ret, "#eR;335;no issue found", FP.FP); 
        putDef(ret, "#lR;54;Okay", FP.FP); 
        putDef(ret, "#rR;113;within TextUtilities you can write another static , where you 	can check /compare if (currentCharType != TextUtilities.WORD_CHAR)  	and based on it can return the value", FP.FP); 
        putDef(ret, "#oR;62;	using ternary operator  , you can reduce the code", FP.FP); 
        putDef(ret, "#uR;129;okay", FP.FP); 
        putDef(ret, "#pR;74;Okay", FP.FP); 
        putDef(ret, "#sR;123;Okay", FP.FP);
        putDef(ret, "#aR;709;No issues found", FP.FP);
        putDef(ret, "#kR;42;Third parameter is not clear", FP.FP);
        putDef(ret, "#tR;137;Class name is wrong it should be Match", P2.TYPO_MATCH);
        putDef(ret, "#dR;229;Can be written in multiline comment using /* ... */", FP.FP);
        putDef(ret, "#bR;50;No issues found", FP.FP);
        putDef(ret, "#cR;57;Have access to seachMatcher so no need to set this", FP.FP);
        putDef(ret, "#oR;62;No clear idea why _ is written", FP.FP);
        putDef(ret, "#gR;1196;Explicit casting may cause issue , use instaceOf and then toString()", FP.FP);
        putDef(ret, "#aR;713;This line of code can be added before filter.setEnabled as it will resuse the same value", FP.FP);
        putDef(ret, "#nR;243;Issue pointed out before too , please check using instanceOf before explicit casting", FP.FP);
        putDef(ret, "#lR;57;Prefer using foreach", FP.FP);
        putDef(ret, "#fR;656;Explicit casting should not be used without checking is instanceOf", FP.FP);
        putDef(ret, "#oR;60;Check if noWordSep is null or not , if its null it will set this.noWordSep also as null", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#qR;87;Please check if end is greater than start else , result will be undesirable", FP.FP);
        
        
        //Third group of developers - pilot study
        
        
        putDef(ret, "#gR;1196;throws proper related Exception", P2.WRONG_PROPERTY);
        putDef(ret, "#aR;709;need to verify condition , before setting boolean value", FP.FP); 
        putDef(ret, "#dR;229;for group comment, must use /*  */ block", FP.FP);
        putDef(ret, "#bR;50;write seetter and getter for the noWordSep variable", FP.FP); 
        putDef(ret, "#cR;57;write seetter and getter for the noWordSep variable", FP.FP); 
        putDef(ret, "#jR;174;Condition is wrong", P2.MISSING_CONTINUE);
        putDef(ret, "#eR;333;no issue", FP.FP); 
        putDef(ret, "#rR;111;within TextUtilities you can write another static , where you can check /compare if (currentCharType != TextUtilities.WORD_CHAR) and based on it can return the value", FP.FP);
        putDef(ret, "#oR;61;instead of this.noWordSep = searchMatcher.getNoWordSep(); you can use noWordSep = searchMatcher.getNoWordSep();", FP.FP);
        putDef(ret, "#uR;128;For standard practice, method \"String getNoWordSep()\" should be created with public keyword", FP.FP);
        putDef(ret, "#tR;136;no change", FP.FP);
        putDef(ret, "#lR;55;Need a verification of pattern value by using StringUtilities class. "
        		+ "Because while creating an object we  may pass \"NULL\" values too. So cahnce of getting an \"NullPointerException\".  "
        		+ "For instance, as in SearchAndReplacer class created an Object for BoyerMooreSearchMatcher class. And condition for passing first argument is "
        		+ "like if(search == null || \"\".equals(search)){return null;}. So here their is an chance of getting \"NullPointerException\".", FP.FP);
        putDef(ret, "#oR;61;Add this statement too : if(this.noWordSep == null && this.noWordSep.isEmpty()){this.noWordSep = \"_\"}", FP.FP);
        putDef(ret, "#oR;62;No need of flower brackets, when we are writing one statement inside the if or else blocks as you know.", FP.FP);
        putDef(ret, "#qR;89;Here, charAt(start), start can be 0(zero) index also. So it throws an  "
        		+ "exception as \"java.lang.StringIndexOutOfBoundsException\" when we check for charAt(start - 1). So have to do verify for \"prevChar\" value.", P2.MISSING_CHECK_START);
        putDef(ret, "#fR;656;Avoid typecasting instead of this used toString() or any other method.", FP.FP);
        putDef(ret, "#gR;1196;Avoid typecasting,instead of this used toString() or any other method.", FP.FP);
        putDef(ret, "#nR;243;Avoid typecasting,instead of this use toString().", FP.FP);
        putDef(ret, "#pR;75;Access modifiier should be public of getNoWordSep() to acess anywhere if it is required to access.make it public.", FP.FP);
        putDef(ret, "#tR;137;Naming convension should be correct.please correct Mathc to Match for clear understanding.", P2.TYPO_MATCH);
        putDef(ret, "#oR;60;noWordSep should be checked if it is null or not.", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;60;noWordSep variable should be checked for whether it's null or not", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#mR;91;before returning control, 'end' should update.", P2.MATCH_END_NOT_SET);
        putDef(ret, "#gR;1196;instaed of type casting we should use .toString() and if typecating is there we should use instaceof checker.", FP.FP);
        putDef(ret, "#nR;243;instaed of type casting we should use .toString() and if typecating is there we should use instaceof checker.", FP.FP);
        putDef(ret, "#oR;65;null check is missing for else condition.", FP.FP);
        putDef(ret, "#qR;88;'start' can be 0, so there should be some checks on 'start' and 'end' parameters.", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;88;'start' can be 0, so there should be some checks on 'start' and 'end' parameters.", P2.MISSING_CHECK_END);
        putDef(ret, "#aR;710;okay", FP.FP);
        putDef(ret, "#cR;57;okay", FP.FP);
        putDef(ret, "#hR;62;okay", FP.FP);
        putDef(ret, "#jR;174;okay", FP.FP);
        putDef(ret, "#iR;77;okay", FP.FP);
        putDef(ret, "#kR;41;okay", FP.FP);
        putDef(ret, "#mR;86;okay", FP.FP);
        putDef(ret, "#lR;53;okay", FP.FP);
        putDef(ret, "#rR;110;Casting is not required", FP.FP);
        putDef(ret, "#oR;62;okay", FP.FP);
        putDef(ret, "#qR;87;okay", FP.FP);
        putDef(ret, "#uR;128;okay", FP.FP);
        putDef(ret, "#pR;74;okay", FP.FP);
        putDef(ret, "#sR;122;okay", FP.FP);
        putDef(ret, "#tR;137;no issue", FP.FP);



        /**
         * Pilot - internal
         */

        putDef(ret, "#qR;89;check for outofbounds...?", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;94;should be text.charAt(end)", FP.FP);
        putDef(ret, "#qR;95;should be text.charAt(end+1)", FP.FP);
        putDef(ret, "#rR;109;According to the commit message, a word can start or end with a space, but I can't find such implementation.", FP.FP);
        putDef(ret, "#rR;110;getCharType will treat \"_\" as a valid char. Therefore multi-words won't work (?)", FP.FP);
        putDef(ret, "#kR;39;what does  <code>true</code>  mean? should it  be just true?", FP.FP);
        putDef(ret, "#fR;656;treatment missing if  the output of the call cannot be  converted to string", FP.FP);
        putDef(ret, "#gR;1195;treatment missing if  the output of the call cannot be  converted to string", FP.FP);
        putDef(ret, "#qR;89;what if it's the start of the liine? -1?", P2.MISSING_CHECK_START);
        putDef(ret, "#lR;51;?", FP.FP);
        putDef(ret, "#tR;137;typo", P2.TYPO_MATCH);
        putDef(ret, "#mR;83;is thisvariable necessary? why not accessing returnValue.end directly?", FP.FP);
        putDef(ret, "#nR;243;what if it returns null?", FP.FP);
        putDef(ret, "#iR;77;this constructors calls the first one in the chunk above, that calls this one here..  deadlock?", FP.FP);
        putDef(ret, "#cR;57;was method this implemented?", FP.FP);
        putDef(ret, "#gR;1196;Not aware about this property", P2.WRONG_PROPERTY);
        putDef(ret, "#oR;61;not sure the esle condition is necessary", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#qR;87;why passing the full text and 2 indexes instead of just one char sequence from k-1 to n+1", FP.FP);
        putDef(ret, "#qR;90;cursor inverted, but could work too", FP.FP);
        putDef(ret, "#rR;115;I think the method should say if the current character represents the end of a word. However, if the char type in both positions is WORD_CHAR, it returns true. I'm expecting that if I check in the middle of a word, this method would fail.", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#mR;91;match should be updated by the calling of nextMatch.", P2.MATCH_END_NOT_SET);
        putDef(ret, "#dR;231;not sure this works", FP.FP);

        /**
         * first 10 developers
         */

        putDef(ret, "#kR;42;this(pattern, ignoreCase);", FP.FP);
        putDef(ret, "#tR;138;return \"Math[\" + this.start + \",\" + this.end + \"+\",\"+Arrays.toString(substitutions)]\";", P2.TYPO_MATCH);
        putDef(ret, "#mR;88;if(match.isNull)", FP.FP);
        putDef(ret, "#cR;53;String searchString, SearchMatcher searchMatcher,String noWordSep)", FP.FP);
        putDef(ret, "#cR;57;this.noWordSep = NoWordSep();", FP.FP);
        putDef(ret, "#iR;76;public PatternSearch(Pattern re, boolean ignoreCase, boolean wholeWord){", FP.FP);
        putDef(ret, "#mR;89;return match;", FP.FP);
        putDef(ret, "#kR;42;need to update function argument list as boolean wholeword in defination", FP.FP);
        putDef(ret, "#qR;87;needt add code for check start with line,end with line,symboll or space", FP.FP);
        putDef(ret, "#tR;129;need to add getorsettor", FP.FP);
        putDef(ret, "#lR;54;update defination as comment", FP.FP);
        putDef(ret, "#mR;84;need to check start and end condition", FP.FP);
        putDef(ret, "#lR;54;wholeWord has been changed", FP.FP);
        putDef(ret, "#lR;66;purpose of wholeWord is not achieved", FP.FP);
        putDef(ret, "#mR;84;remove the boiler plate code", FP.FP);
        putDef(ret, "#cR;50;noWordSep has to be initiaze for getting no word search operations", FP.FP);
        putDef(ret, "#iR;77;best way to write the code", FP.FP);
        putDef(ret, "#eR;333;removing boiler plate code", FP.FP);
        putDef(ret, "#tR;138;return \"A [start=\" + start + \", end=\" + end + \", substitutions=\" + Arrays.toString(substitutions) + \"]\";", P2.TYPO_MATCH);
        putDef(ret, "#cR;57;this.noWordSep = searchMatcher;", FP.FP);
        putDef(ret, "#iR;76;public PatternSearchMatcher(Pattern re, boolean ignoreCase,boolean wholeWord) {", FP.FP);
        putDef(ret, "#gR;1190;not checking", FP.FP);
        putDef(ret, "#qR;90;not always", P2.MISSING_CHECK_START);
        putDef(ret, "#tR;134;Not all used", FP.FP);
        putDef(ret, "#lR;53;not implements anything", FP.FP);
        putDef(ret, "#lR;55;Not used", FP.FP);
        putDef(ret, "#lR;56;not checked", FP.FP);
        putDef(ret, "#lR;57;All parameters are not used", FP.FP);
        putDef(ret, "#lR;58;not used", FP.FP);
        putDef(ret, "#mR;75;not", FP.FP);
        putDef(ret, "#mR;77;not always", FP.FP);
        putDef(ret, "#mR;84;not", FP.FP);
        putDef(ret, "#cR;46;Not implemented anythings", FP.FP);
        putDef(ret, "#nR;240;Not checking validity", FP.FP);
        putDef(ret, "#hR;62;Not needed", FP.FP);
        putDef(ret, "#eR;317;not always", FP.FP);
        putDef(ret, "#tR;138;return \"Match[\" + this.start + \",\" + this.end + \",\" +Arrays.toString(substitutions)+ \"]\";", P2.TYPO_MATCH);
        putDef(ret, "#mR;88;if (match.isNull()) {", FP.FP);
        putDef(ret, "#cR;53;String searchString, SearchMatcher searchMatcher, String noWordSep) {", FP.FP);
        putDef(ret, "#cR;57;this.noWordSep = noWordSep;", FP.FP);
        putDef(ret, "#iR;76;public PatternSearchMatcher(Pattern re, boolean ignoreCase, boolean wholeWord) {", FP.FP);
        putDef(ret, "#tR;138;return \"Match[\" + this.start + \",\" + this.end + \",\" +this.substitutions[]+ \"]\";", P2.TYPO_MATCH);
        putDef(ret, "#cR;50;search word space is required to find space", FP.FP);
        putDef(ret, "#jR;173;it throws an error", P2.MISSING_CONTINUE);

        /**
         * E - first 30 developers
        putDef(ret, "#iR;77;not declared properly", FP.FP);
        putDef(ret, "#jR;173;comparator is not used", FP.FP);
        putDef(ret, "#tR;131;static inner class", FP.FP);
        putDef(ret, "#lR;53;ignore case is true always", FP.FP);
        putDef(ret, "#lR;58;Pattern is upper case due for search", FP.FP);
        putDef(ret, "#hR;64;Calling an constructor with two parameter with Pattern and ignorecase", FP.FP);
        putDef(ret, "#iR;77;whole word is false", FP.FP);
        putDef(ret, "#tR;131;class should not be public", FP.FP);
        putDef(ret, "#pR;77;return noWordSep;", FP.FP);
        putDef(ret, "#cR;53;String searchString, SearchMatcher searchMatcher,String noWordSep) {", FP.FP);
        putDef(ret, "#hR;64;this.ignoreCase=ignoreCase;", FP.FP);
        putDef(ret, "#iR;77;this(re, ignoreCase);", FP.FP);
        putDef(ret, "#lR;66;wholeWord is assigned but no code for searching the word", FP.FP);
        putDef(ret, "#mR;84;here there may be data discripancy du to end and _end variable", FP.FP);
        putDef(ret, "#mR;87;firstTime    and reverse are not used", FP.FP);
        putDef(ret, "#pR;78;implements this with parents", FP.FP);
        putDef(ret, "#rR;110;next is not used", FP.FP);
        putDef(ret, "#tR;129;It may return different value as per users input", FP.FP);
        putDef(ret, "#lR;54;need to insert equalsIgnoreCase", FP.FP);
        putDef(ret, "#cR;50;we can't return private value", FP.FP);
        putDef(ret, "#eR;334;equalsIgnoreCase need to replace with ignoreCase", FP.FP);
        putDef(ret, "#fR;654;usse String.valueOf", FP.FP);
        putDef(ret, "#qR;89;exception", P2.MISSING_CHECK_START);
        putDef(ret, "#rR;111;conditions doesnt make sense", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#tR;138;Match spelling wrong", P2.TYPO_MATCH);
        putDef(ret, "#lR;55;Could simply make to string upper case and then convert to charArray", FP.FP);
        putDef(ret, "#mR;81;returnValue isnt defined", FP.FP);
        putDef(ret, "#mR;83;should not use _ in naming variables", FP.FP);
        putDef(ret, "#cR;50;unnecessary", FP.FP);
        putDef(ret, "#cR;57;unnecessary", FP.FP);
        putDef(ret, "#dR;231;unnecessary", FP.FP);
        putDef(ret, "#nR;243;unchecked typecasting should use String.valueOf();", FP.FP);
        putDef(ret, "#jR;172;variable naming convention not followed", FP.FP);
        putDef(ret, "#tR;137;There is a typo in retrun string", P2.TYPO_MATCH);
        putDef(ret, "#mR;94;should return value of type SearchMatcher.Match", FP.FP);
        putDef(ret, "#mR;83;Varible name starts with a small letter is recommended", FP.FP);
        putDef(ret, "#kR;36;This javadoc is not meant for this constructor. need to change", FP.FP);
        putDef(ret, "#kR;41;Here is the issue in constructor chaining", FP.FP);
        putDef(ret, "#kR;42;here first just this constructor with having pattern and ignoreCase paramerters an initialized them.", FP.FP);
        putDef(ret, "#fR;657;globle variable should addressing with this", FP.FP);
        putDef(ret, "#lR;53;For this we can initialize the all  three parameter in that constructor and create a different method for this written funcationality.", FP.FP);
        putDef(ret, "#lR;54;here you can use above mention constructor for initializing the two paramerter like this(pattern,ignoreCase) and initialize the other parametre like we normally do.", FP.FP);
        putDef(ret, "#lR;56;remove this peace of code and create a public method and write there. method would be like public string searchMacther(string pattern, boolean ignoreCase)", FP.FP);
        putDef(ret, "#cR;57;in this construtor should have this parameter", FP.FP);

         third batch for inter-rater agreement


        putDef(ret, "#dR;231;should use \"this\" for addressing globle variable", FP.FP);
        putDef(ret, "#nR;242;its better if this local varible have different name to avoid ambiguity", FP.FP);
        putDef(ret, "#nR;244;this noWordSep is a globle variable so it should be addressing with \"this\"", FP.FP);
        putDef(ret, "#hR;62;should used proper contructor chaining here", FP.FP);
        putDef(ret, "#jR;174;instead return null we can print error message or somithig, so we can have idea  what is happing when we see loges or debugging", FP.FP);
        putDef(ret, "#jR;175;here also task mention for true condition and print some message aslo", FP.FP); //discuss
        putDef(ret, "#kR;35;added parameterized constructor to support caseSensitive seach allow/deny feature", FP.FP);
        putDef(ret, "#fR;655;added noWordSep to matcher", FP.FP);
        putDef(ret, "#gR;1196;added wordBreakChars to matcher", FP.FP);
        putDef(ret, "#aR;709;removed unnecessary code lines", FP.FP);
        putDef(ret, "#oR;60;added setNoWordSep to SearchMatcher", FP.FP);
        putDef(ret, "#pR;74;added getNoWordSep to SearchMatcher", FP.FP);
        putDef(ret, "#qR;85;added isWholeWord function to SearchMatcher", FP.FP);
        putDef(ret, "#qR;94;added isWholeWord function to SearchMatcher", FP.FP);
        putDef(ret, "#rR;110;added isEndWord to SearchMatcher", FP.FP);
        putDef(ret, "#rR;116;added isEndWord to SearchMatcher", FP.FP);
        putDef(ret, "#tR;126;added wholeWord, noWordSep to SearchMatcher", FP.FP);
        putDef(ret, "#tR;137;updated toString method of SearchMatcher", FP.FP);
        putDef(ret, "#lR;51;wholeWord param decl", FP.FP);
        putDef(ret, "#lR;54;wholeWord added to constructor param", FP.FP);
        putDef(ret, "#lR;66;wholeWord added to constructor return", FP.FP);
        putDef(ret, "#mR;88;added code to support pattern  matching", FP.FP);
        putDef(ret, "#cR;50;noWordSep added in constructor to support HyperSearch", FP.FP);
        putDef(ret, "#cR;57;noWordSep returned in constructor to support HyperSearch", FP.FP);
        putDef(ret, "#dR;229;noWordSep added in SearchFn to support HyperSearch", FP.FP);
        putDef(ret, "#nR;243;updated HyperSearchRequest to support HyperSearch feature", FP.FP);
        putDef(ret, "#hR;58;created PatternSearchMatcher constructor", FP.FP);
        putDef(ret, "#iR;78;updated PatternSearchMatcher constructor", FP.FP);
        putDef(ret, "#jR;174;implemented nextMatch in PatternSearchMatcher", FP.FP);
        putDef(ret, "#eR;331;removed previous manual search options", FP.FP);
        putDef(ret, "#eR;333;updated BoyerMooreSearchMatcher instanciation", FP.FP);
        putDef(ret, "#cR;50;noWordSep added to support HyperSearch", FP.FP);
        putDef(ret, "#mR;72;syntax", FP.FP);
        putDef(ret, "#lR;54;wholeword must be string", FP.FP);

        putDef(ret, "#kR;37;add code", FP.FP);
        putDef(ret, "#fR;655;add code", FP.FP);
        putDef(ret, "#gR;1195;add code", FP.FP);
        putDef(ret, "#aR;710;add code", FP.FP);
        putDef(ret, "#oR;62;add code", FP.FP);
        putDef(ret, "#pR;71;add code", FP.FP);
        putDef(ret, "#qR;83;add code", FP.FP);
        putDef(ret, "#rR;111;add code", FP.FP);
        putDef(ret, "#rR;117;add code", FP.FP);
        putDef(ret, "#tR;125;add code", FP.FP);
        putDef(ret, "#tR;136;add code", FP.FP);
        putDef(ret, "#lR;50;add code", FP.FP);
        putDef(ret, "#lR;53;change code", FP.FP);
        putDef(ret, "#lR;66;add code", FP.FP);
        putDef(ret, "#mR;84;add code", FP.FP);
        putDef(ret, "#cR;49;add code", FP.FP);
        putDef(ret, "#cR;57;add code", FP.FP);
        putDef(ret, "#dR;228;add code", FP.FP);
        putDef(ret, "#nR;243;add code", FP.FP);
        putDef(ret, "#hR;54;add code", FP.FP);
        putDef(ret, "#iR;77;add code", FP.FP);
        putDef(ret, "#jR;173;add code", FP.FP);
        putDef(ret, "#eR;334;add code", FP.FP);
        putDef(ret, "#lR;56;if ignore case is true or false what is the use of this condition", FP.FP);
        */



        /**
         * Final version - first 30 developers
         * 1st batch
         */

        putDef(ret, "#iR;77;not declared properly", FP.FP);
        putDef(ret, "#jR;173;comparator is not used", FP.FP);
        putDef(ret, "#tR;131;static inner class", FP.FP);
        putDef(ret, "#lR;53;ignore case is true always", FP.FP);
        putDef(ret, "#lR;58;Pattern is upper case due for search", FP.FP);
        putDef(ret, "#hR;64;Calling an constructor with two parameter with Pattern and ignorecase", FP.FP);
        putDef(ret, "#iR;77;whole word is false", FP.FP);
        putDef(ret, "#tR;131;class should not be public", FP.FP);
        putDef(ret, "#pR;77;return noWordSep;", FP.FP);
        putDef(ret, "#cR;53;String searchString, SearchMatcher searchMatcher,String noWordSep) {", FP.FP);
        putDef(ret, "#hR;64;this.ignoreCase=ignoreCase;", FP.FP);
        putDef(ret, "#iR;77;this(re, ignoreCase);", FP.FP);
        putDef(ret, "#lR;66;wholeWord is assigned but no code for searching the word", FP.FP);
        putDef(ret, "#mR;84;here there may be data discripancy du to end and _end variable", FP.FP);
        putDef(ret, "#mR;87;firstTime    and reverse are not used", FP.FP);
        putDef(ret, "#pR;78;implements this with parents", FP.FP);
        putDef(ret, "#rR;110;next is not used", FP.FP);
        putDef(ret, "#tR;129;It may return different value as per users input", FP.FP);
        putDef(ret, "#lR;54;need to insert equalsIgnoreCase", FP.FP);
        putDef(ret, "#cR;50;we can't return private value", FP.FP);
        putDef(ret, "#eR;334;equalsIgnoreCase need to replace with ignoreCase", FP.FP);
        putDef(ret, "#fR;654;usse String.valueOf", FP.FP);
        putDef(ret, "#qR;89;exception", P2.MISSING_CHECK_START);
        putDef(ret, "#rR;111;conditions doesnt make sense", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#tR;138;Match spelling wrong", P2.TYPO_MATCH);
        putDef(ret, "#lR;55;Could simply make to string upper case and then convert to charArray", FP.FP);
        putDef(ret, "#mR;81;returnValue isnt defined", FP.FP);
        putDef(ret, "#mR;83;should not use _ in naming variables", FP.FP);
        putDef(ret, "#cR;50;unnecessary", FP.FP);
        putDef(ret, "#cR;57;unnecessary", FP.FP);
        putDef(ret, "#dR;231;unnecessary", FP.FP);
        putDef(ret, "#nR;243;unchecked typecasting should use String.valueOf();", FP.FP);
        putDef(ret, "#jR;172;variable naming convention not followed", FP.FP);
        putDef(ret, "#tR;137;There is a typo in retrun string", P2.TYPO_MATCH);
        putDef(ret, "#mR;94;should return value of type SearchMatcher.Match", FP.FP);
        putDef(ret, "#mR;83;Varible name starts with a small letter is recommended", FP.FP);
        putDef(ret, "#kR;36;This javadoc is not meant for this constructor. need to change", FP.FP);
        putDef(ret, "#kR;41;Here is the issue in constructor chaining", FP.FP);
        putDef(ret, "#kR;42;here first just this constructor with having pattern and ignoreCase paramerters an initialized them.", FP.FP);
        putDef(ret, "#fR;657;globle variable should addressing with this", FP.FP);
        putDef(ret, "#lR;53;For this we can initialize the all  three parameter in that constructor and create a different method for this written funcationality.", FP.FP);
        putDef(ret, "#lR;54;here you can use above mention constructor for initializing the two paramerter like this(pattern,ignoreCase) and initialize the other parametre like we normally do.", FP.FP);
        putDef(ret, "#lR;56;remove this peace of code and create a public method and write there. method would be like public string searchMacther(string pattern, boolean ignoreCase)", FP.FP);
        putDef(ret, "#cR;57;in this construtor should have this parameter", FP.FP);

        /**
        2nd batch
         */

        putDef(ret, "#dR;231;should use \"this\" for addressing globle variable", FP.FP);
        putDef(ret, "#nR;242;its better if this local varible have different name to avoid ambiguity", FP.FP);
        putDef(ret, "#nR;244;this noWordSep is a globle variable so it should be addressing with \"this\"", FP.FP);
        putDef(ret, "#hR;62;should used proper contructor chaining here", FP.FP);
        putDef(ret, "#jR;174;instead return null we can print error message or somithig, so we can have idea  what is happing when we see loges or debugging", FP.FP);
        putDef(ret, "#jR;175;here also task mention for true condition and print some message aslo", FP.FP);
        putDef(ret, "#kR;35;added parameterized constructor to support caseSensitive seach allow/deny feature", FP.FP);
        putDef(ret, "#fR;655;added noWordSep to matcher", FP.FP);
        putDef(ret, "#gR;1196;added wordBreakChars to matcher", FP.FP);
        putDef(ret, "#aR;709;removed unnecessary code lines", FP.FP);
        putDef(ret, "#oR;60;added setNoWordSep to SearchMatcher", FP.FP);
        putDef(ret, "#pR;74;added getNoWordSep to SearchMatcher", FP.FP);
        putDef(ret, "#qR;85;added isWholeWord function to SearchMatcher", FP.FP);
        putDef(ret, "#qR;94;added isWholeWord function to SearchMatcher", FP.FP);
        putDef(ret, "#rR;110;added isEndWord to SearchMatcher", FP.FP);
        putDef(ret, "#rR;116;added isEndWord to SearchMatcher", FP.FP);
        putDef(ret, "#tR;126;added wholeWord, noWordSep to SearchMatcher", FP.FP);
        putDef(ret, "#tR;137;updated toString method of SearchMatcher", FP.FP);
        putDef(ret, "#lR;51;wholeWord param decl", FP.FP);
        putDef(ret, "#lR;54;wholeWord added to constructor param", FP.FP);
        putDef(ret, "#lR;66;wholeWord added to constructor return", FP.FP);
        putDef(ret, "#mR;88;added code to support pattern  matching", FP.FP);
        putDef(ret, "#cR;50;noWordSep added in constructor to support HyperSearch", FP.FP);
        putDef(ret, "#cR;57;noWordSep returned in constructor to support HyperSearch", FP.FP);
        putDef(ret, "#dR;229;noWordSep added in SearchFn to support HyperSearch", FP.FP);
        putDef(ret, "#nR;243;updated HyperSearchRequest to support HyperSearch feature", FP.FP);
        putDef(ret, "#hR;58;created PatternSearchMatcher constructor", FP.FP);
        putDef(ret, "#iR;78;updated PatternSearchMatcher constructor", FP.FP);
        putDef(ret, "#jR;174;implemented nextMatch in PatternSearchMatcher", FP.FP);
        putDef(ret, "#eR;331;removed previous manual search options", FP.FP);
        putDef(ret, "#eR;333;updated BoyerMooreSearchMatcher instanciation", FP.FP);
        putDef(ret, "#cR;50;noWordSep added to support HyperSearch", FP.FP);
        putDef(ret, "#mR;72;syntax", FP.FP);
        putDef(ret, "#lR;54;wholeword must be string", FP.FP);
        putDef(ret, "#kR;37;add code", FP.FP);
        putDef(ret, "#fR;655;add code", FP.FP);
        putDef(ret, "#gR;1195;add code", FP.FP);
        putDef(ret, "#aR;710;add code", FP.FP);
        putDef(ret, "#oR;62;add code", FP.FP);
        putDef(ret, "#pR;71;add code", FP.FP);
        putDef(ret, "#qR;83;add code", FP.FP);
        putDef(ret, "#rR;111;add code", FP.FP);
        putDef(ret, "#rR;117;add code", FP.FP);
        putDef(ret, "#tR;125;add code", FP.FP);

        /**
         * 3rd batch
         */

        putDef(ret, "#tR;136;add code", FP.FP);
        putDef(ret, "#lR;50;add code", FP.FP);
        putDef(ret, "#lR;53;change code", FP.FP);
        putDef(ret, "#lR;66;add code", FP.FP);
        putDef(ret, "#mR;84;add code", FP.FP);
        putDef(ret, "#cR;49;add code", FP.FP);
        putDef(ret, "#cR;57;add code", FP.FP);
        putDef(ret, "#dR;228;add code", FP.FP);
        putDef(ret, "#nR;243;add code", FP.FP);
        putDef(ret, "#hR;54;add code", FP.FP);
        putDef(ret, "#iR;77;add code", FP.FP);
        putDef(ret, "#jR;173;add code", FP.FP);
        putDef(ret, "#eR;334;add code", FP.FP);
        putDef(ret, "#lR;56;if ignore case is true or false what is the use of this condition", FP.FP);

        /*
        E - second 30 developers
         */

        putDef(ret, "#tR;138;add subtitutions in toString method", FP.FP);
        putDef(ret, "#dR;231;this. is missing from searchMatcher calling setNoWordSep method", FP.FP);
        putDef(ret, "#tR;137;toString() method is not properly overriden", FP.FP);
        putDef(ret, "#lR;65;pattern_end variable is not declared anywhere", FP.FP);
        putDef(ret, "#mR;83;ts recommended to start variable name with small case .not recommended to start with underscore", FP.FP);
        putDef(ret, "#jR;166;its recommended to start variable name with small case .not recommended to start with underscore", FP.FP);
        putDef(ret, "#tR;138;toString() method is not properly overriden.", FP.FP);
        putDef(ret, "#lR;65;This variable(pattern_end) is not declared", FP.FP);
        putDef(ret, "#mR;83;its recommended to start variable name with small case .not recommended to start with underscore", FP.FP);
        putDef(ret, "#jR;167;its recommended to start variable name with small case .not recommended to start with underscore", FP.FP);
        putDef(ret, "#lL;41;since each string lateral is validated in pattern , wholeword is removed", FP.FP);
        putDef(ret, "#lR;50;parameter is included", FP.FP);
        putDef(ret, "#lR;54;boolean parameter is added to the constructor", FP.FP);
        putDef(ret, "#mR;85;Logic implementation  to check characters in the wholeword with eah individual character", FP.FP);
        putDef(ret, "#cR;50;A String literal is declared", FP.FP);
        putDef(ret, "#cR;57;noWordSep is added in the constructor", FP.FP);
        putDef(ret, "#dR;231;      Method calling searches for the  word in sequence and returns count", FP.FP);
        putDef(ret, "#gR;1190;dont use special characters whilce declaring any methodname", FP.FP);
        putDef(ret, "#aR;702;use proper parentesis to understand code (ex- use \"(\")", FP.FP);
        putDef(ret, "#mR;83;Dont use underscore in variable, use normal characters only", FP.FP);
        putDef(ret, "#iR;76;use proper naming for pattern variable", FP.FP);
        putDef(ret, "#jR;172;dont use special character as _ in method paameters", FP.FP);
        putDef(ret, "#fR;651;buffer.getLineOfOffset(start)));", FP.FP);
        putDef(ret, "#gR;1196;Should not be typecast to String", FP.FP);
        putDef(ret, "#gR;1201;buffer.getLineOfOffset(end)) - 1 );", FP.FP);
        putDef(ret, "#pR;77;return noWordSep", FP.FP);
        putDef(ret, "#qR;94;char lastChar = text.charAt(end);", FP.FP);
        putDef(ret, "#qR;95;char nextChar = text.charAt(start+1);", FP.FP);
        putDef(ret, "#tR;138;return \"Mathc[\" + this.start + \",\" + this.end + \",\"+ Arrays.toString(substitutions) \"]\";", FP.FP);
        putDef(ret, "#cR;53;String searchString, SearchMatcher searchMatcher,String noWordSep", FP.FP);
        putDef(ret, "#cR;57;this.noWordSep = noWordSep", FP.FP);
        putDef(ret, "#nR;260;buffer.getLineOfOffset(offset));", FP.FP);
        putDef(ret, "#nR;264;startOfLine,endOfLine,counter,", FP.FP);
        putDef(ret, "#kR;42;there should be this.pattern = pattern.toCharArray();", FP.FP);
        putDef(ret, "#tR;138;matc relpace by match", TYPO_MATCH);
        putDef(ret, "#lR;66;not required", FP.FP);
        putDef(ret, "#mL;63;pos == -1", FP.FP);
        putDef(ret, "#mR;77;pos == -1", FP.FP);
        putDef(ret, "#lR;54;We cannot used boolean as a mothod name", FP.FP);
        putDef(ret, "#mR;84;instead of anonymous inner class we can take lambda expression to reduce line of code", FP.FP);
        putDef(ret, "#eR;333;This method not used in any where", FP.FP);
        putDef(ret, "#rR;109;change current to firstChar and next to secondChar", FP.FP);
        putDef(ret, "#tR;138;typo mistake should be Match", TYPO_MATCH);
        putDef(ret, "#gR;1190;method name should start with small case", FP.FP);
        putDef(ret, "#tR;136;toString method is not properly overriden", FP.FP);
        putDef(ret, "#jR;166;variable name should be start with small case", FP.FP);
        putDef(ret, "#jR;167;variable name should be start with small case", FP.FP);
        putDef(ret, "#lR;53;This method should be using protected access modifier", FP.FP);
        putDef(ret, "#kR;42;Contructor for Pattern and ignoreCase", FP.FP);
        putDef(ret, "#lR;52;ignoreCase = true", FP.FP);
        putDef(ret, "#lR;66;check the Wholeword logic", FP.FP);
        putDef(ret, "#mR;73;Return type is SearchMatcher but returning is Macther", FP.FP);
        putDef(ret, "#mR;81;returnValue is not declared properly", FP.FP);
        putDef(ret, "#mR;83;Check the logic for Wholeword with start and end Matcher class of Regualr Expression is used to check logic for start and end the word with everytime match is null it will return null", FP.FP);
        putDef(ret, "#cR;50;noWordSep variable is added", FP.FP);
        putDef(ret, "#dR;228;Now setting the HyperSearchOperationNode class variable", FP.FP);
        putDef(ret, "#nR;242;Writing the logic of noWordSep", FP.FP);
        putDef(ret, "#iR;77;Code Changes in PatternSearchMatcher class for current instance Pattern and ignoreCase is added", FP.FP);
        putDef(ret, "#jR;172;It will check the Wholeword logic looking for text and start and end if doesn't matches return null.", FP.FP);
        putDef(ret, "#dR;229;Comment should be above method", FP.FP);
        putDef(ret, "#iR;77;there is only 2 patameters", FP.FP);
        putDef(ret, "#jR;175;what will be in else", FP.FP);

 /*
        P - second 30 developers
         */

        putDef(ret, "#hR;64;this.re=re this.ignoreCase=ignoreCase", FP.FP);
        putDef(ret, "#iR;77;only two variables need  pass", FP.FP);
        putDef(ret, "#mR;75;Not able to se match method returning int.", FP.FP);
        putDef(ret, "#iR;77;Constructor has 3 parameters so this(re, ignoreCase, flase) must match with parameters of constructor", FP.FP);
        putDef(ret, "#tR;138;return \"Match[\" + this.start + \",\" + this.end + \"]\";", P2.TYPO_MATCH);
        putDef(ret, "#hR;64;this.ignoreCase = ignoreCase;", FP.FP);
        putDef(ret, "#dL;224;provide method use description in multiline comment on method top", FP.FP);
        putDef(ret, "#tR;134;substitutions variable is not used", FP.FP);
        putDef(ret, "#kR;35;added constructor to BoyerMooreSearchMatcher", FP.FP);
        putDef(ret, "#fR;654;obtained noWordSep and added to matcher in find()", FP.FP);
        putDef(ret, "#gR;1195;obtained wordBreakChars and added to matcher in replace fn", FP.FP);
        putDef(ret, "#aR;710;eliminated unwanted line", FP.FP);
        putDef(ret, "#oR;55;implemented setNoWordSep", FP.FP);
        putDef(ret, "#pR;68;implemented getNoWordSep", FP.FP);
        putDef(ret, "#qR;79;implemented isWholeWord fn", FP.FP);
        putDef(ret, "#rR;109;implemented isEndWord", FP.FP);
        putDef(ret, "#tR;120;added wholeWord and noWordSep protected fields", FP.FP);
        putDef(ret, "#tR;135;implemented toString function", FP.FP);
        putDef(ret, "#lR;50;updated javadoc", FP.FP);
        putDef(ret, "#lR;55;added boolean field for wholeWord", FP.FP);
        putDef(ret, "#lR;66;added wholeWord boolean return", FP.FP);
        putDef(ret, "#mR;83;updated nextMatch function", FP.FP);
        putDef(ret, "#cR;50;added noWordSep field", FP.FP);
        putDef(ret, "#cR;57;updated constructor for field noWordSep", FP.FP);
        putDef(ret, "#dR;227;added Update on NoWordSep in getSeachMatcher fn", FP.FP);
        putDef(ret, "#nR;242;obtained noWordSep and added to doHyperSearch", FP.FP);
        putDef(ret, "#hR;53;added PatternSearchMatcher constructor", FP.FP);
        putDef(ret, "#iR;77;enhanced PatternSearchMatcher to go", FP.FP);
        putDef(ret, "#jR;171;optimized PatternSearchMatcher for wholeWord case return", FP.FP);
        putDef(ret, "#eR;334;updated getSearchMatcher functionality calls to new approach", FP.FP);
        putDef(ret, "#mR;81;do not know type of returnVaue", FP.FP);
        putDef(ret, "#mR;94;do not know type of return value", FP.FP);
        putDef(ret, "#lR;49;new whole word parameter added", FP.FP);
        putDef(ret, "#mR;83;new condition added", FP.FP);
        putDef(ret, "#cR;49;noWordSep added", FP.FP);
        putDef(ret, "#mR;84;reccurssion causes issues better to use loop", DANGEROUS_RECURSION);
        putDef(ret, "#rR;116;method must return a boolean type vlaue", FP.FP);
        putDef(ret, "#tR;138;Match", TYPO_MATCH);
        putDef(ret, "#lR;53;there should be null check for String pattern", FP.FP);
        putDef(ret, "#lR;55;variable should be char array instead ofthis.pattern", FP.FP);
        putDef(ret, "#kR;42;Constructor having only two parameters", FP.FP);

        return ret;
    }
}