package de.unihannover.orderingExperimentAnalysis;

import java.io.IOException;
import java.io.Reader;

final class Tokenizer {
    private final Reader reader;
    private char preread;

    private enum State {
        BEFORE,
        IN,
        STRING,
        STRING_ESC,
        CHAR,
        CHAR_ESC,
        POTENTIAL_COMMENT,
        LINE_COMMENT,
        BLOCK_COMMENT,
        POTENTIAL_COMMENT_END
    }

    public Tokenizer(Reader reader) {
        this.reader = reader;
        this.preread = '\0';
    }

    public String next() throws IOException {
        Tokenizer.State state = State.BEFORE;
        final StringBuilder cur = new StringBuilder();
        loop: while (true) {
            final char ch;
            if (this.preread != '\0') {
                ch = this.preread;
                this.preread = '\0';
            } else {
                final int i = this.reader.read();
                if (i < 0) {
                    break loop;
                }
                ch = (char) i;
            }
            switch (state) {
            case BEFORE:
                if (ch == '"') {
                    state = State.STRING;
                    cur.append(ch);
                } else if (ch == '\'') {
                    state = State.CHAR;
                    cur.append(ch);
                } else if (ch == '/') {
                    state = State.POTENTIAL_COMMENT;
                } else if (Character.isJavaIdentifierPart(ch) || ch == '@') {
                    state = State.IN;
                    cur.append(ch);
                } else if (!Character.isWhitespace(ch)) {
                    cur.append(ch);
                    break loop;
                }
                break;
            case IN:
                if (Character.isJavaIdentifierPart(ch)) {
                    cur.append(ch);
                } else {
                    this.preread = ch;
                    break loop;
                }
                break;
            case STRING:
                cur.append(ch);
                if (ch == '"') {
                    break loop;
                } else if (ch == '\n') {
                    //should normally not happen but helps to get the parser back on track
                    break loop;
                } else if (ch == '\\') {
                    state = State.STRING_ESC;
                }
                break;
            case STRING_ESC:
                cur.append(ch);
                state = State.STRING;
                break;
            case CHAR:
                cur.append(ch);
                if (ch == '\'') {
                    break loop;
                } else if (ch == '\n') {
                    //should normally not happen but helps to get the parser back on track
                    break loop;
                } else if (ch == '\\') {
                    state = State.CHAR_ESC;
                }
                break;
            case CHAR_ESC:
                cur.append(ch);
                state = State.CHAR;
                break;
            case POTENTIAL_COMMENT:
                if (ch == '/') {
                    state = State.LINE_COMMENT;
                } else if (ch == '*') {
                    state = State.BLOCK_COMMENT;
                } else {
                    cur.append('/');
                    this.preread = ch;
                    break loop;
                }
                break;
            case LINE_COMMENT:
                if (ch == '\n') {
                    state = State.BEFORE;
                }
                break;
            case BLOCK_COMMENT:
                if (ch == '*') {
                    state = State.POTENTIAL_COMMENT_END;
                }
                break;
            case POTENTIAL_COMMENT_END:
                if (ch == '/') {
                    state = State.BEFORE;
                } else {
                    state = State.BLOCK_COMMENT;
                }
                break;
            default:
                throw new AssertionError("invalid state " + state);
            }
        }
        if (cur.length() == 0) {
            return null;
        }
        return cur.toString();
    }
}