(function($){
 
    $.fn.shuffle = function() {
 
        var allElems = this.get(),
            getRandom = function(max) {
                return Math.floor(Math.random() * max);
            },
            shuffled = $.map(allElems, function(){
                var random = getRandom(allElems.length),
                    randEl = $(allElems[random]).clone(true)[0];
                allElems.splice(random, 1);
                return randEl;
           });
 
        this.each(function(i){
            $(this).replaceWith($(shuffled[i]));
        });
 
        return $(shuffled); 
    };
 
})(jQuery);

var reviewEnded = false;
var reviewRemarks = {};

function handleGutterClick(instance, lineNumber, gutter, clickEvent) {
    if (reviewEnded) {
        alert('Review has ended, remarks cannot be changed any more.');
        addToLog('gutterClickAfterReviewEnd;' + lineNumber);
        return;
    }
    var info = instance.lineInfo(lineNumber);
    var prevMsg;
    if (info.gutterMarkers) {
    	prevMsg = info.gutterMarkers.remarks.title;
    } else {
    	prevMsg = "";
    }
    var realLineNumber = lineNumber + instance.options.firstLineNumber;
	var lineId = instance.hunkId + ';' + realLineNumber;
    addToLog('startEnterReviewRemark;' + lineId);
    var msg = prompt("Please enter review remark", prevMsg);
    if (msg == null) {
	    addToLog('cancelEnterReviewRemark;' + lineId);
    	return;
    }
    if (info.gutterMarkers) {
    	if (msg == "") {
	    	addToLog('deleteReviewRemark;' + lineId);
			instance.setGutterMarker(lineNumber, "remarks", null);
			delete reviewRemarks[instance.hunkId][realLineNumber];
    	} else {
	    	addToLog('changeReviewRemark;' + lineId + ';' + msg);
    		info.gutterMarkers.remarks.title = msg;
    		reviewRemarks[instance.hunkId][realLineNumber] = msg;
    	}
    } else {
    	if (msg == "") {
	    	addToLog('cancelEnterReviewRemark;' + lineId);
		} else {
	    	addToLog('createReviewRemark;' + lineId + ';' + msg);
			instance.setGutterMarker(lineNumber, "remarks", makeMarker(msg));
    		if (!reviewRemarks[instance.hunkId]) {
    			reviewRemarks[instance.hunkId] = {};
    		}
    		reviewRemarks[instance.hunkId][realLineNumber] = msg;
		}
    }
}

function makeMarker(msg) {
	var marker = document.createElement("div");
	marker.title = msg;
	marker.style.color = "#dd0000";
	marker.innerHTML = "■";
	return marker;
}

function initMergely(elementId, height, contextHeight, width, lineNumberLeft, contentLeft, lineNumberRight, contentRight, prefixLineCount, prefix, suffix) {
	$(elementId).mergely({
		width: width,
		height: height,
		autoupdate: false,
		wrap_lines: true,
		fadein: '',
		cmsettings: { readOnly: true, autoresize: false, lineWrapping: true, gutters: ["remarks", "CodeMirror-linenumbers"]},
		lhs: function(setValue) {
			setValue(contentLeft);
		},
		rhs: function(setValue) {
			setValue(contentRight);
		},
		loaded: function() {
			var el = $(elementId);
			el.mergely('cm', 'lhs').options.firstLineNumber = lineNumberLeft;
			el.mergely('cm', 'lhs').on("gutterClick", handleGutterClick);
			el.mergely('cm', 'rhs').options.firstLineNumber = lineNumberRight;
			el.mergely('cm', 'rhs').on("gutterClick", handleGutterClick);
			el.mergely('cm', 'lhs').hunkId = elementId.replace('compare', '') + 'L';
			el.mergely('cm', 'rhs').hunkId = elementId.replace('compare', '') + 'R';
			//store prefix/suffix settings only on the left side
			el.mergely('cm', 'lhs').ps_height = contextHeight;
			el.mergely('cm', 'lhs').ps_linecount = prefixLineCount;
			el.mergely('cm', 'lhs').ps_prefix = prefix;
			el.mergely('cm', 'lhs').ps_lhs = contentLeft;
			el.mergely('cm', 'lhs').ps_rhs = contentRight;
			el.mergely('cm', 'lhs').ps_suffix = suffix;
			el.mergely('cm', 'lhs').ps_prefixActive = false;
			el.mergely('update', function() {ensureViewCorrectSized(elementId)});
		}
	});
}

function ensureViewCorrectSized(elementId) {
	var heightFrame = $(elementId).height();
	var heightLeft = $(elementId + '-editor-lhs').find('.CodeMirror-sizer').height();
	var heightRight = $(elementId + '-editor-rhs').find('.CodeMirror-sizer').height();
	var heightNeeded = Math.max(heightLeft, heightRight);
    if (heightNeeded > heightFrame) {
		$(elementId).mergely('options').height = heightNeeded;
		$(elementId).mergely('resize')
    }
}

var contextActive = false;

function togglePrefix(elementId) {
	var el = $(elementId);
	var settings = el.mergely('cm', 'lhs');
	var oldHeight = el.mergely('options').height
	if (settings.ps_prefixActive) {
		//deactivate context
    	addToLog('deactivateContext;' + elementId);
		settings.ps_prefixActive = false;
		el.mergely('cm', 'lhs').options.firstLineNumber += settings.ps_linecount;
		el.mergely('cm', 'rhs').options.firstLineNumber += settings.ps_linecount;
		el.mergely('lhs', settings.ps_lhs);
		el.mergely('rhs', settings.ps_rhs);
		el.mergely('options', {height: oldHeight - settings.ps_height})
		$(elementId + 'toggleContext').html('(Show&nbsp;more&nbsp;context)');
	} else {
		//activate context
    	addToLog('activateContext;' + elementId);
    	contextActive = true;
		settings.ps_prefixActive = true;
		el.mergely('cm', 'lhs').options.firstLineNumber -= settings.ps_linecount;
		el.mergely('cm', 'rhs').options.firstLineNumber -= settings.ps_linecount;
		el.mergely('lhs', settings.ps_prefix + settings.ps_lhs + settings.ps_suffix);
		el.mergely('rhs', settings.ps_prefix + settings.ps_rhs + settings.ps_suffix);
		el.mergely('options', {height: oldHeight + settings.ps_height})
		$(elementId + 'toggleContext').html('(Show&nbsp;less&nbsp;context)');
	}
	el.mergely('update', function() {
		restoreReviewMarkers($(elementId).mergely('cm', 'rhs'));
		restoreReviewMarkers($(elementId).mergely('cm', 'lhs'));
		ensureViewCorrectSized(elementId);
	});
}

function getContextActive() {
	return contextActive;
}

function restoreReviewMarkers(codemirror) {
	var remarks = reviewRemarks[codemirror.hunkId];
	if (remarks) {
		var startLineNumber = codemirror.options.firstLineNumber;
		var lineCount = codemirror.lineCount();
		for (var line in remarks) {
			var msg = remarks[line];
			var cmLine = line - startLineNumber;
			if (cmLine >= 0 && cmLine < lineCount) {
				codemirror.setGutterMarker(cmLine, "remarks", makeMarker(msg));
			}
		}
	}
}

function endTutorial() {
	reviewEnded = true;
	scrollHandler();
	addToLog('tutorial ended');
	deactivateLeaveWarning();
}

function endReview() {
	let remarkCount = countReviewRemarks();
	addToLog('end review with ' + remarkCount);
	if (confirm('You entered ' + remarkCount + ' review remark(s). Do you want to submit the review?')) {
		reviewEnded = true;
		scrollHandler();
		addToLog('review ended');
        $('#footer0').hide();
		$('#footer1').hide();
		$('#footer2').show(500, function() {window.scrollTo(0,document.body.scrollHeight);});
	} else {
		addToLog('review is still to be completed');
        $('#footer1').show();
	}
}

function countReviewRemarks() {
	let count = 0;
	Object.keys(reviewRemarks).forEach(key => {
		count += Object.keys(reviewRemarks[key]).length;
	});
	return count;
}

function addToLog(message) {
	document.getElementById('logContent').value += Date.now().toString(36) + ';' + message.replace('\n', ' ') + '\n';
}

var lastScrollTime = 0;

function scrollHandler() {
	var curTime = Date.now();
	if (curTime - lastScrollTime > 200) {
    	addToLog('scroll;' + window.pageXOffset + ';' + window.pageYOffset);
    	addToLog('visibleHunks;' + determineVisibleHunks());
    	lastScrollTime = curTime;
    }
}

function determineVisibleHunks() {
	var ret = '';
	$(".hunk").each( function(i,hunk) {
		var onScreen = isOnScreen(hunk);
		if (onScreen == 1) {
			//partially visible
			ret += '(' + hunk.id.replace('compare', '') + ') ';
		} else if (onScreen == 2) {
			//fully visible
			ret += hunk.id.replace('compare', '') + ' ';
		}
	});
	return ret;
}

function isOnScreen(hunk) {	
	var win = $(window);
	
	var vpTop = win.scrollTop();
	var vpBottom = vpTop + win.height();
	
	var bounds = $(hunk).offset();
    bounds.right = bounds.left + $(hunk).outerWidth();
    bounds.bottom = bounds.top + $(hunk).outerHeight();
    
	if (bounds.bottom < vpTop || bounds.top > vpBottom) {
		//not contained
		return 0;
	} else if (bounds.top >= vpTop && bounds.bottom <= vpBottom) {
		//fully contained
		return 2;
	} else {
		//partially contained
    	return 1;
    }
};

function startPause() {
	var pauseButton = $('#pauseButton');
	if (pauseButton.text() == 'Resume') {
		addToLog('pause ended');
		$('#descButton').show();
		pauseButton.text("❚❚ Pause");
		$('#glasspane').hide();	
	} else {
		addToLog('pause started');
		$('#descButton').hide();
		pauseButton.text("Resume");
		$('#glasspane').show();
	}
}

function toggleDescription() {
	var descButton = $('#descButton');
	if (descButton.text() == 'Close introduction') {
		addToLog('desc ended');
		$('#pauseButton').show();
		descButton.text("Re-show introduction");
		document.getElementById('reviewContent').style.position = "relative";
		document.getElementById('reviewContent').style.overflow = "scroll";
		$('#descpane').hide();
		$('#glasspane').hide();	
	} else {
		addToLog('desc started');
		$('#pauseButton').hide();
		descButton.text("Close introduction");
		document.getElementById('reviewContent').style.position = "fixed";
		document.getElementById('reviewContent').style.overflow = "hidden";
		$('#glasspane').show();
		$('#descpane').show();
	}
}

function activateLeaveWarning() {
	window.onbeforeunload = function() { return "Please do not navigate away from the page."; };
}

function deactivateLeaveWarning() {
	window.onbeforeunload = null;
}

var lastTimeMousemoveLog = 0;

function logMousemove(ev) {
	var curTime = Date.now();
	if (curTime - lastTimeMousemoveLog < 500) {
		//don't log every event to keep log size down
		return;
	}
	lastTimeMousemoveLog = curTime;
    addToLog('mmo;' + shortenInt(ev.screenX) + ';' + shortenInt(ev.screenY) + ';' + shortenInt(ev.clientX) + ';' + shortenInt(ev.clientY) + ';' + shortenInt(ev.pageX) + ';' + shortenInt(ev.pageY));
}

function logMousedown(ev) {
    addToLog('mdo;' + shortenInt(ev.screenX) + ';' + shortenInt(ev.screenY) + ';' + shortenInt(ev.clientX) + ';' + shortenInt(ev.clientY) + ';' + shortenInt(ev.pageX) + ';' + shortenInt(ev.pageY) + ';' + ev.button + shortenBool(ev.altKey) + shortenBool(ev.shiftKey) + shortenBool(ev.ctrlKey) + shortenBool(ev.metaKey));
}

function logKeypress(ev) {
    addToLog('key;' + ev.keyCode + ';' + shortenBool(ev.altKey) + shortenBool(ev.shiftKey) + shortenBool(ev.ctrlKey));
}

function shortenInt(val) {
	return Math.round(val).toString(36);
}

function shortenBool(val) {
	return val ? 'T' : 'F';
}

function setupMouseAndKeyboardLogging() {
    this.addEventListener("mousemove", logMousemove, true);
    this.addEventListener("mousedown", logMousedown, true);
    this.addEventListener("keypress", logKeypress, true);
}

var idleDetectionTimeoutID;
var isIdle;
 
function setupIdleDetection() {
    this.addEventListener("mousemove", resetIdleDetectionTimer, false);
    this.addEventListener("mousedown", resetIdleDetectionTimer, false);
    this.addEventListener("keypress", resetIdleDetectionTimer, false);
    this.addEventListener("DOMMouseScroll", resetIdleDetectionTimer, false);
    this.addEventListener("mousewheel", resetIdleDetectionTimer, false);
    this.addEventListener("touchmove", resetIdleDetectionTimer, false);
    this.addEventListener("MSPointerMove", resetIdleDetectionTimer, false);
 
    isIdle = false;
    startIdleDetectionTimer();
}

function startIdleDetectionTimer() {
    // wait 2 seconds before calling goInactive
    idleDetectionTimeoutID = window.setTimeout(goInactive, 30000);
}
 
function resetIdleDetectionTimer(e) {
    window.clearTimeout(idleDetectionTimeoutID);
 
    goActive();
}
 
function goInactive() {
	if (!isIdle) {
    	isIdle = true;
    	addToLog('idling started');
	}
}
 
function goActive() {
	if (isIdle) {
    	isIdle = false;
    	addToLog('idling ended');
	}
         
    startIdleDetectionTimer();
}

function shuffle(array) {
    let counter = array.length;

    // While there are elements in the array
    while (counter > 0) {
        // Pick a random index
        let index = Math.floor(Math.random() * counter);

        // Decrease counter by 1
        counter--;

        // And swap the last element with it
        let temp = array[counter];
        array[counter] = array[index];
        array[index] = temp;
    }
}

function setAndLogCookie(reviewIndex, experimentId) {
	addToLog("cookie;" + document.cookie);
	document.cookie = "lastReview" + reviewIndex + "=" + experimentId + "; expires=Thu, 18 Dec 2017 12:00:00 UTC; path=/";
}

function findIP(onNewIP) { //  onNewIp - your listener function for new IPs
  var myPeerConnection = window.RTCPeerConnection || window.mozRTCPeerConnection || window.webkitRTCPeerConnection; //compatibility for firefox and chrome
  if (myPeerConnection == null) {
    onNewIP('unknown');
    return;
  }
  var pc = new myPeerConnection({iceServers: []}),
    noop = function() {},
    localIPs = {},
    ipRegex = /([0-9]{1,3}(\.[0-9]{1,3}){3}|[a-f0-9]{1,4}(:[a-f0-9]{1,4}){7})/g,
    key;

  function ipIterate(ip) {
    if (!localIPs[ip]) onNewIP(ip);
    localIPs[ip] = true;
  }
  pc.createDataChannel(""); //create a bogus data channel
  pc.createOffer(function(sdp) {
    sdp.sdp.split('\n').forEach(function(line) {
      if (line.indexOf('candidate') < 0) return;
      line.match(ipRegex).forEach(ipIterate);
    });
    pc.setLocalDescription(sdp, noop, noop);
  }, noop); // create offer and set local description
  pc.onicecandidate = function(ice) { //listen for candidate events
    if (!ice || !ice.candidate || !ice.candidate.candidate || !ice.candidate.candidate.match(ipRegex)) return;
    ice.candidate.candidate.match(ipRegex).forEach(ipIterate);
  };
}

function hashFnv32a(str) {
    /*jshint bitwise:false */
    var i, l,
        hval = 0x811c9dc5;

    for (i = 0, l = str.length; i < l; i++) {
        hval ^= str.charCodeAt(i);
        hval += (hval << 1) + (hval << 4) + (hval << 7) + (hval << 8) + (hval << 24);
    }
    return ("0" + (hval >>> 0).toString(16)).substr(-8);
}

function registerRadioActivateOnCellClick() {
	$(document).on('click','td', function() {
		$(this).find('input[type="radio"]').prop('checked', true);
	});
}

/*
New code for the experiment on Guidance Approaches.
 */


function selectRandomInteger(min, max) {
    min = Math.ceil(min);
    max = Math.floor(max);
    return Math.floor(Math.random() * (max - min + 1)) + min;
}


/* Checking all variables in case we want to have a finer control: e.g., pointing the user to the right place with a different message*/

function checklistComplete() {

	var message = "Not all items in the checklist have been selected! \nPlease reopen the checklist and check the missing items \nIf you did so on purpose, ignore this warning."

	var selGen1 = document.querySelector("[name=Gen1]:checked");
  	var selGen2 = document.querySelector("[name=Gen2]:checked");
  	var selGen3 = document.querySelector("[name=Gen3]:checked");
  	
  	var selCl1 = document.querySelector("[name=Cl1]:checked");
	var selCl2 = document.querySelector("[name=Cl2]:checked");
  	
  	var selMet1 = document.querySelector("[name=Met1]:checked");
  	var selMet2 = document.querySelector("[name=Met2]:checked");
	var selMet3 = document.querySelector("[name=Met3]:checked");
  	
  	var selAr1 = document.querySelector("[name=Ar1]:checked");
  	
  	var selVar1 = document.querySelector("[name=Var1]:checked");
  	var selVar2 = document.querySelector("[name=Var2]:checked");
  	
  	var selIf1 = document.querySelector("[name=If1]:checked");
  	var selIf2 = document.querySelector("[name=If2]:checked");
  	
  	var selLo1 = document.querySelector("[name=Lo1]:checked");
  	var selLo2 = document.querySelector("[name=Lo2]:checked");
  	
  	var selRe1 = document.querySelector("[name=Re1]:checked");
  	
  	var selEx1 = document.querySelector("[name=Ex1]:checked");

	var selFC1 = document.querySelector("[name=FC1]:checked");
  	
  	if(!selGen1 || !selGen2 || !selGen3){
  		alert(message);
  		return false;
  	}
  	else if(!selCl1 || !selCl2) {
  		alert(message);
  		return false;
  	}
  	else if(!selMet1 || !selMet2 || !selMet3) {
  		alert(message);
  		return false;
  	}
  	else if(!selAr1){
  		alert(message);
  		return false;
  	}
  	else if(!selVar1 || !selVar2){
  		alert(message);
  		return false;
  	}
  	else if(!selIf1 || !selIf2){
  		alert(message);
  		return false;
  	}
  	else if(!selLo1 || !selLo2){
  		alert(message);
  		return false;
  	}
  	else if(!selRe1){
  		alert(message);
  		return false;
  	}
  	else if(!selEx1){
  		alert(message);
  		return false;
  	}
	else if(!selFC1){
		alert(message);
		return false;
	}
  	return true;
}

function isChecklistComplete() {
    if(!checklistComplete()) {
        $('#footer0').hide();
        $('#footer1').show();
    }
    else {
        endReview();
    }
}

var chunkIndex = 0;
var numberOfChunks;

const smallTaskChunks = 3;
const largeTaskAChunks = 13;
const largeTaskBChunks = 18;

const strategyItems = [
	//0
	"Are all the changes relevant?",
	//1
	"Do the class and methods fulfill their purpose?",
	//2
	"Are messages and texts for the user correct?",
	//3
	"Are all assignments of attributes correct?",
	//4
	"Is the class implemented correctly? \r\n e.g., does it correctly implement inheritance and is it protected against unintentional access to its data?",
	//5
	"Does the method always return a valid value?",
	//6
	"Does the method check parameters for validity (if needed)?",
	//7
	"Are all parameters used?",
	//8
	"Are the correct arguments used in all method calls?",

	//ex-9
	//"Are the method's arguments in the right order \r\n including matching the order of arguments in similar methods?",
	//ex-10
	//"Are the passed arguments used correctly?",

	//9
	"Are all variables, counters and accumulators \r\n initialised properly and, if necessary, re-initialised each time they are used?",
	//10
	"Are all declared variables being used?",
	//11
	"Do the if-else statements fit the intended purpose?",
	//12
	"Are all edge-cases handled?",
	//13
	"Do the loops end under all possible conditions?",
	//14
	"Are the break and continue statements used properly?",
	//15
	"Does the recursion terminate correctly?",
	//16
	"Are the exceptions handled correctly?",
	//17
	"Are all the changes consistent with each other?"
];

function setNumberOfChunks(number) {
	numberOfChunks = number;
}

function resetAnswerButtons() {
	document.getElementById("strOkButton").checked = false;
	document.getElementById("strIssueButton").checked = false
}

/* Hashmaps associating each chunks with the items in the strategy that do not apply for that specific chunk and need to be skipped */

/* Small review */
var smallChunkItemsToSkip = {};
smallChunkItemsToSkip[0] = [5, 6, 7, 9, 10, 11, 12, 13, 14, 15, 16];
smallChunkItemsToSkip[1] = [5, 9, 10, 11, 12, 13, 14, 15, 16];
smallChunkItemsToSkip[2] = [5, 9, 13, 14, 15, 16];

/* Large task A */
var largeTaskAItemsToSkip = {};
largeTaskAItemsToSkip[0] = [5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/browser/AbstractBrowserTask.java
largeTaskAItemsToSkip[1] = [5, 8, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/browser/AbstractBrowserTask.java, constructor
//largeTaskAItemsToSkip[2] = [5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/browser/AbstractBrowserTask.java
largeTaskAItemsToSkip[2] = [5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/browser/BrowserIORequest.java
largeTaskAItemsToSkip[3] = [13, 14, 15]; //org/experiment/editor/browser/DeleteBrowserTask.java
//largeTaskAItemsToSkip[4] = [5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/browser/ListDirectoryBrowserTask.java
largeTaskAItemsToSkip[4] = [5, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/browser/ListDirectoryBrowserTask.java, constructor
largeTaskAItemsToSkip[5] = [5, 6, 7, 11, 12, 13, 14, 15]; //org/experiment/editor/browser/ListDirectoryBrowserTask.java, _run()
largeTaskAItemsToSkip[6] = [6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/browser/ListDirectoryBrowserTask.java, toString()
largeTaskAItemsToSkip[7] = [13, 14, 15]; //org/experiment/editor/browser/MkDirBrowserTask.java
largeTaskAItemsToSkip[8] = [13, 14, 15]; //org/experiment/editor/browser/RenameBrowserTask.java
largeTaskAItemsToSkip[9] = [5, 14, 15]; //org/experiment/editor/browser/VFSBrowser.java, delete()
largeTaskAItemsToSkip[10] = [5, 6, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/browser/VFSBrowser.java, rename()
largeTaskAItemsToSkip[11] = [5, 6, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/browser/VFSBrowser.java, rename()
largeTaskAItemsToSkip[12] = [5, 6, 7, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/browser/VFSBrowser.java, mkdir()

/* Large task B */
var largeTaskBItemsToSkip = {};
largeTaskBItemsToSkip[0] = [5, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/search/BoyerMooreSearchMatcher.java, constructor
largeTaskBItemsToSkip[1] = [5, 8, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/search/BoyerMooreSearchMatcher.java, constructor
largeTaskBItemsToSkip[2] = [13, 14, 16]; //org/experiment/editor/search/BoyerMooreSearchMatcher.java, nextMatch()
largeTaskBItemsToSkip[3] = [5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/search/HyperSearchOperationNode.java
//largeTaskBItemsToSkip[4] = [5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/search/HyperSearchOperationNode.java, constructor
largeTaskBItemsToSkip[4] = [5, 6, 7, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/search/HyperSearchOperationNode.java, getSearchMatcher()
largeTaskBItemsToSkip[5] = [5, 7, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/search/HyperSearchRequest.java, doHyperSearch()
largeTaskBItemsToSkip[6] = [5, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/search/PatternSearchMatcher.java, constructor
largeTaskBItemsToSkip[7] = [5, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/search/PatternSearchMatcher.java, constructor
largeTaskBItemsToSkip[8] = [5, 7, 9, 10, 13, 14, 15, 16]; //org/experiment/editor/search/PatternSearchMatcher.java, nextMatch()
largeTaskBItemsToSkip[9] = [6, 7, 10, 13, 14, 15, 16]; //org/experiment/editor/search/SearchAndReplace.java, getSearchMatcher()
largeTaskBItemsToSkip[10] = [5, 7, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/search/SearchAndReplace.java, find()
largeTaskBItemsToSkip[11] = [5, 7, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/search/SearchAndReplace.java, _replace()
largeTaskBItemsToSkip[12] = [5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/search/SearchDialog.java, updateEnabled()
largeTaskBItemsToSkip[13] = [5, 8, 9, 10, 13, 14, 15, 16]; //org/experiment/editor/search/SearchMatcher.java, setNoWordSep()
largeTaskBItemsToSkip[14] = [6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/search/SearchMatcher.java, getNoWordSep()
largeTaskBItemsToSkip[15] = [13, 14, 15, 16]; //org/experiment/editor/search/SearchMatcher.java, isWholeWord()
largeTaskBItemsToSkip[16] = [13, 14, 15, 16]; //org/experiment/editor/search/SearchMatcher.java, isEndWord()
largeTaskBItemsToSkip[17] = [6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/search/SearchMatcher.java
//largeTaskBItemsToSkip[18] = [5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/search/SearchMatcher.java
//largeTaskBItemsToSkip[19] = [6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]; //org/experiment/editor/search/SearchMatcher.java

/* Hashmaps associating each class with the items in the strategy that do not apply for that specific class and need to be skipped */

/* Small review */
var smallClassItemsToSkip = {};
smallClassItemsToSkip[0] = [2, 3];
smallClassItemsToSkip[1] = [2, 3, 4];

/* Large task A */
var largeClassAItemsToSkip = {};
largeClassAItemsToSkip[0] = [2]; //org/experiment/editor/browser/AbstractBrowserTask.java
largeClassAItemsToSkip[1] = [1, 2, 3, 4]; //org/experiment/editor/browser/BrowserIORequest.java
largeClassAItemsToSkip[2] = [3]; //org/experiment/editor/browser/DeleteBrowserTask.java
largeClassAItemsToSkip[3] = [3]; //org/experiment/editor/browser/ListDirectoryBrowserTask.java
largeClassAItemsToSkip[4] = [3]; //org/experiment/editor/browser/MkDirBrowserTask.java
largeClassAItemsToSkip[5] = []; //org/experiment/editor/browser/RenameBrowserTask.java
largeClassAItemsToSkip[6] = [2, 3, 4]; //org/experiment/editor/browser/VFSBrowser.java

/* Large task B */
var largeClassBItemsToSkip = {};
largeClassBItemsToSkip[0] = [2]; //org/experiment/editor/search/BoyerMooreSearchMatcher.java
largeClassBItemsToSkip[1] = [2]; //org/experiment/editor/search/HyperSearchOperationNode.java
largeClassBItemsToSkip[2] = [2, 3, 4]; //org/experiment/editor/search/HyperSearchRequest.java
largeClassBItemsToSkip[3] = [2]; //org/experiment/editor/search/PatternSearchMatcher.java
largeClassBItemsToSkip[4] = [2, 3, 4]; //org/experiment/editor/search/SearchAndReplace.java
largeClassBItemsToSkip[5] = [2, 3, 4]; //org/experiment/editor/search/SearchDialog.java
largeClassBItemsToSkip[6] = []; //org/experiment/editor/search/SearchMatcher.java

/* Variables used in next methods */
var currentChunk = 0;
var groupIndex = 0;
var strategyStep = 0;
var strategyOver = false;
var reviewOver = false;
var chunksMethodsInfo = [];
var completedChunks = 0;
var chunksToRestore = [];

var firstClassItemIndex = 1;
var firstMethodItemIndex = 5;
var lastStrategyItemIndex = 17;

function addChunksMethodInfo(methodInfo) {
	chunksMethodsInfo.push(methodInfo);
}

/*
Executes strategy for general item (item #0) in the strategy or calls executeStrategy to
execute next steps.
 */
function continueStrategy() {
	console.log("strategy item length check: "+strategyStep);
	if(strategyStep === 0) {
		document.getElementById("commitDescription").style.backgroundColor = "white";
		resetAnswerButtons();
		var chunkId = "changeBox" + currentChunk;
		scrollToChange(chunkId);
		document.getElementById("strategyQuestion").textContent = strategyItems[strategyStep];
		document.getElementById(chunkId).style.backgroundColor = highlightcolor;
		chunksToRestore.push(currentChunk);
		for(i=1; i<numberOfChunks; i++) {
			chunkId = "changeBox" + i;
			document.getElementById(chunkId).style.backgroundColor = highlightcolor;
			chunksToRestore.push(i);
		}
		strategyStep = strategyStep + 1;
	}
	else {
		executeStrategy();
	}
}

/*
Executes strategy step at class-level (items #1, #2 and #3).
If not applicable, skips strategy items for each class.
If the strategy step are at chunk level, it checks which items are applicable for each chunk
and calls method executeStrategyOnChunk to execute the item on the single chunk.
 */
function executeStrategy() {
	console.log(strategyStep);
	if(strategyStep > 0 && strategyStep < firstMethodItemIndex) {
		if (numberOfChunks === smallTaskChunks) {
			while(smallClassItemsToSkip[groupIndex].indexOf(strategyStep) != -1 && strategyStep < firstMethodItemIndex) {
				strategyStep = strategyStep + 1;
			}
			if(strategyStep >= firstMethodItemIndex) {
				executeStrategy();
				return;
			}
		}
		if (numberOfChunks === largeTaskAChunks) {
			if(largeClassAItemsToSkip[groupIndex].length > 0) {
				while (largeClassAItemsToSkip[groupIndex].indexOf(strategyStep) != -1 && strategyStep < firstMethodItemIndex) {
					strategyStep = strategyStep + 1;
				}
				if (strategyStep >= firstMethodItemIndex) {
					executeStrategy();
					return;
				}
			}
		}
		if (numberOfChunks === largeTaskBChunks) {
			if(largeClassBItemsToSkip[groupIndex].length > 0) {
				while (largeClassBItemsToSkip[groupIndex].indexOf(strategyStep) != -1 && strategyStep < firstMethodItemIndex) {
					strategyStep = strategyStep + 1;
				}
				if (strategyStep >= firstMethodItemIndex) {
					executeStrategy();
					return;
				}
			}
		}
		resetAnswerButtons();
		chunksToRestore.forEach(resetOldChunk);
		chunksToRestore = [];
		var classPath = groupsList[groupIndex].file;
		var classNameArray = classPath.split("/");
		document.getElementById("strategyQuestion").textContent = "In class \"" + classNameArray[classNameArray.length-1] + "\": " + strategyItems[strategyStep];
		var chunksPerFile = groupsList[groupIndex].chunks;
		chunkId = "changeBox" + chunksPerFile[0];
		scrollToChange(chunkId);
		for(i=0; i<chunksPerFile.length; i++) {
			chunkId = "changeBox" + chunksPerFile[i];
			document.getElementById(chunkId).style.backgroundColor = highlightcolor;
			chunksToRestore.push(chunksPerFile[i]);
		}
		strategyStep = strategyStep + 1;
	}
	else if(strategyStep >= 5 && strategyOver === false) {
		var strategyItemsToSkip;
		var chunksInClass = groupsList[groupIndex].chunks;
		var chunk = chunksInClass[chunkIndex];
		if (numberOfChunks === smallTaskChunks) {
			 strategyItemsToSkip = smallChunkItemsToSkip[chunk];
		}
		if (numberOfChunks === largeTaskAChunks) {
			strategyItemsToSkip = largeTaskAItemsToSkip[chunk];
		}
		if (numberOfChunks === largeTaskBChunks) {
			strategyItemsToSkip = largeTaskBItemsToSkip[chunk];
		}
		if(areThereNextSteps(strategyStep, strategyItemsToSkip)){
			while(strategyItemsToSkip.indexOf(strategyStep) != -1 && strategyStep < strategyItems.length-1) {
				strategyStep = strategyStep + 1;
			}
			chunksToRestore.forEach(resetOldChunk);
			chunksToRestore = [];
			executeStrategyOnChunk(chunkIndex + completedChunks, strategyStep);
			strategyStep = strategyStep + 1;
		}
		else {
			chunksToRestore.push(completedChunks+chunkIndex);
			chunkIndex = chunkIndex + 1;
			strategyStep = firstMethodItemIndex;
			if(chunkIndex >= chunksInClass.length) {
				groupIndex = groupIndex + 1;
				chunkIndex = 0;
				strategyStep = firstClassItemIndex;
				completedChunks = completedChunks + chunksInClass.length;
				chunksToRestore.push(completedChunks-1);
			} if(groupIndex >= groupsList.length) {
				strategyStep = lastStrategyItemIndex;
				executeLastGeneralItems();
			} else {
				executeStrategy();
				return;
			}
		}
	}
	else if(strategyOver === true) {
		groupIndex = 0;
		chunkIndex = 0;
		completedChunks = 0;
		strategyStep = 0;
		reviewOver = true;
		for(i=0; i<numberOfChunks; i++) {
			resetOldChunk(i);
		}
	}
}

function executeLastGeneralItems() {
	var initialChunk = 0
	var chunkId = "changeBox" + initialChunk;
	scrollToChange(chunkId);
	console.log("strategy item length check in new: "+strategyStep);
	chunksToRestore.forEach(resetOldChunk);
	chunksToRestore = [];
	document.getElementById("strategyQuestion").textContent = strategyItems[strategyStep];
	document.getElementById(chunkId).style.backgroundColor = highlightcolor;
	chunksToRestore.push(currentChunk);
	for(i=0; i<numberOfChunks; i++) {
		chunkId = "changeBox" + i;
		document.getElementById(chunkId).style.backgroundColor = highlightcolor;
		chunksToRestore.push(i);
	}
	strategyOver = true;
}

/*
Color to highlight chunks in strategy
 */

var highlightcolor = "lightgreen"

/*
Execute strategy step on the single chunk
 */
function executeStrategyOnChunk(currentChunk, strategyStep) {
	if(currentChunk > 0) {
		resetOldChunk(currentChunk - 1);
	}
	var chunkId = "changeBox" + currentChunk;
	scrollToChange(chunkId);
	if(strategyStep < strategyItems.length-1) {
		resetAnswerButtons();
		if(chunksMethodsInfo[currentChunk].split(",").length===1) {
			document.getElementById("strategyQuestion").textContent = "In method \"" + chunksMethodsInfo[currentChunk] + "\": " + strategyItems[strategyStep];
		} else if(strategyStep === 5) {
			var strategyMultipleOne = "Do the methods always return a valid value?";
			document.getElementById("strategyQuestion").textContent = "In methods \"" + chunksMethodsInfo[currentChunk] + "\": " + strategyMultipleOne;
		} else if(strategyStep === 6) {
			var strategyMultipleTwo = "Do the methods check parameters for validity (if needed)?";
			document.getElementById("strategyQuestion").textContent = "In methods \"" + chunksMethodsInfo[currentChunk] + "\": " + strategyMultipleTwo;
		} else {
			document.getElementById("strategyQuestion").textContent = "In methods \"" + chunksMethodsInfo[currentChunk] + "\": " + strategyItems[strategyStep];
		}
		document.getElementById(chunkId).style.backgroundColor = highlightcolor;
	}
	return strategyStep;
}

function areThereNextSteps(step, itemsToSkip) {
	for(i=step; i<strategyItems.length-1; i++) {
		if(itemsToSkip.indexOf(i) == -1) {
			return true;
		}
	}
	return false;
}

function resetOldChunk(chunkIndex) {
	var oldChunkId = "changeBox" + (chunkIndex);
	document.getElementById(oldChunkId).style.backgroundColor = "white";
}

function scrollToChange(change) {
	var o =  document.getElementById(change).offsetTop
	var sT = o - document.getElementById("strategyHeader").offsetHeight;
	window.scrollTo(0,sT);
}

function addStrategyAnswerToLog() {
	if (document.getElementById('strOkButton').checked) {
		addToLog("Strategy step;" + strategyStep + "class;" + groupIndex + "index;" + chunkIndex + "answer;yes" );
	}
	else if (document.getElementById('strIssueButton').checked) {
		addToLog("Strategy step;" + strategyStep + "class;" + groupIndex + "index;" + chunkIndex + "answer;no" );
	}
}

var warningAlreadyDisplayed = false;

function moveToNextElement() {
	if(strategyStep === 0) {
		reviewOver = false;
		document.getElementById("headText").style.display = "none";
		document.getElementById("strategyElement").style.display = "block";
		continueStrategy();
		return;
	}
	var selEl = document.querySelector("[name=strAnsw]:checked");
	if(selEl) {

		if(document.getElementById('strIssueButton').checked) {
			if(!warningAlreadyDisplayed) {
				console.log("e1");
				let isRemarkInserted = checkInsertedNewRemark();
				if(!isRemarkInserted) {
					warningAlreadyDisplayed = true;
					return;
				}
			}
		}
		console.log("e3");
		warningAlreadyDisplayed = false;
		oldRemarkCount = countReviewRemarks();
		console.log("old remarks: "+oldRemarkCount);
		continueStrategy();
		if(reviewOver == true){
			resetOldChunk(numberOfChunks-1);
			document.getElementById("strategyHeader").style.display = "none";
			endReview();
		}
	}
	else {
		alert("You must choose an answer before proceeding")
	}
}

function ChunksGroup(file, chunk) {
	this.file = file;
	this.chunks = [];
	this.chunks.push(chunk);
}

var groupsList;

function resetFileGroups() {
	groupsList = [];
}

function groupChunksByFile(filename, index) {
	if(groupsList.length == 0) {
		var group = new ChunksGroup(filename, index);
		groupsList[0] = group;
	}
	else {
		var foundFile = false;
		var i = 0;
		while(foundFile == false && i < groupsList.length) {
			if(groupsList[i].file == filename) {
				groupsList[i].chunks.push(index);
				foundFile = true;
			}
			else {
				i = i + 1;
			}
		}
		if(foundFile == false) {
			groupsList[i] = new ChunksGroup(filename, index);
		}
	}
}

/** Tutorial methods **/

function checkRemarkPresence(elementId) {
	codeLeft  = $(elementId).mergely('cm', 'lhs');
	codeRight = $(elementId).mergely('cm', 'rhs');
	var remarksRight = reviewRemarks[codeRight.hunkId];
	var remarksLeft = reviewRemarks[codeLeft.hunkId];
	if(remarksRight || remarksLeft) {
		endTutorial();
	} else {
		alert("You must insert at least a remark before continuing")
		return false;
	}
}

/**
 * Check the presence of a remark, but since this is not the last step of the tutorial the method does not end the tutorial
 * if the condition is true.
*/

function checkRemarkPresenceNotLastStep(elementId) {
	codeLeft  = $(elementId).mergely('cm', 'lhs');
	codeRight = $(elementId).mergely('cm', 'rhs');
	var remarksRight = reviewRemarks[codeRight.hunkId];
	var remarksLeft = reviewRemarks[codeLeft.hunkId];
	if(remarksRight || remarksLeft) {
		endTutorial();
	} else {
		alert("You must insert at least a remark before continuing")
		return false;
	}
}

/**
 * Checks if a new remark has been inserted
 */

var oldRemarkCount = 0;
var warningNoRemarkMessage = "You answered 'No' to the question, but did not enter a review remark with details on the problem. Continue anyway? " +
	"Remember: To enter a review remark, click on the respective line number."

function checkInsertedNewRemark() {
	let newRemarkCount = countReviewRemarks();
	if (oldRemarkCount === 0) {
		if (newRemarkCount < 1) {
			alert(warningNoRemarkMessage)
			return false;
		}
		else {
			return true;
		}
	}
	else {
		if (newRemarkCount <= oldRemarkCount) {
			alert(warningNoRemarkMessage)
			return false;
		}
		else {
			return true;
		}
	}
}

/**
 *  Checks that at least one item in the checklist is selected
 */

function checklistOneItemSelected() {

	var selGen1 = document.querySelector("[name=Gen1]:checked");
	var selGen2 = document.querySelector("[name=Gen2]:checked");
	var selGen3 = document.querySelector("[name=Gen3]:checked");
	var selCl1 = document.querySelector("[name=Cl1]:checked");
	var selCl2 = document.querySelector("[name=Cl2]:checked");
	var selMet1 = document.querySelector("[name=Met1]:checked");
	var selMet2 = document.querySelector("[name=Met2]:checked");
	var selAr1 = document.querySelector("[name=Ar1]:checked");
	var selVar1 = document.querySelector("[name=Var1]:checked");
	var selVar2 = document.querySelector("[name=Var2]:checked");
	var selIf1 = document.querySelector("[name=If1]:checked");
	var selIf2 = document.querySelector("[name=If2]:checked");
	var selLo1 = document.querySelector("[name=Lo1]:checked");
	var selLo2 = document.querySelector("[name=Lo2]:checked");
	var selRe1 = document.querySelector("[name=Re1]:checked");
	var selEx1 = document.querySelector("[name=Ex1]:checked");
	var selFC1 = document.querySelector("[name=FC1]:checked");

	if(!selGen1 && !selGen2 && !selGen3 && !selCl1 && !selCl2 && !selMet1 && !selMet2 &&
		!selAr1 && !selVar1 && !selVar2
		&& !selIf1 && !selIf2 && !selLo1 && !selLo2 && !selRe1 && !selEx1 && !selFC1) {
		alert("You must check at least one item in the checklist to proceed");
		return false;
	}
	else {
		endTutorial();
	}
}

/**
 * Checks first item in the guided checklist.
 */

function checkFirstStrategyItem() {
	var firstItem = document.querySelector("[name=strAnsw]:checked");
	if(!firstItem){
		alert("You must check the item in the guided checklist to proceed");
		return false;
	}
	else {
		endTutorial();
	}

}


/*
Check correctness of the answers in intro.hmtl and show the user the correct one if the one inserted was wrong,
 before proceeding to the next page.
 */

function checkIntroAnswers() {

	var intro1_1 = document.querySelector("[id=intro1_1]:checked");
	var intro2_2 = document.querySelector("[id=intro2_2]:checked");

	if(intro1_1!=null && intro2_2!=null) {
		return true;
	}
	else if(intro1_1==null && intro2_2!=null) {
		alert("Please note: \n\n Unlike in real life, we are not interested in maintainability or design issues, but only in correctness defects (\"bugs\").")
		return true;
	}
	else if(intro1_1!=null && intro2_2==null) {
		alert("Please note: \n\n A defect can belong to a single part of the code, but it can also be due to the incorrect interaction of several parts of the code.")
		return true;
	}
	else {
		alert("Please note: \n\n Unlike in real life, we are not interested in maintainability or design issues, but only in correctness defects (\"bugs\") \n\n " +
			"A defect can belong to a single part of the code, but it can also be due to the incorrect interaction of several parts of the code.");
		return true;
	}
}

/*
Saves answer from review context questions
 */
function confirmAnswer() {
	if(numberOfChunks == smallTaskChunks) {
		const options = document.querySelectorAll('input[name="system0"]');
		let selectedValue;
		for (const option of options) {
			if (option.checked) {
				selectedValue = option.value;
				break;
			}
		}
		addToLog("Small question: "+selectedValue);
		if(selectedValue !== "VFSBrowser") {
			alert("Access to the file system is encapsulated by a \"virtual file system\" (abbreviated \"VFS\"). A UI for browsing this file system is implemented in the class VFSBrowser");
		}
		document.getElementById("questionContext0").style.display = "none";
	}
	else if(numberOfChunks == largeTaskAChunks) {
		const options = document.querySelectorAll('input[name="system1"]');
		let selectedValue;
		for (const option of options) {
			if (option.checked) {
				selectedValue = option.value;
				break;
			}
		}
		addToLog("Task question: "+selectedValue);
		if(selectedValue !== "Task") {
			alert("To keep the UI reactive, certain tasks are performed outside the UI's event processing thread. There is a new way " +
				"of doing this, using subclasses of the abstract class Task.");
		}
		document.getElementById("questionContext1").style.display = "none";
	}
	else if(numberOfChunks == largeTaskBChunks) {
		const options = document.querySelectorAll('input[name="system2"]');
		let selectedValue;
		for (const option of options) {
			if (option.checked) {
				selectedValue = option.value;
				break;
			}
		}
		addToLog("Hypersearch question: "+selectedValue);
		if(selectedValue !== "openDialog") {
			alert("A special feature of the editor is the so called \"hyper search\" that opens a separate dialog with all matches, possibly in different files.");
		}
		document.getElementById("questionContext2").style.display = "none";
	}
	document.getElementById("questionButton").style.display = "none";
}