package de.unihannover.orderingExperimentUi;

import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Random;
import java.util.Set;

/**
 * User session of the experiment.
 */

public class Experiment {

    private static Random PRIVATE_ID_RND;

    private static final Set<String> VALID_VARIABLE_NAMES = new HashSet<>(Arrays.asList(
                    "intro1",
                    "intro2",
                    "system1",
                    "system2",
                    "clientSideIpHash",
                    "role",
                    "profDevExp",
                    "javaExp",
                    "CrExp",
                    "reviewPract",
                    "programPract",
                    "workStartHours",
                    "fitness",
                    "methodAdded",
                    "idStatement",
                    "changeDescription",
                    "loadInfo",
                    "deletionTask",
                    "renaming",
                    "charChange",
                    "nextMatchImpl",
                    "matcherCreation",
                    "comparisonUnderstanding",
                    "explanationComparisonUnderstanding",
                    "comparisonComplicated",
                    "explanationComparisonComplicated",
                    "jedit",
                    "furtherRemarks",
                    "reviewExp",
                    "MLDifficult_0",
                    "MLDifficult_1",
                    "MLDifficult_2",
                    "MLComplicated_0",
                    "MLComplicated_1",
                    "MLComplicated_2",
                    "MLChallenge_0",
                    "MLChallenge_1",
                    "MLChallenge_2",
                    "MLUn_0",
                    "MLUn_1",
                    "MLUn_2",
                    "MLEasy_0",
                    "MLEasy_1",
                    "MLEasy_2",
                    "MLSol_0",
                    "MLSol_1",
                    "MLSol_2",
                    "MELittle_0",
                    "MELittle_1",
                    "MELittle_2",
                    "MEHard_0",
                    "MEHard_1",
                    "MEHard_2",
                    "MENotHard_0",
                    "MENotHard_1",
                    "MENotHard_2",
                    "MEEff_0",
                    "MEEff_1",
                    "MEEff_2",
                    "MEFoc_0",
                    "MEFoc_1",
                    "MEFoc_2",
                    "MEBes_0",
                    "MEBes_1",
                    "MEBes_2",
                    "Skill_0",
                    "Skill_1",
                    "Skill_2",
                    "Flow0_0",
                    "Flow1_0",
                    "Flow2_0",
                    "Flow3_0",
                    "Flow4_0",
                    "Flow5_0",
                    "Flow6_0",
                    "Flow7_0",
                    "Flow8_0",
                    "Flow0_1",
                    "Flow1_1",
                    "Flow2_1",
                    "Flow3_1",
                    "Flow4_1",
                    "Flow5_1",
                    "Flow6_1",
                    "Flow7_1",
                    "Flow8_1",
                    "Flow0_2",
                    "Flow1_2",
                    "Flow2_2",
                    "Flow3_2",
                    "Flow4_2",
                    "Flow5_2",
                    "Flow6_2",
                    "Flow7_2",
                    "Flow8_2",
                    "checkUnder",
                    "checkSys",
                    "checkHelp",
                    "checkDef",
                    "checkFreq",
                    "checkCom",
                    "checkEasy",
                    "checkSup",
                    "checkInt",
                    "checkInc",
                    "checkLea",
                    "checkCumb",
                    "checkCon",
                    "checkMor",
                    "age",
                    "gender",
                    "open_gender",
                    "english",
                    "education",
                    "hours_worked",
                    "stress"));

    private final long id;
    private final int privateKey;
    private final Map<String, String> variables = new LinkedHashMap<>();

    private String state;
    private Date lastStateChange;
    private Treatments.TreatmentCombination treatmentCombination;
    private int testReviewScore;

    private GuidanceApproach guidanceApproach;


    public Experiment(long id) {
        this.id = id;
        this.privateKey = getNextPrivateId(id);
        this.setState("fresh");
        this.guidanceApproach = GuidanceApproach.randomGuidanceApproach();
    }

    public long getId() {
        return this.id;
    }

    public String getIdAlpha() {
        final long hash = ((this.id & 0xFF) << 8) + (((this.id >> 8) + (this.id >> 16) + (this.id >> 24)) & 0xFF);
        return Long.toString(hash, 36);
    }

    public String getPrivateKey() {
        return Integer.toString(this.privateKey, 36);
    }

    private static synchronized int getNextPrivateId(long seed) {
        if (PRIVATE_ID_RND == null) {
            PRIVATE_ID_RND = new Random(seed);
        }
        return PRIVATE_ID_RND.nextInt(500) + 100;
    }

    public void start() {
        DataLog.log(this.id, "experiment started");
        this.setState("started");
    }

    public void setState(String string) {
        DataLog.log(this.id, "change state;" + string);
        this.state = string;
        this.lastStateChange = new Date();
    }

    private void assignTreatments(Treatments ts) {
        this.treatmentCombination = ts.getNextBestTreatment(this.getTestReviewScore());
        DataLog.log(this.id, "assigned treatment combination " + this.treatmentCombination);
    }

    private int getTestReviewScore() {
        return this.testReviewScore;
    }

    public void setTestReviewScore(int score) {
        DataLog.log(this.id, "setTestReviewScore;" + score);
        this.testReviewScore = score;
    }

    public boolean isVariableName(String key) {
        return VALID_VARIABLE_NAMES.contains(key);
    }

    public void setVariable(String key, String[] value) {
        final String valStr = value.length == 1 ? value[0] : Arrays.toString(value);
        DataLog.log(this.id, "set variable;" + key + ";" + valStr);
    }

    public synchronized Treatments.TreatmentCombination getAssignedTreatment(Treatments treatments) {
        if (this.treatmentCombination == null) {
           this.assignTreatments(treatments);
        }
        return this.treatmentCombination;
    }

    public synchronized String getAssignedDiffDescription(int number) {
        final Diff d = number == 1 ? this.treatmentCombination.getD1() : this.treatmentCombination.getD2();
        switch (d) {
        case DIFF1:
            return "file system tasks";
        case DIFF2:
            return "search algorithm";
        }
        throw new AssertionError();
    }

    public String getState() {
        return this.state;
    }

    public Date getLastStateChange() {
        return this.lastStateChange;
    }

    public synchronized void cancel(Treatments treatments) {
        if (this.treatmentCombination != null) {
            treatments.remove(this.treatmentCombination, this.getTestReviewScore());
        }
        this.setState("cancelled");
    }

    public boolean isCancelled() {
        return this.state.equals("cancelled");
    }

    public GuidanceApproach getGuidanceApproach() {
        return this.guidanceApproach;
    }

}
